@@ -1,4 +1,4 @@
-//  $Id: attachment.cpp 921 2007-02-28 05:43:34Z hiker $
+//  $Id: attachment.cpp 1507 2008-02-17 12:58:12Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 Joerg Henrichs
@@ -28,6 +28,7 @@
 #include "world.hpp"
 #include "sound_manager.hpp"
 #include "stk_config.hpp"
+#include "user_config.hpp"
 
 Attachment::Attachment(Kart* _kart)
 {
@@ -36,7 +37,7 @@ Attachment::Attachment(Kart* _kart)
     m_kart           = _kart;
     m_holder         = new ssgSelector();
     m_previous_owner = NULL;
-    m_kart->getModel()->addKid(m_holder);
+    m_kart->getModelTransform()->addKid(m_holder);
 
     for(int i=ATTACH_PARACHUTE; i<=ATTACH_TINYTUX; i++)
     {
@@ -64,17 +65,45 @@ void Attachment::set(attachmentType _type, float time, Kart *current_kart)
 // -----------------------------------------------------------------------------
 void Attachment::hitGreenHerring()
 {
-    switch (rand()%3)
+    if(user_config->m_profile) return;
+    int random_attachment;
+    float leftover_time   = 0.0f;
+    switch(getType())   // If there already is an attachment, make it worse :)
     {
-    case 0: set( ATTACH_PARACHUTE, stk_config->m_parachute_time ) ;
+    case ATTACH_BOMB:  projectile_manager->newExplosion(m_kart->getCoord());
+                       // Best solution would probably be to trigger the
+                       // explosion, and then to attach a new, random
+                       // attachment. Unfortunately, handleExplosion() is not
+                       // really severe enough, and forceRescue() attaches
+                       // tinytux, so that the new attachment is immediately lost.
+                       //m_kart->handleExplosion(m_kart->getCoord()->xyz, true);
+                       m_kart->forceRescue();
+                       clear();
+                       random_attachment = rand()%3;
+                       break;
+    case ATTACH_ANVIL :// if the kart already has an anvil, attach a new anvil, 
+                       // and increase the overall time 
+                       random_attachment = 2;
+                       leftover_time     = m_time_left;
+                       break;
+    case ATTACH_PARACHUTE:
+                       random_attachment = 2;  // anvil
+                       leftover_time     = m_time_left;
+                       break;
+    default:           random_attachment = rand()%3;
+    }   // switch
+    
+    switch (random_attachment)
+    {
+    case 0: set( ATTACH_PARACHUTE, stk_config->m_parachute_time+leftover_time);
         // if ( m_kart == m_kart[0] )
         //   sound -> playSfx ( SOUND_SHOOMF ) ;
         break ;
-    case 1: set( ATTACH_BOMB, stk_config->m_bomb_time ) ;
+    case 1: set( ATTACH_BOMB, stk_config->m_bomb_time+leftover_time);
         // if ( m_kart == m_kart[0] )
         //   sound -> playSfx ( SOUND_SHOOMF ) ;
         break ;
-    case 2: set( ATTACH_ANVIL, stk_config->m_anvil_time ) ;
+    case 2: set( ATTACH_ANVIL, stk_config->m_anvil_time+leftover_time);
         // if ( m_kart == m_kart[0] )
         //   sound -> playSfx ( SOUND_SHOOMF ) ;
         // Reduce speed once (see description above), all other changes are
@@ -95,7 +124,7 @@ void Attachment::moveBombFromTo(Kart *from, Kart *to)
 }   // moveBombFromTo
 
 //-----------------------------------------------------------------------------
-void Attachment::update(float dt, sgCoord *velocity)
+void Attachment::update(float dt)
 {
     if(m_type==ATTACH_NOTHING) return;
     m_time_left -=dt;
@@ -114,44 +143,19 @@ void Attachment::update(float dt, sgCoord *velocity)
                            break;
     case ATTACH_TINYTUX:   if(m_time_left<=0.0) m_kart->endRescue();
                            break;
-#ifdef USE_MAGNET
-    case ATTACH_MAGNET:    break;
-    case ATTACH_MAGNET_BZZT: float cdist; int closest;
-        m_kart->getClosestKart(&cdist, &closest);
-        // if no closest kart, set type to
-        // non-active magnet
-        if(closest==-1)
-        {
-            if ( m_type == ATTACH_MAGNET_BZZT )
-                set( ATTACH_MAGNET, m_time_left ) ;
-            return;
-        }
-        // Otherwise: set type to active magnet.
-        if(m_type==ATTACH_MAGNET)
-        {
-            if(m_kart->isPlayerKart() || closest==0)
-            {
-                sound_manager->playSfx(SOUND_BZZT);
-            }
-            set(ATTACH_MAGNET_BZZT,
-                    m_time_left<4.0?4.0:m_time_left);
-        }
-        m_kart->handleMagnet(cdist, closest);
-        break;
-#endif
-
     }   // switch
 
     // Detach attachment if its time is up.
     if ( m_time_left <= 0.0f)
     {
-        clear();
         if(m_type==ATTACH_ANVIL) 
         {
             // Resets the weight, and multiplies the velocity by 1.0, 
             // i.e. no change of velocity.
+            m_kart->getAttachment()->clear();
             m_kart->adjustSpeedWeight(1.0f);
         }
+        clear();
     }   // if m_time_left<0
 }   // update
 //-----------------------------------------------------------------------------
