@@ -1,4 +1,4 @@
-//  $Id: attachment.hpp 921 2007-02-28 05:43:34Z hiker $
+//  $Id: attachment.hpp 1379 2008-01-09 12:09:55Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 Joerg Henrichs
@@ -22,15 +22,11 @@
 
 #include "stk_config.hpp"
 class Kart;
-class ssgEntity;
 
 // Some loop in Attachment.cpp depend on PARACHUTE being the first element,
 // and TINYTUX being the last one. So if new elemts are added, make sure
 // to add them in between those values.
 enum attachmentType { ATTACH_PARACHUTE,
-#ifdef USE_MAGNET
-                      ATTACH_MAGNET, ATTACH_MAGNET_BZZT,
-#endif
                       ATTACH_BOMB,
                       ATTACH_ANVIL, ATTACH_TINYTUX,
                       ATTACH_MAX, ATTACH_NOTHING};
@@ -58,6 +54,7 @@ public:
                                     }
     attachmentType getType () const { return m_type;           }
     float getTimeLeft      () const { return m_time_left;      }
+    void setTimeLeft       (float t){ m_time_left = t;         }
     Kart* getPreviousOwner () const { return m_previous_owner; }
     float WeightAdjust     () const {
                                       return m_type==ATTACH_ANVIL
@@ -74,7 +71,7 @@ public:
                                           ?stk_config->m_anvil_speed_factor:1.0f;
                                     }
     void  hitGreenHerring();
-    void  update (float dt, sgCoord *velocity);
+    void  update (float dt);
     void  moveBombFromTo(Kart *from, Kart *to);
 };
 
