@@ -1,4 +1,4 @@
-//  $Id: camera.cpp 1153 2007-06-14 22:54:22Z hiker $
+//  $Id: camera.cpp 1477 2008-02-13 03:42:18Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004-2005 Steve Baker <sjbaker1@airmail.net>
@@ -25,9 +25,9 @@
 #include "track.hpp"
 #include "camera.hpp"
 #include "user_config.hpp"
+#include "constants.hpp"
 
-void
-Camera::setScreenPosition ( int numPlayers, int pos )
+void Camera::setScreenPosition ( int numPlayers, int pos )
 {
     assert(pos >= 0 && pos <= 3);
 
@@ -47,7 +47,21 @@ Camera::setScreenPosition ( int numPlayers, int pos )
             break;
         }
     }
-    else if (numPlayers == 3 || numPlayers == 4)
+    else if (numPlayers == 3)
+    {
+        m_context -> setFOV ( 50.0f, 0.0f );
+        switch ( pos )
+        {
+        case 0 : m_x = 0.0f; m_y = 0.5f; m_w = 0.5f; m_h = 0.5f;
+            break;
+        case 1 : m_x = 0.5f; m_y = 0.5f; m_w = 0.5f; m_h = 0.5f;
+            break;
+        case 2 : m_x = 0.0f; m_y = 0.0f; m_w = 1.0f; m_h = 0.5f;
+            m_context -> setFOV ( 85.0f, 85.0f*3.0f/8.0f ) ;
+            break;
+        }
+    }
+    else if (numPlayers == 4)
     {
         m_context -> setFOV ( 50.0f, 0.0f );
         switch ( pos )
@@ -63,10 +77,11 @@ Camera::setScreenPosition ( int numPlayers, int pos )
         }
     }
     m_LastPitch = 0.0f;
-}
+}  // setScreenPosition
 
 //-----------------------------------------------------------------------------
 Camera::Camera ( int numPlayers, int which_ )
+  : m_reverse(false)
 {
     m_which_kart = which_ ;   // Just for now
     m_mode = CM_NORMAL;
@@ -81,14 +96,28 @@ Camera::Camera ( int numPlayers, int which_ )
 
     setScreenPosition ( numPlayers, m_which_kart ) ;
     m_last_steer_offset = 0;
-}
+}   // Camera
 
 //-----------------------------------------------------------------------------
-void
-Camera::setMode(Mode mode_)
+void Camera::setMode(Mode mode_)
 {
     m_mode = mode_;
-}
+}   // setMode
+
+//-----------------------------------------------------------------------------
+void Camera::setReverseHeading(bool b)
+{
+  m_reverse = b;
+}   // setReverseHeading
+
+//-----------------------------------------------------------------------------
+/** Reset is called when a new race starts. Make sure that the camera
+    is aligned neutral, and not like in the previous race
+*/
+void Camera::reset()
+{
+    m_LastPitch = 0.0f;
+}   // reset
 
 //-----------------------------------------------------------------------------
 void Camera::update (float dt)
@@ -97,19 +126,26 @@ void Camera::update (float dt)
     if ( m_which_kart >= int(world->getNumKarts()) || m_which_kart < 0 ) m_which_kart = 0 ;
 
     sgCoord kartcoord;
+    const Kart *kart=world->getPlayerKart(m_which_kart);
     sgCopyCoord(&kartcoord, world->getPlayerKart(m_which_kart)->getCoord());
 
+    // Use the terrain pitch to avoid the camera following a wheelie the kart is doing
+    kartcoord.hpr[1]=RAD_TO_DEGREE( kart->getTerrainPitch(DEGREE_TO_RAD(kartcoord.hpr[0])) );
     kartcoord.hpr[2] = 0;
-
-    // If the car angle is 'significantly' different from the camera angle,
-    // start adjusting the camera. This helps with steep declines, where
-    // otherwise the track is not visible anymore.
-    if(fabsf(kartcoord.hpr[1]-m_LastPitch)>1.0f) {
-        kartcoord.hpr[1] = m_LastPitch + (kartcoord.hpr[1]-m_LastPitch)*2.0f*dt;
-    }
-    else
+    // Only adjust the pitch if it's not the first frame (which is indicated by having
+    // dt=0). Otherwise the camera will change pitch during ready-set-go.
+    if(dt>0)
     {
-        kartcoord.hpr[1]=m_LastPitch;
+        // If the terrain pitch is 'significantly' different from the camera angle,
+        // start adjusting the camera. This helps with steep declines, where
+        // otherwise the track is not visible anymore.
+        if(fabsf(kartcoord.hpr[1]-m_LastPitch)>1.0f) {
+            kartcoord.hpr[1] = m_LastPitch + (kartcoord.hpr[1]-m_LastPitch)*2.0f*dt;
+        }
+        else
+        {
+            kartcoord.hpr[1]=m_LastPitch;
+        }
     }
     m_LastPitch = kartcoord.hpr[1];
 
@@ -132,7 +168,17 @@ void Camera::update (float dt)
     else
         sgMakeTransMat4(cam_pos, 0.f, -3.5f, 1.5f);
 
-    if (m_mode == CM_NO_FAKE_DRIFT)
+    if (m_reverse)
+    {
+      // If player is looking back all other camera options are ignored.
+      sgMat4 cam_lb;
+      sgMakeTransMat4(cam_pos, 0.f, -2.5f, 0.75f);
+
+      // Applies 'look back' rotation.
+      sgMakeRotMat4(cam_lb, 180, 0, 0);
+      sgMultMat4(relative, cam_pos, cam_lb);
+    }
+    else if (m_mode == CM_NO_FAKE_DRIFT)
     {
         const float STEER_OFFSET = world->getPlayerKart(m_which_kart)->getSteerAngle()*-10.0f;
 
@@ -145,12 +191,14 @@ void Camera::update (float dt)
     }
     else
     {
-        sgMat4 cam_rot;
-        if (m_mode == CM_CLOSEUP)
-            sgMakeRotMat4(cam_rot, 0, -15, 0);
-        else
-            sgMakeRotMat4(cam_rot, 0, -5, 0);
-        sgMultMat4(relative, cam_pos, cam_rot);
+      sgMat4 cam_rot;
+
+      if (m_mode == CM_CLOSEUP)
+        sgMakeRotMat4(cam_rot, 0, -15, 0);
+      else
+        sgMakeRotMat4(cam_rot, 0, -5, 0);
+
+      sgMultMat4(relative, cam_pos, cam_rot);
     }
 
     sgMat4 result;
@@ -160,7 +208,7 @@ void Camera::update (float dt)
     sgSetCoord(&cam, result);
 
     m_context -> setCamera (&cam) ;
-}
+}   // update
 
 //-----------------------------------------------------------------------------
 void Camera::apply ()
@@ -174,5 +222,5 @@ void Camera::apply ()
                  (int)((float)height * m_h) ) ;
 
     m_context -> makeCurrent () ;
-}
+}   // apply
 
