@@ -1,4 +1,4 @@
-//  $Id: collectable.cpp 943 2007-03-28 02:20:44Z hiker $
+//  $Id: collectable.cpp 1507 2008-02-17 12:58:12Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 Joerg Henrichs
@@ -28,21 +28,21 @@
 //-----------------------------------------------------------------------------
 Collectable::Collectable(Kart* kart_)
 {
-    owner  = kart_;
-    type   = COLLECT_NOTHING;
-    number = 0;
+    m_owner  = kart_;
+    m_type   = COLLECT_NOTHING;
+    m_number = 0;
 }   // Collectable
 
 //-----------------------------------------------------------------------------
-void Collectable::set(collectableType _type, int n)
+void Collectable::set(CollectableType type, int n)
 {
-    if (type==_type)
+    if (m_type==type)
     {
-        number+=n;
+        m_number+=n;
         return;
     }
-    type=_type;
-    number=n;
+    m_type=type;
+    m_number=n;
 }  // set
 
 //-----------------------------------------------------------------------------
@@ -50,50 +50,32 @@ Material *Collectable::getIcon()
 {
     // Check if it's one of the types which have a separate
     // data file which includes the icon:
-    return collectable_manager->getIcon(type);
+    return collectable_manager->getIcon(m_type);
 }
 
 //-----------------------------------------------------------------------------
 void Collectable::use()
 {
-#ifdef USE_MAGNET
-    if(user_config->disableMagnet)
+    m_number--;
+    switch (m_type)
     {
-        attachmentType at=owner->getAttachment();
-        if(at==ATTACH_MAGNET)
-        {
-            owner->setAttachmentType(ATTACH_MAGNET_BZZT);
-        }
-        else if(at==ATTACH_MAGNET_BZZT)
-        {
-            owner->setAttachmentType(ATTACH_MAGNET     );
-        }   // if MAGNET_BZZT
-    }  // user_config->disableMagnet
-#endif
-    number--;
-    switch (type)
-    {
-#ifdef USE_MAGNET
-    case COLLECT_MAGNET:   owner->attach(ATTACH_MAGNET_BZZT, stk_config->m_magnet_time);
+    case COLLECT_ZIPPER:   m_owner->handleZipper();
         break ;
-#endif
-    case COLLECT_ZIPPER:   owner->handleZipper();
-        break ;
-    case COLLECT_HOMING_MISSILE:
+    case COLLECT_HOMING:
     case COLLECT_SPARK:
     case COLLECT_MISSILE:
-        if(owner->isPlayerKart())
+        if(m_owner->isPlayerKart())
             sound_manager->playSfx(SOUND_SHOT);
 
-        projectile_manager->newProjectile(owner, type);
+        projectile_manager->newProjectile(m_owner, m_type);
         break ;
 
     case COLLECT_ANVIL:
         //Attach an anvil(twice as good as the one given
         //by the bananas) to the kart in the 1st position.
-        for(int i = 0 ; i < world->getNumKarts(); ++i)
+        for(unsigned int i = 0 ; i < world->getNumKarts(); ++i)
         {
-            if(world->getKart(i) == owner) continue;
+            if(world->getKart(i) == m_owner) continue;
             if(world->getKart(i)->getPosition() == 1)
             {
                 world->getKart(i)->
@@ -115,10 +97,10 @@ void Collectable::use()
             //Attach a parachutte(that last as twice as the
             //one from the bananas) to all the karts that
             //are in front of this one.
-            for(int i = 0 ; i < world->getNumKarts(); ++i)
+            for(unsigned int i = 0 ; i < world->getNumKarts(); ++i)
             {
-                if(world->getKart(i) == owner) continue;
-                if(owner->getPosition() > world->
+                if(world->getKart(i) == m_owner) continue;
+                if(m_owner->getPosition() > world->
                    getKart(i)->getPosition())
                 {
                     world->getKart(i)->attach(
@@ -139,7 +121,7 @@ void Collectable::use()
     default :              break ;
     }
 
-    if ( number <= 0 )
+    if ( m_number <= 0 )
     {
         clear();
     }
@@ -152,29 +134,29 @@ void Collectable::hitRedHerring(int n)
     //depending on how bad the owner's position is. For the first
     //driver the posibility is none, for the last player is 15 %.
 
-    if(owner->getPosition() != 1 && type == COLLECT_NOTHING)
+    if(m_owner->getPosition() != 1 && m_type == COLLECT_NOTHING)
     {
         const int SPECIAL_PROB = (int)(15.0 / ((float)world->getNumKarts() /
-                                         (float)owner->getPosition()));
+                                         (float)m_owner->getPosition()));
         const int RAND_NUM = rand()%100;
         if(RAND_NUM <= SPECIAL_PROB)
         {
             //If the driver in the first position has finished, give the driver
             //the parachute.
-            for(int i=0; i < world->getNumKarts(); ++i)
+            for(unsigned int i=0; i < world->getNumKarts(); ++i)
             {
-                if(world->getKart(i) == owner) continue;
+                if(world->getKart(i) == m_owner) continue;
                 if(world->getKart(i)->getPosition() == 1 && world->getKart(i)->
                    raceIsFinished())
                 {
-                    type = COLLECT_PARACHUTE;
-                    number = 1;
+                    m_type = COLLECT_PARACHUTE;
+                    m_number = 1;
                     return;
                 }
             }
 
-            type = rand()%(2) == 0 ? COLLECT_ANVIL : COLLECT_PARACHUTE;
-            number = 1;
+            m_type = rand()%(2) == 0 ? COLLECT_ANVIL : COLLECT_PARACHUTE;
+            m_number = 1;
             return;
         }
     }
@@ -182,7 +164,7 @@ void Collectable::hitRedHerring(int n)
     //rand() is moduled by COLLECT_MAX - 1 - 2 because because we have to
     //exclude the anvil and the parachute, but later we have to add 1 to prevent
     //having a value of 0 since that isn't a valid collectable.
-    collectableType newC;
+    CollectableType newC;
     if(!user_config->m_profile)
     {
         // A zipper should not be used during time trial, since it
@@ -190,24 +172,25 @@ void Collectable::hitRedHerring(int n)
         // in time trail, but only the top 2 in any other mode
         int nIgnore = (world->m_race_setup.m_mode == RaceSetup::RM_TIME_TRIAL) 
                     ? 3 : 2;
-        newC = (collectableType)(rand()%(COLLECT_MAX - 1 - nIgnore) + 1);
+        newC = (CollectableType)(rand()%(COLLECT_MAX - 1 - nIgnore) + 1);
     }
     else
-    {
+      {   // for now: no collectables when profiling
+        return;
         // No random effects when profiling!
         static int simpleCounter=-1;
         simpleCounter++;
-        newC = (collectableType)(simpleCounter%(COLLECT_MAX - 1 - 2) + 1);
+        newC = (CollectableType)(simpleCounter%(COLLECT_MAX - 1 - 2) + 1);
     }
-    if(type==COLLECT_NOTHING)
+    if(m_type==COLLECT_NOTHING)
     {
-        type=newC;
-        number = n;
+        m_type=newC;
+        m_number = n;
     }
-    else if(newC==type)
+    else if(newC==m_type)
     {
-        number+=n;
-        if(number > MAX_COLLECTABLES) number = MAX_COLLECTABLES;
+        m_number+=n;
+        if(m_number > MAX_COLLECTABLES) m_number = MAX_COLLECTABLES;
     }
     // Ignore new collectable if it is different from the current one
 }   // hitRedHerring
