@@ -1,4 +1,4 @@
-//  $Id: cup_data.hpp 855 2006-11-17 01:50:37Z coz $
+//  $Id: cup_data.hpp 1322 2007-12-06 20:57:42Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004-2006 Ingo Ruhnke <grumbel@gmx.de>
@@ -21,6 +21,10 @@
 #ifndef HEADER_CUPDATA_H
 #define HEADER_CUPDATA_H
 
+#include <string>
+#include <vector>
+#include <cassert>
+
 #include "herring_manager.hpp"
 
 /** Simple class that hold the data relevant to a 'cup', aka. a number
@@ -31,13 +35,15 @@ class CupData
     std::string m_filename;     // Original filename, only for error handling needed
     std::string m_description;  // Description for this track
     std::string m_herring_style; // herring style which overwrites the track default
-
-public:
-
     /** The ident of the tracks in this cup in their right order, ident
         means the filename of the .track file without .track extension
         (ie. 'volcano') */
     std::vector<std::string> m_tracks;
+    /** The number of laps that each track should be raced, in the right
+     * order*/
+    std::vector<int> m_laps;
+
+public:
 
     /** Load the CupData from the given filename */
     CupData        (const std::string filename);
@@ -46,6 +52,9 @@ public:
     const std::string& getDescription ()  const { return m_description; }
     const std::string& getHerringStyle()  const { return m_herring_style;}
     const std::string& getFilename    ()  const { return m_filename;    }
+    const std::string& getTrack(size_t track_index) const { assert(track_index < m_tracks.size()); return m_tracks[track_index]; }
+    size_t             getTrackCount() const    { return m_tracks.size(); }
+    const int&         getLaps(size_t lap_index) const { assert(lap_index < m_tracks.size()); return m_laps[lap_index]; }
 }
 ;   // CupData
 
