@@ -1,4 +1,4 @@
-//  $Id: explosion.cpp 917 2007-02-26 04:47:07Z hiker $
+//  $Id: explosion.cpp 1327 2007-12-09 07:14:53Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>
@@ -19,19 +19,16 @@
 
 #include <plib/ssg.h>
 #include "explosion.hpp"
-#include "world.hpp"
 #include "projectile_manager.hpp"
 #include "sound_manager.hpp"
+#include "scene.hpp"
 
 
 Explosion::Explosion(sgCoord* coord) : ssgTransform()
 {
+    this->ref();
     ssgCutout *cut = new ssgCutout();
-    addKid(cut);
-    // Make sure that the transform object does not get deleted when it is
-    // removed from the scene graphs. This can happen when the explosion is
-    // moved to the deletedExplosion list in the projecile_manager.
-    ref();
+    addKid(cut);  // derefing the explosion will free the cutout
     m_seq   = projectile_manager->getExplosionModel();
     cut->addKid(m_seq);
     init(coord);
@@ -44,7 +41,7 @@ void Explosion::init(sgCoord* coord)
 
     setTransform(coord);
     m_step = -1;
-    world->addToScene(this);
+    scene->add(this);
 }
 
 //-----------------------------------------------------------------------------
@@ -53,8 +50,8 @@ void Explosion::update (float dt)
 
     if ( ++m_step >= m_seq->getNumKids() )
     {
+        scene->remove((ssgTransform*)this);
         projectile_manager->FinishedExplosion();
-        world->removeFromScene((ssgTransform*)this);
         return ;
     }
 
