@@ -1,4 +1,4 @@
-//  $Id: base_gui.hpp 1006 2007-04-17 07:36:44Z hiker $
+//  $Id: base_gui.hpp 1514 2008-02-18 04:27:49Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -20,21 +20,39 @@
 #ifndef HEADER_BASEGUI_H
 #define HEADER_BASEGUI_H
 
-#include "player.hpp"
+#include <SDL/SDL.h>
+#include "input.hpp"
 
 class BaseGUI
 {
+	void animateWidget(const int, const int);
+		
 public:
-    BaseGUI() {}
+    BaseGUI() : m_locked(false) {}
     virtual ~BaseGUI() {}
 
     virtual void update(float dt);
     virtual void select() = 0;
+	
+	virtual void handle(GameAction, int);
+	
+	virtual void inputKeyboard(SDLKey, int);
+	
+	virtual void countdown();
 
-    virtual void input(InputType type, int id0, int id1, int id2, int value);
-    virtual void inputKeyboard(int key, int pressed);
-    void  TimeToString(const float time, char *s);
+    void inputPointer(int x, int y);
+
+    //At times, we want to make sure that we won't be getting any kind of
+    //input to the gui, for example, during transitions from one screen
+    //to another. At those times, it's best to lock the input and unlock it
+    //afterwards.
+    void lockInput() { m_locked = true; }
+    void unlockInput() { m_locked = false; }
+
+    void  TimeToString(const double time, char *s);
 protected:
+
+    bool m_locked;
     int m_menu_id;
 };
 
