@@ -1,4 +1,4 @@
-//  $Id: char_sel.hpp 906 2007-02-04 01:37:49Z coz $
+//  $Id: char_sel.hpp 1413 2008-01-23 19:47:32Z scifly $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -31,10 +31,11 @@ private:
     ssgContext* m_context;
     ssgTransform* m_kart;
     int m_current_kart;
-    int m_kart_name_label;
     float m_clock;
 
     int m_player_index;
+    
+    bool kartAvailable(int kart);
 
 public:
     CharSel(int which_player);
