@@ -1,4 +1,4 @@
-//  $Id: config_display.hpp 854 2006-11-06 14:10:45Z coz $
+//  $Id: config_display.hpp 1500 2008-02-16 03:53:39Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -20,20 +20,42 @@
 #ifndef HEADER_CONFIGDISPLAY_H
 #define HEADER_CONFIGDISPLAY_H
 
+#include <vector>
+
 #include "base_gui.hpp"
+#include "translation.hpp"
+#include "display_res_confirm.hpp"
 
 class ConfigDisplay: public BaseGUI
 {
-    void CreateMenu();
 public:
     ConfigDisplay();
     ~ConfigDisplay();
 
-    void update(float dt);
     void select();
 
 private:
-    int m_fullscreen_menu_id;
+    std::vector< std::pair<int,int> > m_sizes;
+    int m_curr_res;
+
+    int m_curr_width;
+    int m_curr_height;
+
+    // changeResolution() reverse param is set true when changing to a previous resolution,
+    // thought it is disabled for now.
+    void changeResolution(int width, int height/*, bool reverse*/);
+    void getScreenModes();
+    void changeApplyButton();
+
+#if 0
+    // isBlacklisted() returns the index of the resolution in the blacklist 
+    // or -1 if not in the blacklist
+    int isBlacklisted(); 
+#endif
+    bool isBlacklisted(int width, int height);
+    void showBlacklistButtons();
+
+    void loadDefaultModes();
 };
 
 #endif
