@@ -1,4 +1,4 @@
-//  $Id: credits_menu.hpp 854 2006-11-06 14:10:45Z coz $
+//  $Id: credits_menu.hpp 1347 2007-12-12 21:30:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 Joerg Henrichs
@@ -20,24 +20,15 @@
 #ifndef HEADER_CREDITSMENU_H
 #define HEADER_CREDITSMENU_H
 
-#include <string>
-#include <plib/fnt.h>
-#include "scrolled_text.hpp"
 #include "base_gui.hpp"
-#include "player.hpp"
 
-
-class CreditsMenu: public ScrolledText
+class CreditsMenu: public BaseGUI
 {
-protected:
-    int        m_xLeft, m_xRight, m_yBottom, m_yTop;
-    float      m_yPos, m_ySpeed;
-    int        m_numberOfLines;
-    int        m_fontSize;
-    StringList m_string_list;
 public:
     CreditsMenu();
     ~CreditsMenu();
+
+    void select ();
 };
 
 #endif
