@@ -1,4 +1,4 @@
-//  $Id: difficulty.cpp 915 2007-02-20 23:50:35Z hiker $
+//  $Id: difficulty.cpp 1540 2008-02-20 00:33:44Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -17,61 +17,78 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-#include "difficulty.hpp"
 #include "race_manager.hpp"
-#include "widget_set.hpp"
+#include "difficulty.hpp"
+#include "widget_manager.hpp"
 #include "menu_manager.hpp"
 #include "translation.hpp"
 
-enum WidgetTokens {
+enum WidgetTokens
+{
+    WTOK_TITLE,
+
     WTOK_HARD,
-    WTOK_MEDIUM,
+    //    WTOK_MEDIUM,
     WTOK_EASY,
-    WTOK_BACK
+
+    WTOK_QUIT
 };
 
 Difficulty::Difficulty()
 {
-    m_menu_id = widgetSet -> vstack(0);
+    const bool SHOW_RECT = true;
+    const bool SHOW_TEXT = true;
+    widget_manager->setInitialRectState(SHOW_RECT, WGT_AREA_ALL, WGT_TRANS_BLACK);
+    widget_manager->setInitialTextState(SHOW_TEXT, "", WGT_FNT_MED,
+        WGT_FONT_GUI );
 
-    widgetSet -> label(m_menu_id, _("Choose your skill level"), GUI_LRG, GUI_ALL, 0, 0);
+    widget_manager->insertColumn();
+    widget_manager->addWgt(WTOK_TITLE, 60, 7);
+    widget_manager->showWgtRect(WTOK_TITLE);
+    widget_manager->showWgtText(WTOK_TITLE);
+    widget_manager->setWgtText(WTOK_TITLE,
+        _("Choose your skill level"));
 
-    const int VA = widgetSet -> varray(m_menu_id);
-    widgetSet -> space(m_menu_id);
-    widgetSet -> space(m_menu_id);
-    widgetSet -> state(VA, _("Racer"),  GUI_MED, WTOK_HARD);
-    widgetSet -> state(VA, _("Driver"), GUI_MED, WTOK_MEDIUM);
-    widgetSet -> start(VA, _("Novice"), GUI_MED, WTOK_EASY);
-    widgetSet -> space(VA);
-    widgetSet -> state(VA, _("Press <ESC> to go back"), GUI_SML, WTOK_BACK);
+    widget_manager->setInitialActivationState(true);
+    widget_manager->addWgt(WTOK_HARD, 60, 7);
+    widget_manager->setWgtText(WTOK_HARD, _("Racer"));
 
-    widgetSet -> layout(m_menu_id, 0, 0);
+    //widget_manager->addWgt(WTOK_MEDIUM, 60, 7);
+    //widget_manager->setWgtText(WTOK_MEDIUM, _("Driver"));
+
+    widget_manager->addWgt(WTOK_EASY, 60, 7);
+    widget_manager->setWgtText(WTOK_EASY, _("Novice"));
+
+    widget_manager->addWgt(WTOK_QUIT, 60, 7);
+    widget_manager->setWgtText(WTOK_QUIT, _("Press <ESC> to go back"));
+
+    widget_manager->layout(WGT_AREA_ALL);
 }   // Difficulty
 
 //-----------------------------------------------------------------------------
 Difficulty::~Difficulty()
 {
-    widgetSet -> delete_widget(m_menu_id) ;
+    widget_manager->reset();
 }   // ~Difficulty
 
 //-----------------------------------------------------------------------------
 void Difficulty::select()
 {
-    switch ( widgetSet -> token (widgetSet -> click()) )
+    switch ( widget_manager->getSelectedWgt())
     {
     case WTOK_EASY:
         race_manager->setDifficulty(RD_EASY);
         menu_manager->pushMenu(MENUID_CHARSEL_P1);
         break;
-    case WTOK_MEDIUM:
-        race_manager->setDifficulty(RD_MEDIUM);
-        menu_manager->pushMenu(MENUID_CHARSEL_P1);
-        break;
+	//    case WTOK_MEDIUM:
+	//        race_manager->setDifficulty(RD_MEDIUM);
+	//        menu_manager->pushMenu(MENUID_CHARSEL_P1);
+	//        break;
     case WTOK_HARD:
         race_manager->setDifficulty(RD_HARD);
         menu_manager->pushMenu(MENUID_CHARSEL_P1);
         break;
-    case WTOK_BACK:
+    case WTOK_QUIT:
         menu_manager->popMenu();
         break;
     default: break;
