@@ -1,4 +1,4 @@
-//  $Id: game_mode.cpp 915 2007-02-20 23:50:35Z hiker $
+//  $Id: game_mode.cpp 1442 2008-02-04 14:28:44Z scifly $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -18,48 +18,71 @@
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "game_mode.hpp"
-#include "widget_set.hpp"
+#include "widget_manager.hpp"
 #include "race_manager.hpp"
 #include "menu_manager.hpp"
 #include "translation.hpp"
+#include "user_config.hpp"
 
-enum WidgetTokens {
+enum WidgetTokens
+{
+    WTOK_TITLE,
     WTOK_GP,
     WTOK_QUICKRACE,
     WTOK_TIMETRIAL,
+    WTOK_EMPTY,
     WTOK_BACK
 };
 
 GameMode::GameMode()
 {
-    m_menu_id = widgetSet -> vstack(0);
+    const bool SHOW_RECT = true;
+    const bool SHOW_TEXT = true;
+    widget_manager->setInitialRectState(SHOW_RECT, WGT_AREA_ALL, WGT_TRANS_BLACK);
+    widget_manager->setInitialTextState(SHOW_TEXT, "", WGT_FNT_MED,
+        WGT_FONT_GUI );
 
-    widgetSet -> label(m_menu_id, _("Choose a Race Mode"), GUI_LRG);
+    widget_manager->insertColumn();
+    widget_manager->addWgt(WTOK_TITLE, 50, 7);
+    widget_manager->setWgtText( WTOK_TITLE, _("Choose a Race Mode"));
+    widget_manager->setWgtTextSize( WTOK_TITLE, WGT_FNT_LRG );
 
-    const int VA = widgetSet -> varray(m_menu_id);
-    widgetSet -> space(m_menu_id);
-    widgetSet -> start(VA, _("Grand Prix"),  GUI_MED, WTOK_GP);
-    widgetSet -> state(VA, _("Quick Race"),  GUI_MED, WTOK_QUICKRACE);
+    widget_manager->setInitialActivationState(true);
+    widget_manager->addWgt(WTOK_GP, 50, 7);
+    widget_manager->setWgtText( WTOK_GP, _("Grand Prix"));
 
-    if (race_manager->getNumPlayers() == 1)
-        widgetSet -> state(VA, _("Time Trial"),  GUI_MED, WTOK_TIMETRIAL);
+    widget_manager->addWgt(WTOK_QUICKRACE, 50, 7);
+    widget_manager->setWgtText( WTOK_QUICKRACE, _("Quick Race"));
 
-    widgetSet -> space(VA);
-    widgetSet -> state(VA,_("Press <ESC> to go back"), GUI_SML, WTOK_BACK);
-    widgetSet -> space(VA);
-    widgetSet -> layout(m_menu_id, 0, 0);
+    if( race_manager->getNumPlayers() == 1 )
+    {
+        widget_manager->addWgt(WTOK_TIMETRIAL, 50, 7);
+        widget_manager->setWgtText( WTOK_TIMETRIAL, _("Time Trial"));
+    }
+
+    widget_manager->addWgt(WTOK_EMPTY, 50, 7);
+    widget_manager->hideWgtRect( WTOK_EMPTY );
+    widget_manager->hideWgtText( WTOK_EMPTY );
+    widget_manager->deactivateWgt( WTOK_EMPTY );
+
+    widget_manager->addWgt(WTOK_BACK, 50, 7);
+    widget_manager->setWgtText( WTOK_BACK, _("Press <ESC> to go back"));
+    widget_manager->setWgtTextSize( WTOK_BACK, WGT_FNT_SML );
+
+    widget_manager->layout(WGT_AREA_ALL);
+    
 }
 
 //-----------------------------------------------------------------------------
 GameMode::~GameMode()
 {
-    widgetSet -> delete_widget(m_menu_id) ;
+    widget_manager->reset();
 }
 
 //-----------------------------------------------------------------------------
 void GameMode::select()
 {
-    switch ( widgetSet -> token (widgetSet -> click()) )
+    switch ( widget_manager->getSelectedWgt() )
     {
     case WTOK_GP:
         race_manager->setRaceMode(RaceSetup::RM_GRAND_PRIX);
