@@ -1,4 +1,4 @@
-// $Id: grand_prix_ending.cpp 1076 2007-05-14 12:37:19Z hiker $
+// $Id: grand_prix_ending.cpp 1458 2008-02-12 00:07:00Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 Eduardo Hernandez Munoz
@@ -20,26 +20,35 @@
 #include <sstream>
 #include <string>
 
+#include <SDL/SDL.h>
+
 #include "loader.hpp"
 #include "sound_manager.hpp"
 #include "grand_prix_ending.hpp"
 #include "kart_properties_manager.hpp"
-#include "preprocessor.hpp"
-#include "widget_set.hpp"
+#include "widget_manager.hpp"
 #include "race_manager.hpp"
-#include "start_screen.hpp"
-#include "empty_screen.hpp"
-#include "screen_manager.hpp"
+#include "game_manager.hpp"
 #include "user_config.hpp"
 #include "menu_manager.hpp"
 #include "kart_properties.hpp"
 #include "translation.hpp"
 #include "kart.hpp"
 #include "world.hpp"
-#include "screen_manager.hpp"
+#include "scene.hpp"
 #if defined(WIN32) && !defined(__CYGWIN__)
 #  define snprintf _snprintf
 #endif
+
+enum WidgetTokens
+{
+    WTOK_TITLE,
+    WTOK_QUIT,
+
+    WTOK_FIRSTKART
+};
+
+
 GrandPrixEnd::GrandPrixEnd()
         : m_kart(0)
 {
@@ -49,38 +58,17 @@ GrandPrixEnd::GrandPrixEnd()
     m_context = new ssgContext;
     oldContext->makeCurrent();
 
-    m_menu_id = widgetSet -> vstack(0);
-
-    int highest = 0;
-    //FIXME: We go from the back to the front because the players are in the
-    //back and in case of a tie they will win against the AI, *but* if it's
-    //player vs. player, the player who goes first would win.
-    for(int i = race_manager->getNumKarts() - 1; i > 0; --i)
-        if(race_manager->getKartScore(i) > race_manager->getKartScore(highest))
-            highest = i;
-
-    const std::string KART_NAME = race_manager->getKartName(highest);
-    const KartProperties* WINNING_KART = kart_properties_manager->getKart(KART_NAME);
-
-    static char output[MAX_MESSAGE_LENGTH];
-    snprintf(output, sizeof(output),
-             _("The winner is %s!"),WINNING_KART->getName().c_str());
-    widgetSet -> label(m_menu_id, output, GUI_LRG, GUI_ALL, 0, 0);
-
-
     const unsigned int MAX_STR_LEN = 60;
     const unsigned int NUM_KARTS = world->getNumKarts();
 
-    const int VA = widgetSet->varray(m_menu_id);
-
-    Kart *kart;
     int *scores   = new int[NUM_KARTS];
     int *position = new int[NUM_KARTS];
+    double *race_time = new double[NUM_KARTS];
     for( unsigned int i = 0; i < NUM_KARTS; ++i )
     {
-        kart = world->getKart(i);
         position[i] = i;
         scores[i]   = race_manager->getKartScore(i);
+        race_time[i] = race_manager->getKartOverallTime(i);
     }
 
     //Bubblesort
@@ -90,59 +78,96 @@ GrandPrixEnd::GrandPrixEnd()
         sorted = true;
         for( unsigned int i = 0; i < NUM_KARTS - 1; ++i )
         {
-            if( scores[i] < scores[i+1] )
+            if( scores[i] < scores[i+1] || (scores[i] == scores[i+1] 
+              && race_time[i] > race_time[i+1]))
             {
                 int tmp_score[2];
+                double tmp_time;
 
                 tmp_score[0] = position[i];
                 tmp_score[1] = scores[i];
+                tmp_time = race_time[i];
 
                 position[i] = position[i+1];
                 scores[i] = scores[i+1];
+                race_time[i] = race_time[i+1];
 
                 position[i+1] = tmp_score[0];
                 scores[i+1] = tmp_score[1];
+                race_time[i+1] = tmp_time;
 
                 sorted = false;
             }
         }
     } while(!sorted);
+    
+    Kart *kart;
+    kart = world->getKart(position[0]);
+    
+    static char output[MAX_MESSAGE_LENGTH];
+    snprintf(output, sizeof(output),
+             _("The winner is %s!"),kart->getName().c_str());
+    widget_manager->addWgt( WTOK_TITLE, 60, 10);
+    widget_manager->showWgtRect(WTOK_TITLE);
+    widget_manager->showWgtText(WTOK_TITLE);
+    widget_manager->setWgtText(WTOK_TITLE, output);
+    widget_manager->setWgtTextSize(WTOK_TITLE, WGT_FNT_LRG);
+    widget_manager->breakLine();
 
     m_score = new char[MAX_STR_LEN*NUM_KARTS];
 
     for(unsigned int i=0; i < NUM_KARTS; ++i)
     {
+        char sTime[20];
+        TimeToString(race_time[i], sTime);
         kart = world->getKart(position[i]);
-        sprintf((char*)(m_score + MAX_STR_LEN * i), "%d. %s %d",
-                i + 1, kart->getName().c_str(), scores[i]);
-        widgetSet -> label(VA, (char*)(m_score + MAX_STR_LEN * i), GUI_MED,
-            GUI_ALL);
+        sprintf((char*)(m_score + MAX_STR_LEN * i), "%d. %s %d %s",
+                i + 1, kart->getName().c_str(), scores[i], sTime );
+
+        widget_manager->addWgt(WTOK_FIRSTKART + i, 40, 5);
+        widget_manager->showWgtRect(WTOK_FIRSTKART + i);
+        widget_manager->showWgtText(WTOK_FIRSTKART + i);
+        widget_manager->setWgtText(WTOK_FIRSTKART + i,
+            (char*)(m_score + MAX_STR_LEN * i));
+        widget_manager->setWgtTextSize(WTOK_FIRSTKART + i, WGT_FNT_SML);
+    widget_manager->breakLine();
     }
+    const std::string KART_NAME = race_manager->getKartName(position[0]);
+    const KartProperties* WINNING_KART = kart_properties_manager->getKart(KART_NAME);
     delete []scores;
     delete []position;
+    delete []race_time;
 
-    widgetSet -> space(m_menu_id);
-    widgetSet -> label(m_menu_id, _("Back to the main menu"), GUI_LRG, GUI_ALL, 0, 0);
+    widget_manager->addWgt(WTOK_QUIT, 50, 7);
+    widget_manager->activateWgt(WTOK_QUIT);
+    widget_manager->showWgtRect(WTOK_QUIT);
+    widget_manager->showWgtText(WTOK_QUIT);
+    widget_manager->setWgtText(WTOK_QUIT, _("Back to the main menu"));
 
-    widgetSet -> layout(m_menu_id, 0, 1);
+    widget_manager->layout(WGT_AREA_TOP);
 
     m_kart = new ssgTransform;
     m_kart->ref();
     ssgEntity* kartentity = WINNING_KART->getModel();
     m_kart->addKid(kartentity);
-    preProcessObj ( m_kart, 0 );
 
     sound_manager->playSfx(SOUND_WINNER);
 
     m_clock = 0;
 
-    screen_manager->setScreen(new EmptyScreen());
+    //FIXME: this is taken from RaceMode::exit_race,
+    //this should be organized better.
+    scene->clear();
+    delete world;
+    world = 0;
+    race_manager->m_active_race = false;
+
 }
 
 //-----------------------------------------------------------------------------
 GrandPrixEnd::~GrandPrixEnd()
 {
-    widgetSet -> delete_widget(m_menu_id);
+    widget_manager->reset();
     ssgDeRefDelete(m_kart);
 
     delete m_context;
@@ -187,14 +212,11 @@ void GrandPrixEnd::update(float dt)
 
     glDisable (GL_DEPTH_TEST);
     oldContext->makeCurrent();
-
     BaseGUI::update(dt);
 }
 
 //-----------------------------------------------------------------------------
 void GrandPrixEnd::select()
 {
-    startScreen = new StartScreen();
-    screen_manager->setScreen(startScreen);
     menu_manager->switchToMainMenu();
 }
