@@ -1,4 +1,4 @@
-//  $Id: grand_prix_ending.hpp 694 2006-08-29 07:42:36Z hiker $
+//  $Id: grand_prix_ending.hpp 1023 2007-04-29 22:23:46Z coz $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 Eduardo Hernandez Munoz
@@ -29,18 +29,19 @@ class ssgContext;
 class GrandPrixEnd: public BaseGUI
 {
 private:
-    ssgContext* context;
-    ssgTransform* kart;
-    int current_kart;
-    int kart_name_label;
-    float clock;
+    ssgContext* m_context;
+    ssgTransform* m_kart;
+    int m_current_kart;
+    int m_kart_name_label;
+    char* m_score;
+    float m_clock;
 
 public:
-	GrandPrixEnd();
-	~GrandPrixEnd();
+    GrandPrixEnd();
+    ~GrandPrixEnd();
 
-	void update(float dt);
-	void select();
+    void update(float dt);
+    void select();
 };
 
 #endif
