@@ -21,22 +21,42 @@
 #include "loader.hpp"
 #include "string_utils.hpp"
 #include "grand_prix_select.hpp"
-#include "widget_set.hpp"
+#include "widget_manager.hpp"
 #include "menu_manager.hpp"
 #include "race_manager.hpp"
 #include "user_config.hpp"
 #include "translation.hpp"
 
+enum WidgetTokens
+{
+    WTOK_TITLE,
+
+    //FIXME: finish the tokens
+
+    WTOK_EMPTY0,
+    WTOK_DESCRIPTION,
+    WTOK_EMPTY1,
+    WTOK_QUIT,
+
+    WTOK_FIRSTPRIX
+};
+
 GrandPrixSelect::GrandPrixSelect()
 {
-    m_menu_id = widgetSet -> varray(0);
+    const bool SHOW_RECT = true;
+    const bool SHOW_TEXT = true;
+    widget_manager->setInitialRectState(SHOW_RECT, WGT_AREA_ALL, WGT_TRANS_BLACK);
+    widget_manager->setInitialTextState(SHOW_TEXT, "", WGT_FNT_MED,
+        WGT_FONT_GUI );
 
-    widgetSet -> label(m_menu_id, _("Choose a Grand Prix"), GUI_LRG, GUI_ALL, 0, 0);
-    widgetSet -> space(m_menu_id);
+    widget_manager->insertColumn();
+    widget_manager->addWgt(WTOK_TITLE, 40, 7);
+    widget_manager->setWgtText(WTOK_TITLE,  _("Choose a Grand Prix"));
 
     std::set<std::string> result;
     loader->listFiles(result, "data");
 
+    widget_manager->setInitialActivationState(true);
     // Findout which grand prixs are available and load them
     int nId = 0;
     for(std::set<std::string>::iterator i  = result.begin();
@@ -45,54 +65,50 @@ GrandPrixSelect::GrandPrixSelect()
             if (StringUtils::has_suffix(*i, ".cup"))
             {
                 std::string fullPath= "data/" + (std::string)*i;
-                CupData *cup = new CupData(fullPath.c_str());
+                CupData cup(fullPath.c_str());
                 m_all_cups.push_back(cup);
-                if(nId==0)
-                    widgetSet -> start(m_menu_id, cup->getName().c_str(), GUI_SML, nId, 0);
-                else
-                    widgetSet -> state(m_menu_id, cup->getName().c_str(), GUI_SML, nId, 0);
+                widget_manager->addWgt(WTOK_FIRSTPRIX + nId, 40, 7);
+                widget_manager->setWgtText(WTOK_FIRSTPRIX + nId, cup.getName());
                 nId++;
             }   // if
         }   // for i
-    widgetSet -> space(m_menu_id);
-    widgetSet -> state(m_menu_id,_("Press <ESC> to go back"), GUI_SML, -1);
-    widgetSet -> layout(m_menu_id, 0, 0);
-    m_rect = widgetSet->rect(10, 10, user_config->m_width-20, 34, GUI_ALL, 10);
+
+    widget_manager->setInitialActivationState(false);
+    widget_manager->addWgt(WTOK_EMPTY0, 60, 7);
+    widget_manager->hideWgtRect(WTOK_EMPTY0);
+    widget_manager->hideWgtText(WTOK_EMPTY0);
+
+    widget_manager->addWgt(WTOK_DESCRIPTION, 80, 7);
+    widget_manager->setWgtText(WTOK_DESCRIPTION, _("No Grand Prix selected"));
+    widget_manager->setWgtTextSize(WTOK_DESCRIPTION, WGT_FNT_SML);
+
+    widget_manager->addWgt(WTOK_EMPTY1, 60, 7);
+    widget_manager->hideWgtRect(WTOK_EMPTY1);
+    widget_manager->hideWgtText(WTOK_EMPTY1);
+
+    widget_manager->addWgt(WTOK_QUIT, 60, 7);
+    widget_manager->setWgtText(WTOK_QUIT, _("Press <ESC> to go back"));
+    widget_manager->setWgtTextSize(WTOK_QUIT, WGT_FNT_SML);
+    widget_manager->activateWgt(WTOK_QUIT);
+
+    widget_manager->layout(WGT_AREA_ALL);
 }   // GrandPrixSelect
 
 //-----------------------------------------------------------------------------
 GrandPrixSelect::~GrandPrixSelect()
 {
-    widgetSet -> delete_widget(m_menu_id) ;
-    glDeleteLists(m_rect, 1);
+    widget_manager->reset();
 }   // GrandPrixSelect
 
 //-----------------------------------------------------------------------------
 void GrandPrixSelect::update(float dt)
 {
     BaseGUI::update(dt);
-    const int CLICKED_TOKEN = widgetSet->token(widgetSet->click());
-    if(CLICKED_TOKEN == -1) return;
+    const int CLICKED_TOKEN = widget_manager->getSelectedWgt();
+    if(CLICKED_TOKEN < WTOK_FIRSTPRIX) return;
 
-    glMatrixMode(GL_PROJECTION);
-    glPushMatrix();
-    glLoadIdentity();
-    glOrtho(0.0, user_config->m_width, 0.0, user_config->m_height, -1.0, +1.0);
-    glMatrixMode(GL_MODELVIEW);
-    glEnable(GL_BLEND);
-    CupData *cup = m_all_cups[CLICKED_TOKEN];
-    glPushMatrix();
-    glBindTexture(GL_TEXTURE_2D, 0);
-    const GLfloat BACKGROUND_COLOUR[4] = { 0.3f, 0.3f, 0.3f, 0.5f };
-    glColor4fv(BACKGROUND_COLOUR);
-    glCallList(m_rect);
-    glPopMatrix();
-    widgetSet->drawText(cup->getDescription(), GUI_MED, SCREEN_CENTERED_TEXT, 10,
-                        255,255,255);
-    glDisable(GL_BLEND);
-    glMatrixMode(GL_PROJECTION);
-    glPopMatrix();
-    glMatrixMode(GL_MODELVIEW);
+    const CupData &cup = m_all_cups[CLICKED_TOKEN - WTOK_FIRSTPRIX];
+    widget_manager->setWgtText(WTOK_DESCRIPTION, cup.getDescription());
 
     return;
 }
@@ -100,13 +116,13 @@ void GrandPrixSelect::update(float dt)
 //-----------------------------------------------------------------------------
 void GrandPrixSelect::select()
 {
-    const int CLICKED_TOKEN = widgetSet->token(widgetSet->click());
-    if(CLICKED_TOKEN == -1)
+    const int CLICKED_TOKEN = widget_manager->getSelectedWgt();
+    if(CLICKED_TOKEN == WTOK_QUIT)
     {
         menu_manager->popMenu();
         return;
     }
-    race_manager->setGrandPrix(m_all_cups[CLICKED_TOKEN]);
+    race_manager->setGrandPrix(m_all_cups[CLICKED_TOKEN-WTOK_FIRSTPRIX]);
     menu_manager->pushMenu(MENUID_DIFFICULTY);
 }   // select
 
