@@ -27,8 +27,7 @@
 class GrandPrixSelect: public BaseGUI
 {
 private:
-    std::vector<CupData*> m_all_cups;
-    int m_rect;
+    std::vector<CupData> m_all_cups;
 public:
     GrandPrixSelect();
     ~GrandPrixSelect();
