@@ -1,4 +1,4 @@
-//  $Id: main_menu.hpp 854 2006-11-06 14:10:45Z coz $
+//  $Id: main_menu.hpp 1305 2007-11-26 14:28:15Z thebohemian $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -29,7 +29,8 @@ public:
     ~MainMenu();
 
     void select();
-    void inputKeyboard(int key, int pressed);
+    void handle(GameAction, int);
+				
 };
 
 #endif
