@@ -1,4 +1,4 @@
-//  $Id: menu_manager.hpp 1034 2007-05-06 13:37:39Z hiker $
+//  $Id: menu_manager.hpp 1611 2008-03-01 03:42:58Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 Patrick Ammann <pammann@aro.ch>
@@ -36,11 +36,9 @@ enum MenuManagerIDs
     MENUID_GAMEMODE,
     MENUID_RACERESULT,
     MENUID_GRANDPRIXEND,
-#if 0 // no needed yet
-    MENUID_NEXTRACE,
-#endif
     MENUID_RACEMENU,
     MENUID_TRACKSEL,
+    MENUID_NUMKARTS,
     MENUID_NUMLAPS,
     MENUID_NUMPLAYERS,
     MENUID_OPTIONS,
@@ -49,6 +47,8 @@ enum MenuManagerIDs
 
     // menu configuration
     MENUID_CONFIG_DISPLAY,
+    MENUID_RESOLUTION_CONFIRM_FS,
+    MENUID_RESOLUTION_CONFIRM_WIN,
     MENUID_CONFIG_SOUND,
     MENUID_CONFIG_CONTROLS,
     MENUID_CONFIG_P1,
@@ -57,8 +57,10 @@ enum MenuManagerIDs
     MENUID_CONFIG_P4,
 
     // help and credit menu
-    MENUID_HELP,
+    MENUID_HELP1,
+    MENUID_HELP2,
     MENUID_CREDITS,
+
     // race gui
     MENUID_RACE,
 };
@@ -75,12 +77,12 @@ public:
     void switchToMainMenu();
 
     // use this function within menu classes
-    void pushMenu(MenuManagerIDs id);
+    void pushMenu(MenuManagerIDs);
     void popMenu();
 
     int getMenuStackSize() {return (int)m_menu_stack.size();}
 
-    bool isCurrentMenu(MenuManagerIDs id) {return (m_menu_stack.back() == id);}
+    bool isCurrentMenu(MenuManagerIDs id) {return (m_menu_stack.back().first == id);}
     bool isSomewhereOnStack(MenuManagerIDs id);
     BaseGUI* getCurrentMenu() const {return m_current_menu;}
     RaceGUI* getRaceMenu   () const {return (RaceGUI*)m_RaceGUI;}
@@ -88,10 +90,10 @@ public:
     void update();
 
 private:
-    std::vector<MenuManagerIDs> m_menu_stack;
+    std::vector< std::pair<MenuManagerIDs, int> > m_menu_stack;
     BaseGUI* m_current_menu;
     BaseGUI* m_RaceGUI;
-    unsigned int m_handled_size;
+    bool m_change_menu;
 };
 
 extern MenuManager* menu_manager;
