@@ -1,4 +1,4 @@
-//  $Id: num_laps.hpp 694 2006-08-29 07:42:36Z hiker $
+//  $Id: num_laps.hpp 924 2007-03-06 17:33:35Z coz $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -21,13 +21,20 @@
 #define HEADER_NUMLAPS_H
 
 #include "base_gui.hpp"
+#include "translation.hpp"
 
-class NumLaps: public BaseGUI {
+class NumLaps: public BaseGUI
+{
 public:
-  NumLaps();
-  ~NumLaps();
-  
-  void select();
+    NumLaps();
+    ~NumLaps();
+
+    void select();
+    
+private:
+    int lap_label_id;
+    int laps;
+    char lap_label[MAX_MESSAGE_LENGTH];
 };
 
 #endif
