@@ -1,4 +1,4 @@
-//  $Id: player_controls.hpp 979 2007-04-11 22:08:08Z hiker $
+//  $Id: player_controls.hpp 1365 2007-12-20 04:37:35Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -20,11 +20,11 @@
 #ifndef HEADER_PLAYERCONTROLS_H
 #define HEADER_PLAYERCONTROLS_H
 
+#include <string>
+#include <SDL/SDL.h>
 #include "base_gui.hpp"
 #include "player.hpp"
 
-#include <string>
-
 class PlayerControls: public BaseGUI
 {
 public:
@@ -32,22 +32,30 @@ public:
     ~PlayerControls();
 
     void select();
-    void input(InputType type, int id0, int id1, int id2, int value);
-    void addKeyLabel(int change_id, KartActions control, bool start);
-    void changeKeyLabel(int grab_id, KartActions control);
-    void setKeyInfoString(KartActions control);
+    void clearMapping();
+	
+	void handle(GameAction, int);
+	void inputKeyboard(SDLKey, int);
+	void addKeyLabel(int change_id, KartAction control, bool start);
+	void setKeyInfoString(KartAction control);
 
 private:
-    int m_grab_id;
+	void updateAllKeyLabels();
+	
     int m_player_index;
-    int m_name_id;
     bool m_grab_input;
-    KartActions m_edit_action;
-    // Stores the heading - making this an attribute here avoids
-    // memory leaks or complicated memory management
-    char m_heading[60];
+	
+	/** Stores the KartAction for which the input is being sensed. */
+    KartAction m_edit_action;
+	
+	// Stores the heading - making this an attribute here avoids
+	// memory leaks or complicated memory management
+	char m_heading[60];
+	
     std::string m_name;
-    std::string m_key_names[KC_FIRE+1];
+    std::string m_key_names[KC_COUNT];
+	
+	static const size_t PLAYER_NAME_MAX;
 };
 
 #endif
