@@ -1,4 +1,4 @@
-//  $Id: race_menu.cpp 918 2007-02-26 23:34:43Z hiker $
+//  $Id: race_menu.cpp 1369 2007-12-25 03:23:32Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -19,43 +19,67 @@
 
 #include <SDL/SDL.h>
 
-#include "race_menu.hpp"
+#include "widget_manager.hpp"
 #include "user_config.hpp"
+#include "race_menu.hpp"
 #include "world.hpp"
-#include "widget_set.hpp"
 
 #include "menu_manager.hpp"
 #include "race_manager.hpp"
+#include "sound_manager.hpp"
 #include "translation.hpp"
 
-enum WidgetTokens {
+enum WidgetTokens
+{
+    WTOK_PAUSE,
     WTOK_RETURN_RACE,
     WTOK_OPTIONS,
     WTOK_HELP,
     WTOK_RESTART_RACE,
     WTOK_SETUP_NEW_RACE,
-    WTOK_EXIT_RACE,
+    WTOK_QUIT,
 };
 
 RaceMenu::RaceMenu()
 {
-    m_menu_id = widgetSet -> vstack(0);
-    widgetSet -> label(m_menu_id, _("Paused"), GUI_LRG, GUI_ALL, 0, 0);
+    const bool SHOW_RECT = true;
+    const bool SHOW_TEXT = true;
+    widget_manager->setInitialRectState(SHOW_RECT, WGT_AREA_ALL, WGT_TRANS_BLACK);
+    widget_manager->setInitialTextState(SHOW_TEXT, "", WGT_FNT_MED,
+        WGT_FONT_GUI );
 
-    const int VA = widgetSet -> varray(m_menu_id);
-    widgetSet -> start(VA, _("Return To Race"),  GUI_MED, WTOK_RETURN_RACE);
-    widgetSet -> state(VA, _("Options"),         GUI_MED, WTOK_OPTIONS);
-    widgetSet -> state(VA, _("Help"),            GUI_MED, WTOK_HELP);
-    widgetSet -> state(VA, _("Restart Race"),    GUI_MED, WTOK_RESTART_RACE);
+    widget_manager->addWgt(WTOK_PAUSE, 30, 7);
+    widget_manager->setWgtText(WTOK_PAUSE, _("Paused"));
+    widget_manager->breakLine();
+
+    widget_manager->setInitialActivationState(true);
+    widget_manager->addWgt(WTOK_RETURN_RACE, 30, 7);
+    widget_manager->setWgtText(WTOK_RETURN_RACE, _("Return To Race"));
+    widget_manager->breakLine();
+
+    widget_manager->addWgt(WTOK_OPTIONS, 30, 7);
+    widget_manager->setWgtText(WTOK_OPTIONS, _("Options"));
+    widget_manager->breakLine();
+
+    widget_manager->addWgt(WTOK_HELP, 30, 7);
+    widget_manager->setWgtText(WTOK_HELP, _("Help"));
+    widget_manager->breakLine();
+
+    widget_manager->addWgt(WTOK_RESTART_RACE, 30, 7);
+    widget_manager->setWgtText(WTOK_RESTART_RACE, _("Restart Race"));
+    widget_manager->breakLine();
 
     if(world->m_race_setup.m_mode==RaceSetup::RM_QUICK_RACE)
     {
-        widgetSet->state(VA, _("Setup New Race"),GUI_MED, WTOK_SETUP_NEW_RACE);
+        widget_manager->addWgt(WTOK_SETUP_NEW_RACE, 30, 7);
+        widget_manager->setWgtText(WTOK_SETUP_NEW_RACE, _("Setup New Race"));
+        widget_manager->breakLine();
     }
 
-    widgetSet -> state(VA, _("Exit Race"),       GUI_MED, WTOK_EXIT_RACE);
+    widget_manager->addWgt(WTOK_QUIT, 30, 7);
+    widget_manager->setWgtText(WTOK_QUIT, _("Exit Race"));
 
-    widgetSet -> layout(m_menu_id, 0, 0);
+    widget_manager->layout(WGT_AREA_ALL);
 
     if(user_config->m_fullscreen) SDL_ShowCursor(SDL_ENABLE);
 }
@@ -63,39 +87,33 @@ RaceMenu::RaceMenu()
 //-----------------------------------------------------------------------------
 RaceMenu::~RaceMenu()
 {
-    widgetSet -> delete_widget(m_menu_id) ;
-
-    if(user_config->m_fullscreen) SDL_ShowCursor(SDL_DISABLE);
+    widget_manager->reset();
 }
 
-//-----------------------------------------------------------------------------
-void RaceMenu::update(float dt)
-{
-    widgetSet -> timer(m_menu_id, dt) ;
-    // widgetSet -> blank();
-    widgetSet -> paint(m_menu_id) ;
-}
 
 //-----------------------------------------------------------------------------
 void RaceMenu::select()
 {
-    int clicked_token = widgetSet->token(widgetSet->click());
-    if(clicked_token != WTOK_OPTIONS && clicked_token != WTOK_HELP)
-        widgetSet -> tgl_paused();
+    int clicked_token = widget_manager->getSelectedWgt();
 
     switch (clicked_token)
     {
     case WTOK_RETURN_RACE:
+        world->unpause();
         menu_manager->popMenu();
+        if(user_config->m_fullscreen) SDL_ShowCursor(SDL_DISABLE);
         break;
 
     case WTOK_SETUP_NEW_RACE:
+        world->unpause();
         race_manager->exit_race();
         menu_manager->pushMenu(MENUID_DIFFICULTY);
         break;
 
     case WTOK_RESTART_RACE:
+        world->unpause();
         menu_manager->popMenu();
+        if(user_config->m_fullscreen) SDL_ShowCursor(SDL_DISABLE);
         world->restartRace();
         break;
 
@@ -104,10 +122,11 @@ void RaceMenu::select()
         break;
 
     case WTOK_HELP:
-        menu_manager->pushMenu(MENUID_HELP);
+        menu_manager->pushMenu(MENUID_HELP1);
         break;
 
-    case WTOK_EXIT_RACE:
+    case WTOK_QUIT:
+        world->unpause();
         race_manager->exit_race();
         break;
 
@@ -117,19 +136,20 @@ void RaceMenu::select()
 }
 
 //-----------------------------------------------------------------------------
-void RaceMenu::inputKeyboard(int key, int pressed)
+void RaceMenu::handle(GameAction ga, int value)
 {
-    switch ( key )
+    switch ( ga )
     {
-    case SDLK_ESCAPE: //ESC
-        if(!pressed)
+    case GA_LEAVE:
+        if (value)
             break;
-        widgetSet -> tgl_paused();
+		
+        world->unpause();
         menu_manager->popMenu();
         break;
 
     default:
-        BaseGUI::inputKeyboard(key, pressed);
+        BaseGUI::handle(ga, value);
         break;
     }
 }
