@@ -1,4 +1,4 @@
-//  $Id: race_menu.hpp 854 2006-11-06 14:10:45Z coz $
+//  $Id: race_menu.hpp 1305 2007-11-26 14:28:15Z thebohemian $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -28,9 +28,8 @@ public:
     RaceMenu();
     ~RaceMenu();
 
-    void update(float dt);
     void select();
-    void inputKeyboard(int key, int pressed);
+    void handle(GameAction ga, int value);
 };
 
 #endif
