@@ -1,4 +1,4 @@
-//  $Id: race_results_gui.cpp 1141 2007-06-05 03:28:53Z hiker $
+//  $Id: race_results_gui.cpp 1449 2008-02-08 04:43:35Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 Eduardo Hernandez Munoz
@@ -21,32 +21,60 @@
 #include <SDL/SDL.h>
 
 #include "race_results_gui.hpp"
-#include "widget_set.hpp"
+#include "widget_manager.hpp"
 #include "kart_properties.hpp"
 #include "world.hpp"
 #include "menu_manager.hpp"
 #include "race_manager.hpp"
+#include "highscore_manager.hpp"
 #include "translation.hpp"
 
-enum WidgetTokens {
+enum WidgetTokens
+{
+    WTOK_TITLE,
+    WTOK_EMPTY0,
+    WTOK_HIGHSCORES,
+    WTOK_RESULTS,
+    WTOK_EMPTY1,
     WTOK_CONTINUE,
     WTOK_RESTART_RACE,
     WTOK_SETUP_NEW_RACE,
+    WTOK_FIRST_HIGHSCORE,
+    //Add 3 because the maximum number of highscores is 3
+    WTOK_FIRST_RESULT = WTOK_FIRST_HIGHSCORE + 3
 };
 
 RaceResultsGUI::RaceResultsGUI()
 {
-    m_menu_id = widgetSet -> vstack(0);
-    widgetSet -> label(m_menu_id, _("Result"), GUI_LRG, GUI_ALL, 0, 0);
-    const unsigned int MAX_STR_LEN = 60;
-    widgetSet -> space(m_menu_id);
+    const bool SHOW_RECT = true;
+    const bool SHOW_TEXT = true;
+    widget_manager->setInitialRectState(SHOW_RECT, WGT_AREA_ALL, WGT_TRANS_BLACK);
+    widget_manager->setInitialTextState(SHOW_TEXT, "", WGT_FNT_MED,
+        WGT_FONT_GUI );
 
-    const int HA = widgetSet->harray(m_menu_id);
-    const int HIGHSCORE_TABLE = widgetSet->varray(HA);
-    const int RESULT_TABLE    = widgetSet->varray(HA);
-    widgetSet -> label(RESULT_TABLE,    _("Race results"),GUI_LRG,GUI_ALL,0,0);
-    widgetSet -> label(HIGHSCORE_TABLE, _("Highscores"),  GUI_LRG,GUI_ALL,0,0);
+    widget_manager->addWgt(WTOK_TITLE, 60, 7);
+    widget_manager->setWgtText(WTOK_TITLE, _("Result"));
+    widget_manager->breakLine();
+
+    widget_manager->addWgt(WTOK_EMPTY0, 60, 5);
+    widget_manager->hideWgtRect(WTOK_EMPTY0);
+    widget_manager->hideWgtText(WTOK_EMPTY0);
+    widget_manager->breakLine();
+
+    widget_manager->addWgt(WTOK_RESULTS, 50, 7);
+    widget_manager->setWgtText(WTOK_RESULTS, _("Race results"));
+
+    widget_manager->addWgt(WTOK_HIGHSCORES, 50, 7);
+    widget_manager->setWgtText(WTOK_HIGHSCORES, _("Highscores"));
+
+    widget_manager->breakLine();
+
+    widget_manager->insertColumn();
+
+
+    const unsigned int MAX_STR_LEN = 60;
     const unsigned int NUM_KARTS = world->getNumKarts();
+
     int*  order = new int [NUM_KARTS];
     m_score = new char[NUM_KARTS * MAX_STR_LEN];
     unsigned int max_name_len = 1;
@@ -56,7 +84,7 @@ RaceResultsGUI::RaceResultsGUI()
         Kart *k = world->getKart(i);
         order[k->getPosition()-1] = i;
         const std::string& s = k->getName();
-        unsigned int l = s.size();
+        unsigned int l = (unsigned int)s.size();
         if(l>max_name_len) max_name_len = l;
     }   // for i
 
@@ -65,39 +93,40 @@ RaceResultsGUI::RaceResultsGUI()
         const Kart *KART = world->getKart(order[i]);
         const std::string& KART_NAME = KART->getName();
         char sTime[20];
-        if(i==NUM_KARTS-1 && world->m_race_setup.m_mode!=RaceSetup::RM_TIME_TRIAL)
+        const float T      = KART->getFinishTime();
+        TimeToString(T, sTime);
+
+        //This shows position + driver name + time + points earned + total points
+        if(world->m_race_setup.m_mode==RaceSetup::RM_GRAND_PRIX)
         {
-            sprintf(sTime,"          ");
+            sprintf((char*)(m_score + MAX_STR_LEN * i), "%d. %s %s +%d %d",
+                KART->getPosition(), KART_NAME.c_str(), sTime,
+                race_manager->getPositionScore(i+1),
+                race_manager->getKartScore(order[i]));
         }
         else
-        {
-            const float T      = KART->getFinishTime();
-            TimeToString(T, sTime);
+            {
+            sprintf((char*)(m_score + MAX_STR_LEN * i), "%d. %s %s",
+                KART->getPosition(), KART_NAME.c_str(), sTime);
         }
-        //This shows position + driver name + time + points earned + total points
-	if(world->m_race_setup.m_mode==RaceSetup::RM_GRAND_PRIX)
-	{
-	    sprintf((char*)(m_score + MAX_STR_LEN * i), "%d. %s %s +%d %d",
-		    KART->getPosition(), KART_NAME.c_str(), sTime,
-		    race_manager->getPositionScore(i+1),
-		    race_manager->getKartScore(order[i]));
-	} 
-	else 
-        {
-	    sprintf((char*)(m_score + MAX_STR_LEN * i), "%d. %s %s",
-		    KART->getPosition(), KART_NAME.c_str(), sTime);
-	}
-	    widgetSet -> label(RESULT_TABLE, (char*)(m_score + MAX_STR_LEN * i),
-			       GUI_MED, GUI_ALL);
+
+        widget_manager->addWgt(WTOK_FIRST_RESULT + i, 50, 7);
+        widget_manager->setWgtText(WTOK_FIRST_RESULT + i,
+            (char*)(m_score + MAX_STR_LEN * i));
     }
 
     delete[] order;
 
+    widget_manager->breakLine();
+    widget_manager->insertColumn();
+
     const Highscores *hs = world->getHighscores();
     int num_scores = hs->getNumberEntries();
     m_highscores = new char[num_scores * MAX_STR_LEN];
+
     for(int i=0; i<num_scores; i++)
     {
+
         std::string kart_name, name;
         float T;
         hs->getEntry(i, kart_name, name, &T);
@@ -106,44 +135,47 @@ RaceResultsGUI::RaceResultsGUI()
         const int   TENTHS = (int) floor ( 10.0f * (T - (float)(SECS + 60*MINS)));
         sprintf((char*)(m_highscores + MAX_STR_LEN * i), 
                 "%s: %3d:%02d.%01d", name.c_str(), MINS, SECS, TENTHS);
-        widgetSet->label(HIGHSCORE_TABLE, (char*)(m_highscores+MAX_STR_LEN*i),
-                         GUI_MED, GUI_ALL);
-
+        widget_manager->addWgt(WTOK_FIRST_HIGHSCORE + i, 50, 7);
+        widget_manager->setWgtText(WTOK_FIRST_HIGHSCORE + i,
+            (char*)(m_highscores+MAX_STR_LEN*i));
     }
-    widgetSet -> space(m_menu_id);
+    widget_manager->breakLine();
+    widget_manager->breakLine();
 
-    //    const int VA = widgetSet -> varray(m_menu_id);
-
-    static int dev_msg_counter = 0;
-    if(dev_msg_counter == 2)
-    {
-        widgetSet -> label(m_menu_id, _("We need more developers! \
-if you want to help contact us!"), GUI_SML, GUI_ALL, 0, 0);
-        dev_msg_counter = 0;
-    }
-    ++dev_msg_counter;
+    widget_manager->addWgt(WTOK_EMPTY1, 60, 5);
+    widget_manager->hideWgtRect(WTOK_EMPTY1);
+    widget_manager->hideWgtText(WTOK_EMPTY1);
+    widget_manager->breakLine();
 
+    widget_manager->setInitialActivationState(true);
+    widget_manager->addWgt( WTOK_CONTINUE, 60, 7);
     if(world->m_race_setup.m_mode==RaceSetup::RM_GRAND_PRIX)
     {
-      widgetSet -> start(m_menu_id, _("Continue Grand Prix"),  GUI_MED, WTOK_CONTINUE);
+        widget_manager->setWgtText( WTOK_CONTINUE, _("Continue Grand Prix"));
     }
     else
     {
-      widgetSet -> start(m_menu_id, _("Back to the main menu"),  GUI_MED, WTOK_CONTINUE);
+        widget_manager->setWgtText( WTOK_CONTINUE, _("Back to the main menu"));
     }
-    widgetSet -> start(m_menu_id, _("Race in this track again"),  GUI_MED, WTOK_RESTART_RACE);
+    widget_manager->breakLine();
+
+    widget_manager->addWgt( WTOK_RESTART_RACE, 60, 7);
+    widget_manager->setWgtText( WTOK_RESTART_RACE, _("Race in this track again"));
+    widget_manager->breakLine();
+
     if(world->m_race_setup.m_mode==RaceSetup::RM_QUICK_RACE)
     {
-        widgetSet -> start(m_menu_id, _("Setup New Race"),  GUI_MED, WTOK_SETUP_NEW_RACE);
+        widget_manager->addWgt( WTOK_SETUP_NEW_RACE, 60, 7);
+        widget_manager->setWgtText( WTOK_SETUP_NEW_RACE, _("Setup New Race"));
     }
 
-    widgetSet -> layout(m_menu_id, 0, 0);
+    widget_manager->layout(WGT_AREA_ALL);
 }  // RaceResultsGUI
 
 //-----------------------------------------------------------------------------
 RaceResultsGUI::~RaceResultsGUI()
 {
-    widgetSet -> delete_widget(m_menu_id) ;
+    widget_manager->reset();
     delete[] m_score;
     delete[] m_highscores;
 }   // ~RaceResultsGUI
@@ -151,21 +183,21 @@ RaceResultsGUI::~RaceResultsGUI()
 //-----------------------------------------------------------------------------
 void RaceResultsGUI::select()
 {
-    switch( widgetSet->token( widgetSet->click() ) )
+    switch( widget_manager->getSelectedWgt() )
     {
     case WTOK_CONTINUE:
-        widgetSet->tgl_paused();
+        world->unpause();
         race_manager->next();
         break;
     case WTOK_RESTART_RACE:
-        widgetSet->tgl_paused();
+        world->unpause();
         menu_manager->popMenu();
         // TODO: Maybe let this go through the race_manager for
         // more flexibility.
         world->restartRace();
         break;
     case WTOK_SETUP_NEW_RACE:
-        widgetSet->tgl_paused();
+        world->unpause();
         race_manager->exit_race();
         menu_manager->pushMenu(MENUID_DIFFICULTY);
         break;
@@ -174,43 +206,16 @@ void RaceResultsGUI::select()
         break;
     }
 }   // select
-
-
 //-----------------------------------------------------------------------------
-void RaceResultsGUI::input(InputType type, int id0, int  id1, int id2, int value)
+void
+RaceResultsGUI::handle(GameAction ga, int value)
 {
-    if( (type==IT_STICKBUTTON && value && id1==1               ) ||
-        (type==IT_MOUSEBUTTON && value && id0==SDL_BUTTON_RIGHT)    )
-    {    // Usually here would be code to close this gui. Not only
-         // that doesn't has any real function in this gui,
-         // but also closing this gui causes bug #9157.
-        widgetSet->tgl_paused();
-        race_manager->next();
-    }
-    else
-    {
-        BaseGUI::input(type, id0, id1, id2, value);
-    }
-
-}   // input
-
-//-----------------------------------------------------------------------------
-void RaceResultsGUI::inputKeyboard(int key, int pressed)
-{
-    if (!pressed)
-        return;
-
-    if(key!=SDLK_ESCAPE)
-    {
-        BaseGUI::inputKeyboard(key, pressed);
-    }
-    else
-    {   // Usually here would be code to close this gui. Not only
-        // that doesn't has any real function in this gui,
-        // but also causes bug #9157.
-        widgetSet->tgl_paused();
-        race_manager->next();
-    }   // sif SDLK_ESCAPE
-}   // inputKeyboard
+  // Attempts to close the menu are silently discarded
+  // since they do not make sense at this point.
+  if (ga == GA_LEAVE)
+   return;
+  else
+    BaseGUI::handle(ga, value);
+}
 
 /* EOF */
