@@ -1,4 +1,4 @@
-//  $Id: race_results_gui.hpp 943 2007-03-28 02:20:44Z hiker $
+//  $Id: race_results_gui.hpp 1305 2007-11-26 14:28:15Z thebohemian $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 Eduardo Hernandez Munoz
@@ -33,10 +33,9 @@ public:
     RaceResultsGUI();
     ~RaceResultsGUI();
 
-    void select();
+    void handle(GameAction, int);
 
-    void input(InputType type, int id0, int  id1, int id2, int value);
-    void inputKeyboard(int key, int pressed);
+    void select();
 };
 
 #endif
