@@ -1,4 +1,4 @@
-//  $Id: track_sel.hpp 854 2006-11-06 14:10:45Z coz $
+//  $Id: track_sel.hpp 1347 2007-12-12 21:30:30Z cosmosninja $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -24,8 +24,6 @@
 
 class TrackSel: public BaseGUI
 {
-protected:
-    int   m_rect;
 public:
     TrackSel();
     ~TrackSel();
