@@ -1,4 +1,4 @@
-//  $Id: herring.cpp 855 2006-11-17 01:50:37Z coz $
+//  $Id: herring.cpp 1449 2008-02-08 04:43:35Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>
@@ -20,6 +20,7 @@
 #include "world.hpp"
 #include "herring.hpp"
 #include "kart.hpp"
+#include "scene.hpp"
 
 Herring::Herring(herringType _type, sgVec3* xyz, ssgEntity* model)
 {
@@ -34,7 +35,7 @@ Herring::Herring(herringType _type, sgVec3* xyz, ssgEntity* model)
     m_rotate->ref();
     m_rotate->addKid(model);
     m_root->addKid(m_rotate);
-    world->addToScene(m_root);
+    scene->add(m_root);
 
     m_type           = _type;
     m_eaten         = false;
@@ -68,7 +69,7 @@ void Herring::update(float delta)
 {
     if(m_eaten)
     {
-        const float T = m_time_to_return - world->m_clock;
+        const float T = m_time_to_return - world->getTime();
         if ( T > 0 )
         {
             sgVec3 hell;
@@ -98,6 +99,6 @@ void Herring::update(float delta)
 void Herring::isEaten()
 {
     m_eaten=true;
-    m_time_to_return=world->m_clock+2.0f;
+    m_time_to_return=world->getTime()+2.0f;
 }
 
