@@ -1,4 +1,4 @@
-//  $Id: herring_manager.cpp 1030 2007-05-03 03:31:24Z hiker $
+//  $Id: herring_manager.cpp 1248 2007-09-19 11:03:40Z thebohemian $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 Joerg Henrichs
@@ -20,7 +20,6 @@
 #include <stdexcept>
 #include <string>
 #include <sstream>
-#include "preprocessor.hpp"
 #include "user_config.hpp"
 #include "herring_manager.hpp"
 #include "loader.hpp"
@@ -133,7 +132,6 @@ void HerringManager::loadDefaultHerrings()
             std::string shortName = StringUtils::without_extension(*i);
             h->ref();
             h->setName(shortName.c_str());
-            preProcessObj(h);
             m_all_models[shortName] = h;
         }   // for i
 
@@ -324,7 +322,6 @@ void HerringManager::CreateDefaultHerring(sgVec3 colour, std::string name)
     tr -> addKid ( sh -> getRoot () ) ;
     tr -> addKid ( gset ) ;
     tr -> ref () ; /* Make sure it doesn't get deleted by mistake */
-    preProcessObj(tr);
     m_all_models[name] = tr;
 
 }   // CreateDefaultHerring
@@ -357,7 +354,7 @@ void HerringManager::loadHerringStyle(const std::string filename)
 
 //-----------------------------------------------------------------------------
 void HerringManager::setHerring(const lisp::Lisp *herring_node,
-                                char *colour, herringType type)
+                                const char *colour, herringType type)
 {
     std::string name;
     herring_node->get(colour, name);
