@@ -1,4 +1,4 @@
-//  $Id: herring_manager.hpp 907 2007-02-04 01:38:54Z coz $
+//  $Id: herring_manager.hpp 1248 2007-09-19 11:03:40Z thebohemian $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 Joerg Henrichs
@@ -49,7 +49,7 @@ private:
     std::string m_user_filename;
     void CreateDefaultHerring(sgVec3 colour, std::string name);
     void setDefaultHerringStyle();
-    void setHerring(const lisp::Lisp *herring_node, char *colour,
+    void setHerring(const lisp::Lisp *herring_node, const char *colour,
                     herringType type);
 
 public:
