@@ -1,4 +1,4 @@
-//  $Id: kart.hpp 1034 2007-05-06 13:37:39Z hiker $
+//  $Id: kart.hpp 1456 2008-02-11 02:20:51Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004-2005 Steve Baker <sjbaker1@airmail.net>
@@ -22,15 +22,14 @@
 #define HEADER_KART_H
 
 #include <plib/sg.h>
+#include "btBulletDynamicsCommon.h"
 
 #include "moveable.hpp"
 #include "particle_system.hpp"
 #include "kart_properties.hpp"
 #include "attachment.hpp"
 #include "collectable.hpp"
-#ifdef BULLET
-#include "btBulletDynamicsCommon.h"
-#endif
+#include "terrain_info.hpp"
 
 struct KartControl
 {
@@ -65,7 +64,7 @@ public:
 };
 
 // =============================================================================
-class Kart : public Moveable
+class Kart : public TerrainInfo, public Moveable
 {
 protected:
     bool         m_on_road;            //true if the kart is on top of the
@@ -79,7 +78,6 @@ protected:
     KartControl  m_controls;           // The position of the karts controls
     int          m_track_sector;       // index in driveline, special values
                                        // e.g. UNKNOWN_SECTOR can be negative!
-    float        m_zipper_time_left;
     sgVec2       m_last_track_coords;
     sgVec3       m_curr_track_coords;
     sgVec3       m_velocity_wc;        // velocity in world coordinates
@@ -87,27 +85,25 @@ protected:
     bool         m_skid_front;         // true if front tires are skidding
     bool         m_skid_rear;          // true if rear tires are skidding
     float        m_max_speed;          // maximum speed of the kart, computed from
-    float        m_wheelie_angle ;
-    float        m_current_friction;   // current friction
+    float        m_max_speed_reverse_ratio;
+    float        m_wheelie_angle;
+    float        m_zipper_time_left;   // zipper time left
     float        m_lap_start_time;     // Time at start of a new lap
+    float        m_kart_length;        // length of kart
     char         m_fastest_lap_message[255];
-    float        m_kart_width;         // width of the kart
     int          m_shortcut_count;     // counts number of times a shortcut is used
     int          m_shortcut_sector;    // segment on which the shortcut was started
     enum        {SC_NONE,              // no shortcut
                  SC_SKIPPED_SECTOR,    // skipped too many sectors
                  SC_OUTSIDE_TRACK}     // too far away from tracj
                  m_shortcut_type  ;    // what kind of shortcut was detected
+
     // physics parameters, storing it saves time
-#ifdef BULLET
     btRaycastVehicle::btVehicleTuning  *m_tuning;
+    btCompoundShape                    m_kart_chassis;
     btVehicleRaycaster                 *m_vehicle_raycaster;
     btRaycastVehicle                   *m_vehicle;
-    btBoxShape                         *m_kart_chassis;
-    btRigidBody                        *m_kart_body;
-    btDefaultMotionState               *m_motion_state;
     float                               m_kart_height;
-#endif
 
 private:
     int                 m_num_herrings_gobbled;
@@ -116,7 +112,7 @@ private:
     KartParticleSystem* m_smoke_system;
     ssgTransform*       m_exhaust_pipe;
 
-    float               m_wheel_position;
+    float               m_wheel_rotation;
     ssgTransform*       m_wheel_front_l;
     ssgTransform*       m_wheel_front_r;
     ssgTransform*       m_wheel_rear_l;
@@ -130,15 +126,11 @@ private:
     float               m_finish_time;
     bool                m_finished_race;
 
-#ifdef BULLET
     float               m_speed;
-#endif
+    bool                m_rescue;
 
 protected:
-    int                 m_rescue;
-#ifdef BULLET
     float               m_rescue_pitch, m_rescue_roll;
-#endif
     const KartProperties *m_kart_properties;
 
     /** Search the given branch of objects that match the wheel names
@@ -146,34 +138,29 @@ protected:
     void  load_wheels          (ssgBranch* obj);
 
 public:
-    Kart(const KartProperties* kartProperties_, int position_, sgCoord init_pos);
-    virtual ~Kart();
-
-    void loadData();
-
-    void placeModel ();
-    const KartProperties* getKartProperties() const
-        { return m_kart_properties; }
+                   Kart(const KartProperties* kartProperties_, int position_, 
+                        sgCoord init_pos);
+    virtual       ~Kart();
+    void           loadData();
+    void           placeModel ();
+    const KartProperties* 
+                   getKartProperties   () const      { return m_kart_properties; }
     void           setKartProperties   (const KartProperties *kp)
-    { m_kart_properties=kp;}
+                                          { m_kart_properties=kp;                }
     void           attach              (attachmentType attachment_, float time)
-    { m_attachment.set(attachment_, time);}
-    void           gotZipper           (float angle, float time)
-    {
-        m_wheelie_angle=angle;
-        m_zipper_time_left=time;
-    }
-    void           setCollectable      (collectableType t, int n)
-    { m_collectable.set(t, n);            }
-    void           setPosition         (int p)    {m_race_position = p;          }
-    int            getSector           () { return m_track_sector;               }
-    float          getDistanceDownTrack() { return m_curr_track_coords[1];       }
-    float          getDistanceToCenter () { return m_curr_track_coords[0];       }
-    Attachment    *getAttachment       () { return &m_attachment;                }
+                                          { m_attachment.set(attachment_, time); }
+    void           setCollectable      (CollectableType t, int n)
+                                          { m_collectable.set(t, n);             }
+    void           setPosition         (int p)    
+                                          { m_race_position = p;                 }
+    int            getSector           () const { return m_track_sector;         }
+    float          getDistanceDownTrack() const { return m_curr_track_coords[1]; }
+    float          getDistanceToCenter () const { return m_curr_track_coords[0]; }
+    Attachment    *getAttachment       ()       { return &m_attachment;          }
     void           setAttachmentType   (attachmentType t, float time_left=0.0f,
                                         Kart*k=NULL)
                                           { m_attachment.set(t, time_left, k);   }
-    Collectable   *getCollectable      () { return &m_collectable;               }
+    Collectable   *getCollectable      ()       { return &m_collectable;         }
     int            getNumCollectables  () const { return  m_collectable.getNum();}
     int            getNumHerring       () const { return  m_num_herrings_gobbled;}
     int            getLap              () const { return  m_race_lap;            }
@@ -183,10 +170,9 @@ public:
     bool           isTakingShortcut    () const  { return m_shortcut_type!=SC_NONE;}
     bool           raceIsFinished      () const  { return m_finished_race;       }
     void           endRescue           ();
+    float          estimateFinishTime  ();
     void           processSkidMarks    ();
     void           getClosestKart      (float *cdist, int *closest);
-    void           handleMagnet        (float cdist, int closest);
-    void           doZipperProcessing  (float dt);
     void           updatePhysics       (float dt);
     float          NormalizedLateralForce(float alpha, float corner) const;
 
@@ -203,46 +189,52 @@ public:
     float          getMaxPower      () const {return m_kart_properties->getMaxPower();}
     float          getTimeFullSteer () const {return m_kart_properties->getTimeFullSteer();}
     float          getBrakeFactor   () const {return m_kart_properties->getBrakeFactor();}
+    float          getBrakeForce    () const {return m_kart_properties->getBrakeForce();}
     float          getWheelBase     () const {return m_kart_properties->getWheelBase();}
     float          getHeightCOG     () const {return m_kart_properties->getHeightCOG();}
     float          getTireGrip      () const {return m_kart_properties->getTireGrip();}
+    float          getFrictionSlip  () const {return m_kart_properties->getFrictionSlip();}
     float          getMaxSteerAngle () const {return m_kart_properties->getMaxSteerAngle();}
     float          getCornerStiffF  () const {return m_kart_properties->getCornerStiffF();}
     float          getCornerStiffR  () const {return m_kart_properties->getCornerStiffR();}
     float          getInertia       () const {return m_kart_properties->getInertia();     }
+    float          getGravityCenterShift   () const
+        {return m_kart_properties->getGravityCenterShift();                    }
     float          getWheelieMaxSpeedRatio () const
-        {return m_kart_properties->getWheelieMaxSpeedRatio();}
+        {return m_kart_properties->getWheelieMaxSpeedRatio();                  }
     float          getWheelieMaxPitch  () const
-        {return m_kart_properties->getWheelieMaxPitch();   }
+        {return m_kart_properties->getWheelieMaxPitch();                       }
     float          getWheeliePitchRate () const
-        {return m_kart_properties->getWheeliePitchRate();  }
+        {return m_kart_properties->getWheeliePitchRate();                      }
     float          getWheelieRestoreRate() const
-        {return m_kart_properties->getWheelieRestoreRate();}
+        {return m_kart_properties->getWheelieRestoreRate();                    }
     float          getWheelieSpeedBoost() const
-        {return m_kart_properties->getWheelieSpeedBoost(); }
+        {return m_kart_properties->getWheelieSpeedBoost();                     }
     float          getSteerAngle    () const
-    {
-        return m_controls.lr*
-               m_kart_properties->getMaxSteerAngle();
-    }
+        {return m_controls.lr*m_kart_properties->getMaxSteerAngle();           }
     float          getAirResistance () const;
     float          getSteerPercent  () const {return m_controls.lr;            }
     float          getMaxSpeed      () const {return m_max_speed;              }
     void           setTimeAtLap     (float t){m_time_at_last_lap=t;            }
     float          getTimeAtLap     () const {return m_time_at_last_lap;       }
+    float          getKartLength    () const {return m_kart_length;            }
     void           createPhysics    (ssgEntity *obj);
-#ifdef BULLET
+    float          getKartHeight    () const {return m_kart_height;            }
+    float          getWheelieAngle  () const {return m_wheelie_angle;          }
     btRaycastVehicle *getVehicle    () const {return m_vehicle;                }
-    btRigidBody      *getKartBody   () const {return m_kart_body;              }
-    void              updateBulletPhysics(float dt);
-    void              draw          ();
-    bool              isInRest      ();
+    void           updateBulletPhysics(float dt);
+    void           draw             ();
+    bool           isInRest         () const;
     //have to use this instead of moveable getVelocity to get velocity for bullet rigid body
-    float             getSpeed      () const {return m_speed;                 }
-    float             handleWheelie(float dt);
-#endif
+    float          getSpeed         () const {return m_speed;                 }
+    float          handleWheelie    (float dt);
+    float          getActualWheelForce();
+    bool           isOnGround       () const;
+    bool           isRescue         () const {return m_rescue;}
+    void           resetBrakes      ();
     void           adjustSpeedWeight(float f);
     void           forceRescue      ();
+    void           handleExplosion  (const sgVec3& pos, bool direct_hit);
     const std::string& getName      () const {return m_kart_properties->getName();}
     virtual int    isPlayerKart     () const {return 0;                        }
     // addMessages gets called by world to add messages to the gui
@@ -250,12 +242,9 @@ public:
     virtual void   collectedHerring (Herring* herring);
     virtual void   reset            ();
     virtual void   handleZipper     ();
-    virtual void   forceCrash       ();
+    virtual void   crashed          () {};
     virtual void   doLapCounting    ();
     virtual void   update           (float dt               );
-    virtual void   doCollisionAnalysis(float dt, float hot    );
-    virtual void   doObjectInteractions();
-    virtual void   OutsideTrack     (int isReset) {m_rescue=true;}
 };
 
 class TrafficDriver : public Kart
@@ -268,7 +257,6 @@ public:
         sgCopyVec3 ( m_reset_pos.xyz, _pos ) ;
         reset () ;
     }
-    virtual void doObjectInteractions () ;
     virtual void doLapCounting        () ;
     virtual void doZipperProcessing   () ;
     virtual void update (float delta) ;
