@@ -1,4 +1,4 @@
-//  $Id: kart_properties.cpp 1025 2007-04-30 14:48:32Z hiker $
+//  $Id: kart_properties.cpp 1423 2008-01-30 04:36:00Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -24,11 +24,11 @@
 #include "lisp/parser.hpp"
 #include "lisp/lisp.hpp"
 #include "loader.hpp"
-#include "preprocessor.hpp"
 #include "string_utils.hpp"
 #include "kart_properties.hpp"
 #include "stk_config.hpp"
 #include "translation.hpp"
+#include "ssg_help.hpp"
 #if defined(WIN32) && !defined(__CYGWIN__)
 #  define snprintf _snprintf
 #endif
@@ -45,7 +45,8 @@ KartProperties::KartProperties() : m_icon_material(0), m_model(0)
 {}   // KartProperties
 
 //-----------------------------------------------------------------------------
-void KartProperties::load(const std::string filename, const std::string node)
+void KartProperties::load(const std::string filename, const std::string node,
+                          bool dont_load_models)
 {
 
     init_defaults();
@@ -79,12 +80,15 @@ void KartProperties::load(const std::string filename, const std::string node)
     // Load material
     m_icon_material = material_manager->getMaterial(m_icon_file);
 
-    // Load model
-    if(m_model_file.length()>0)
+    // Load model, except when called as part of --list-karts
+    if(m_model_file.length()>0 && !dont_load_models)
     {
         m_model = loader->load(m_model_file, CB_KART, false);
         ssgStripify(m_model);
-        preProcessObj(m_model, 0);
+        float x_min, x_max, y_min, y_max, z_min, z_max;
+        MinMax(m_model, &x_min, &x_max, &y_min, &y_max, &z_min, &z_max);
+        m_kart_width = x_max-x_min;
+        m_kart_length = y_max-y_min;
         m_model->ref();
     }  // if
 
@@ -109,14 +113,15 @@ void KartProperties::getAllData(const lisp::Lisp* lisp)
 
     lisp->get("wheel-base",              m_wheel_base);
     lisp->get("heightCOG",               m_height_cog);
-    lisp->get("engine-power",            m_engine_power);
+    lisp->get("bullet-engine-power",     m_engine_power);
     lisp->get("time-full-steer",         m_time_full_steer);
     lisp->get("brake-factor",            m_brake_factor);
+    lisp->get("brake-force",             m_brake_force);
     lisp->get("roll-resistance",         m_roll_resistance);
-    lisp->get("mass",                    m_mass);
+    lisp->get("bullet-mass",             m_mass);
     lisp->get("air-resistance",          m_air_resistance);
     lisp->get("tire-grip",               m_tire_grip);
-    lisp->get("max-steer-angle",         m_max_steer_angle);
+    lisp->get("bullet-max-steer-angle",  m_max_steer_angle);
     lisp->get("corn-f",                  m_corn_f);
     lisp->get("corn-r",                  m_corn_r);
     lisp->get("inertia",                 m_inertia);
@@ -140,7 +145,18 @@ void KartProperties::getAllData(const lisp::Lisp* lisp)
     lisp->get("wheel-width",               m_wheel_width              );
     lisp->get("chassis-linear-damping",    m_chassis_linear_damping   );
     lisp->get("chassis-angular-damping",   m_chassis_angular_damping  );
+    lisp->get("max-speed-reverse-ratio",   m_max_speed_reverse_ratio  );
     lisp->get("maximum-speed",             m_maximum_speed            );
+    lisp->get("gravity-center-shift",      m_gravity_center_shift     );
+    lisp->get("suspension-rest",           m_suspension_rest          );
+    lisp->get("jump-velocity",             m_jump_velocity            );
+    // getVector appends to existing vectors, so a new one must be used to load
+    std::vector<float> temp;
+    lisp->getVector("gear-switch-ratio",   temp);
+    if(temp.size()>0) m_gear_switch_ratio = temp;
+    temp.clear();
+    lisp->getVector("gear-power-increase", temp);
+    if(temp.size()>0) m_gear_power_increase = temp;
 
 }   // getAllData
 
@@ -156,11 +172,14 @@ void KartProperties::init_defaults()
 
     m_color[0] = 1.0f; m_color[1] = 0.0f; m_color[2] = 0.0f;
 
+    m_kart_width                = 1.0f;
+    m_kart_length               = 1.5f;
     m_wheel_base                = stk_config->m_wheel_base;
     m_height_cog                = stk_config->m_height_cog;
     m_engine_power              = stk_config->m_engine_power;
     m_time_full_steer           = stk_config->m_time_full_steer;
     m_brake_factor              = stk_config->m_brake_factor;
+    m_brake_force               = stk_config->m_brake_force;
     m_roll_resistance           = stk_config->m_roll_resistance;
     m_mass                      = stk_config->m_mass;
     m_air_resistance            = stk_config->m_air_resistance;
@@ -190,7 +209,12 @@ void KartProperties::init_defaults()
     m_chassis_linear_damping    = stk_config->m_chassis_linear_damping;
     m_chassis_angular_damping   = stk_config->m_chassis_angular_damping;
     m_maximum_speed             = stk_config->m_maximum_speed;
-
+    m_max_speed_reverse_ratio   = stk_config->m_max_speed_reverse_ratio;
+    m_gravity_center_shift      = stk_config->m_gravity_center_shift;
+    m_suspension_rest           = stk_config->m_suspension_rest;
+    m_jump_velocity             = stk_config->m_jump_velocity;
+    m_gear_switch_ratio         = stk_config->m_gear_switch_ratio;
+    m_gear_power_increase       = stk_config->m_gear_power_increase;
 
 }   // init_defaults
 
