@@ -1,4 +1,4 @@
-//  $Id: kart_properties.hpp 887 2006-12-22 01:56:10Z hiker $
+//  $Id: kart_properties.hpp 1292 2007-11-18 13:44:53Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -21,6 +21,7 @@
 #define HEADER_KARTPROPERTIES_H
 
 #include <string>
+#include <vector>
 #include "lisp/lisp.hpp"
 #include "no_copy.hpp"
 
@@ -49,6 +50,8 @@ protected:
 
     /* Physic properties */
     /* ----------------- */
+    float m_kart_width;               // width of kart
+    float m_kart_length;              // length of kart
     float m_mass;                     // weight of kart
     float m_air_resistance;           // air resistance
     float m_roll_resistance;          // rolling resistance etc
@@ -56,6 +59,7 @@ protected:
     float m_height_cog;               // height of center of gravity
     float m_engine_power;             // maximum force from engine
     float m_brake_factor;             // braking factor * engine_power = braking force
+    float m_brake_force;              // braking force
     float m_tire_grip;                // grip of tires in longitudinal direction
     float m_max_steer_angle;          // maximum steering angle
     float m_time_full_steer;          // time for player karts to reach full steer angle
@@ -83,6 +87,18 @@ protected:
     float m_chassis_linear_damping;
     float m_chassis_angular_damping;
     float m_maximum_speed;
+    float m_max_speed_reverse_ratio;
+    float m_gravity_center_shift;
+    float m_suspension_rest;
+    float m_jump_velocity;            // z velocity set when jumping
+    // The following two vectors define at what ratio of the maximum speed what
+    // gear is selected, e.g. 0.25 means: if speed <=0.25*maxSpeed --> gear 1,
+    //                        0.5  means: if speed <=0.5 *maxSpeed --> gear 2
+    // The next vector contains the increase in max power (to simulate different
+    // gears), e.g. 2.5 as first entry means: 2.5*maxPower in gear 1
+    std::vector<float> m_gear_switch_ratio;
+    std::vector<float> m_gear_power_increase;
+
 
 public:
     KartProperties   ();
@@ -91,7 +107,8 @@ public:
     virtual void  init_defaults    ();
     virtual void  getAllData       (const lisp::Lisp* lisp);
     virtual void  load             (const std::string filename,
-                                    const std::string node="tuxkart-kart");
+                                    const std::string node="tuxkart-kart",
+                                    bool dont_load_models=false);
 
     Material*     getIconMaterial        () const {return m_icon_material;          }
     ssgEntity*    getModel               () const {return m_model;                  }
@@ -106,6 +123,7 @@ public:
     float         getMaxPower            () const {return m_engine_power;           }
     float         getTimeFullSteer       () const {return m_time_full_steer;        }
     float         getBrakeFactor         () const {return m_brake_factor;           }
+    float         getBrakeForce          () const {return m_brake_force;            }
     float         getWheelBase           () const {return m_wheel_base;             }
     float         getHeightCOG           () const {return m_height_cog;             }
     float         getTireGrip            () const {return m_tire_grip;              }
@@ -113,6 +131,7 @@ public:
     float         getCornerStiffF        () const {return m_corn_f;                 }
     float         getCornerStiffR        () const {return m_corn_r;                 }
     float         getInertia             () const {return m_inertia;                }
+    float         getMaxSpeedReverseRatio() const {return m_max_speed_reverse_ratio;}
     float         getWheelieMaxSpeedRatio() const {return m_wheelie_max_speed_ratio;}
     float         getWheelieMaxPitch     () const {return m_wheelie_max_pitch;      }
     float         getWheeliePitchRate    () const {return m_wheelie_pitch_rate;     }
@@ -122,6 +141,8 @@ public:
     float         getWheelieBalanceRecovery()const{return m_wheelie_balance_recovery;}
     float         getWheelieStep         () const {return m_wheelie_step;           }
     float         getWheeliePowerBoost   () const {return m_wheelie_power_boost;    }
+    float         getKartLength          () const {return m_kart_length;            }
+    float         getKartWidth           () const {return m_kart_width;             }
 
     //bullet physics get functions
     float getSuspensionStiffness    () const {return m_suspension_stiffness;     }
@@ -134,7 +155,13 @@ public:
     float getChassisLinearDamping   () const {return m_chassis_linear_damping;   }
     float getChassisAngularDamping  () const {return m_chassis_angular_damping;  }
     float getMaximumSpeed           () const {return m_maximum_speed;            }
-
+    float getGravityCenterShift     () const {return m_gravity_center_shift;     }
+    float getSuspensionRest         () const {return m_suspension_rest;          }
+    float getJumpVelocity           () const {return m_jump_velocity;            }
+    const std::vector<float>& 
+          getGearSwitchRatio        () const {return m_gear_switch_ratio;        }
+    const std::vector<float>& 
+          getGearPowerIncrease      () const {return m_gear_power_increase;      }
 };
 
 #endif
