@@ -1,4 +1,4 @@
-//  $Id: lexer.cpp 1025 2007-04-30 14:48:32Z hiker $
+//  $Id: lexer.cpp 1422 2008-01-30 04:01:55Z hikerstk $
 //
 //  TuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Matthias Braun <matze@braunis.de>
@@ -19,6 +19,7 @@
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <sstream>
 #include <stdexcept>
+#include <cstring>
 
 #include "lexer.hpp"
 #include "translation.hpp"
@@ -43,7 +44,8 @@ namespace lisp
             nextChar();
         }
         catch(EOFException& e)
-        {}
+        {(void)e;  // avoid 'unreferenced local variable' warning
+        }
     }
 
 //-----------------------------------------------------------------------------
