@@ -1,4 +1,4 @@
-//  $Id: writer.cpp 1124 2007-05-28 07:00:05Z hiker $
+//  $Id: writer.cpp 1404 2008-01-17 00:04:29Z hikerstk $
 //
 //  TuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Matthias Braun <matze@braunis.de>
@@ -39,7 +39,6 @@ namespace lisp
 	// With mingw, the files are written dos style (crlf), but
 	// these files can't be read with the lexer here. So we have
 	// to force the file to be written as binary for windows.
-	printf("Setting binary");
         m_out = new std::ofstream(filename.c_str(),::std::ios_base::binary);
 #else
         m_out = new std::ofstream(filename.c_str());
@@ -192,7 +191,19 @@ namespace lisp
     }
 
 //-----------------------------------------------------------------------------
+	
+	void
+    Writer::write(const std::string& name, const std::vector<std::string>& value)
+    {
+        indent();
+        *m_out << '(' << name;
+        for(std::vector<std::string>::const_iterator i = value.begin(); i != value.end(); ++i)
+            *m_out << " " << "\"" << *i << "\"";
+        *m_out << ")\n";
+    }
 
+//-----------------------------------------------------------------------------
+    
     void
     Writer::indent()
     {
