@@ -1,4 +1,4 @@
-//  $Id: loader.hpp 855 2006-11-17 01:50:37Z coz $
+//  $Id: loader.hpp 1302 2007-11-22 01:20:57Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>
@@ -26,15 +26,15 @@
 #include <set>
 #include "callback_manager.hpp"
 
-#ifdef _MSC_VER
-const char DIR_SEPARATOR='\\';
-#else
-const char DIR_SEPARATOR='/';
-#endif
 
 class Loader : public ssgLoaderOptions
 {
 public:
+#ifdef _MSC_VER
+    static const char DIR_SEPARATOR='\\';
+#else
+    static const char DIR_SEPARATOR='/';
+#endif
     Loader();
     ~Loader();
 
@@ -57,6 +57,7 @@ private:
 
     void         makePath(char* path, const char* dir, const char* fname) const;
     ssgBranch   *createBranch(char *data) const;
+    void preProcessObj ( ssgEntity *n, bool mirror );
     ssgBranch   *animInit    (char *data) const;
 };
 
