@@ -1,4 +1,4 @@
-// $Id: main.cpp 1092 2007-05-20 15:30:00Z hiker $
+// $Id: main.cpp 1507 2008-02-17 12:58:12Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004-2005 Steve Baker <sjbaker1@airmail.net>
@@ -38,6 +38,10 @@
 #  include <unistd.h>
 #endif
 #include <stdexcept>
+#include <cstdio>
+#include <string>
+#include <sstream>
+#include <algorithm>
 
 #include "user_config.hpp"
 #include "track_manager.hpp"
@@ -47,9 +51,8 @@
 #include "projectile_manager.hpp"
 #include "race_manager.hpp"
 #include "loader.hpp"
-#include "screen_manager.hpp"
-#include "start_screen.hpp"
-#include "widget_set.hpp"
+#include "game_manager.hpp"
+#include "widget_manager.hpp"
 #include "material_manager.hpp"
 #include "sdldrv.hpp"
 #include "callback_manager.hpp"
@@ -60,33 +63,33 @@
 #include "stk_config.hpp"
 #include "translation.hpp"
 #include "highscore_manager.hpp"
+#include "gui/menu_manager.hpp"
+#include "scene.hpp"
 
-#ifdef BULLET
-#  ifdef __APPLE__
-#    include <GLUT/glut.h>
-#  else
-#    include <GL/glut.h>
-#  endif
+// Only needed for bullet debug!
+#ifdef __APPLE__
+#  include <GLUT/glut.h>
+#else
+#  include <GL/glut.h>
 #endif
 
 void cmdLineHelp (char* invocation)
 {
     fprintf ( stdout,
   _("Usage: %s [OPTIONS]\n\n"
-
     "Run SuperTuxKart, a racing game with go-kart that features"
     " the Tux and friends.\n\n"
-
     "Options:\n"
     "  -N,  --no-start-screen  Quick race\n"
-    //FIXME
-    //"  -t,  --track NAME       Start at track NAME (see --list-tracks)\n"
+    "  -t,  --track NAME       Start at track NAME (see --list-tracks)\n"
+    "       --stk-config FILE  use ./data/FILE instead of ./data/stk_config.data\n"
     "  -l,  --list-tracks      Show available tracks\n"
     "  -k,  --numkarts NUM     Number of karts on the racetrack\n"
-    "       --kart NAME        Use kart number NAME\n"
+    "       --kart NAME        Use kart number NAME (see --list-karts)\n"
     "       --list-karts       Show available karts\n"
     "       --laps N           Define number of laps to N\n"
-    //FIXME     "  --players n             Define number of players to between 1 and 4.\n"
+    "       --mode N           N=1 novice, N=2 driver, N=3 racer\n"
+    "  --players n             Define number of players to between 1 and 4.\n"
     //FIXME     "  --reverse               Enable reverse mode\n"
     //FIXME     "  --mirror                Enable mirror mode (when supported)\n"
     "       --herring STYLE    Use STYLE as your herring style\n"
@@ -97,7 +100,10 @@ void cmdLineHelp (char* invocation)
     // should not be used by unaware users:
     // "  --profile            Enable automatic driven profile mode for 20 seconds\n"
     // "  --profile=n          Enable automatic driven profile mode for n seconds\n"
+    // "                       if n<0 --> (-n) = number of laps to drive
     // "  --history            Replay history file 'history.dat'\n"
+    "  --log=terminal          Write messages to screen\n"
+    "  --log=file              Write messages/warning to log files stdout.log/stderr.log\n"
     "  -h,  --help             Show this help\n"
     "\n"
     "You can visit SuperTuxKart's homepage at "
@@ -125,9 +131,48 @@ int handleCmdLine(int argc, char **argv)
         {
             user_config->m_keyboard_debug=true;
         }
+        else if(sscanf(argv[i], "--track-debug=%d",&n)==1)
+        {
+            user_config->m_track_debug=n;
+        }
+        else if(!strcmp(argv[i], "--track-debug"))
+        {
+            user_config->m_track_debug=1;
+        }
+        else if(!strcmp(argv[i], "--bullet-debug"))
+        {
+            user_config->m_bullet_debug=1;
+        }
         else if( (!strcmp(argv[i], "--kart") && i+1<argc ))
         {
-            race_manager->setPlayerKart(0, argv[i+1]);
+            std::string filename=std::string("data")+
+                                  Loader::DIR_SEPARATOR+argv[i+1]+".tkkf";
+            try
+            {
+                std::string s=loader->getPath(filename);
+                race_manager->setPlayerKart(0, argv[i+1]);
+                fprintf ( stdout, _("You choose to use kart '%s'.\n"), argv[i+1] ) ;
+            }
+            catch(std::runtime_error)
+            {
+	            fprintf(stdout, _("Kart '%s' not found, ignored.\n"),
+		                argv[i+1]);
+            }
+        }
+        else if( (!strcmp(argv[i], "--mode") && i+1<argc ))
+        {
+            switch (atoi(argv[i+1]))
+            {
+            case 1:
+                race_manager->setDifficulty(RD_EASY);
+                break;
+            case 2:
+                race_manager->setDifficulty(RD_MEDIUM);
+                break;
+            case 3:
+                race_manager->setDifficulty(RD_HARD);
+                break;
+            }
         }
         else if( (!strcmp(argv[i], "--track") || !strcmp(argv[i], "-t"))
                  && i+1<argc                                              )
@@ -135,11 +180,23 @@ int handleCmdLine(int argc, char **argv)
             race_manager->setTrack(argv[i+1]);
             fprintf ( stdout, _("You choose to start in track: %s.\n"), argv[i+1] ) ;
         }
+        else if( (!strcmp(argv[i], "--stk-config")) && i+1<argc )
+        {
+            stk_config->load(std::string("data")+Loader::DIR_SEPARATOR+argv[i+1]);
+            fprintf ( stdout, _("STK config will be read from %s.\n"), argv[i+1] ) ;
+        }
         else if( (!strcmp(argv[i], "--numkarts") || !strcmp(argv[i], "-k")) &&
                  i+1<argc )
         {
-            race_manager->setNumKarts(user_config->m_karts = atoi(argv[i+1]));
-            fprintf ( stdout, _("You choose to have %s karts.\n"), argv[i+1] ) ;
+            user_config->m_karts = atoi(argv[i+1]);
+            if(user_config->m_karts>stk_config->m_max_karts) {
+                fprintf(stdout, _("Number of karts reset to maximum number %d\n"),
+                                  stk_config->m_max_karts);
+                user_config->m_karts = stk_config->m_max_karts;
+            }
+            race_manager->setNumKarts(user_config->m_karts );
+            fprintf ( stdout, _("%d karts will be used.\n"), user_config->m_karts);
+            i++;
         }
         else if( !strcmp(argv[i], "--list-tracks") || !strcmp(argv[i], "-l") )
         {
@@ -158,7 +215,8 @@ int handleCmdLine(int argc, char **argv)
         }
         else if( !strcmp(argv[i], "--list-karts") )
         {
-            kart_properties_manager->loadKartData () ;
+            bool dont_load_models=true;
+            kart_properties_manager->loadKartData(dont_load_models) ;
 
             fprintf ( stdout, _("  Available karts:\n") );
             for (unsigned int i = 0; NULL != kart_properties_manager->getKartById(i); i++)
@@ -210,7 +268,15 @@ int handleCmdLine(int argc, char **argv)
 #if !defined(WIN32) && !defined(__CYGWIN)
         else if ( !strcmp(argv[i], "--fullscreen") || !strcmp(argv[i], "-f"))
         {
-            user_config->m_fullscreen = true;
+            // Check that current res is not blacklisted
+            std::ostringstream o;
+    		o << user_config->m_width << "x" << user_config->m_height;
+    		std::string res = o.str();
+            if (std::find(user_config->m_blacklist_res.begin(), 
+              user_config->m_blacklist_res.end(),res) == user_config->m_blacklist_res.end())         
+            	user_config->m_fullscreen = true;
+          	else 
+          		fprintf ( stdout, _("Resolution %s has been blacklisted, so it is not available!\n"), res.c_str());
         }
         else if ( !strcmp(argv[i], "--windowed") || !strcmp(argv[i], "-w"))
         {
@@ -219,9 +285,24 @@ int handleCmdLine(int argc, char **argv)
 #endif
         else if ( !strcmp(argv[i], "--screensize") || !strcmp(argv[i], "-s") )
         {
-            if (sscanf(argv[i+1], "%dx%d", &user_config->m_width, &user_config->m_height) == 2)
-                fprintf ( stdout, _("You choose to be in %dx%d.\n"), user_config->m_width,
-                          user_config->m_height );
+            //Check if fullscreen and new res is blacklisted
+            int width, height;
+            if (sscanf(argv[i+1], "%dx%d", &width, &height) == 2)
+            {
+            	std::ostringstream o;
+    			o << width << "x" << height;
+    			std::string res = o.str();
+                if (!user_config->m_fullscreen || std::find(user_config->m_blacklist_res.begin(), 
+              	  user_config->m_blacklist_res.end(),res) == user_config->m_blacklist_res.end())
+                {
+                	user_config->m_prev_width = user_config->m_width = width;
+               		user_config->m_prev_height = user_config->m_height = height;
+                	fprintf ( stdout, _("You choose to be in %dx%d.\n"), user_config->m_width,
+                    	 user_config->m_height );
+               	}
+               	else
+               		fprintf ( stdout, _("Resolution %s has been blacklisted, so it is not available!\n"), res.c_str());
+            }
             else
             {
                 fprintf(stderr, _("Error: --screensize argument must be given as WIDTHxHEIGHT\n"));
@@ -238,9 +319,26 @@ int handleCmdLine(int argc, char **argv)
 #endif
             return 0;
         }
-        else if( sscanf(argv[i], "--profile=%d",  &n)==1)
+        else if( !strcmp(argv[i], "--log=terminal"))
+        {
+            user_config->m_log_errors=false;
+        }
+        else if( !strcmp(argv[i], "--log=file"))
+        {
+            user_config->m_log_errors=true;
+        }else if( sscanf(argv[i], "--profile=%d",  &n)==1)
         {
             user_config->m_profile=n;
+	    if(n<0) 
+	    {
+                fprintf(stdout,"Profiling %d laps\n",-n);
+                race_manager->setNumLaps(-n);
+	    }
+	    else
+            {
+	        printf("Profiling: %d seconds.\n",user_config->m_profile);
+                race_manager->setNumLaps   (999999); // profile end depends on time
+            }
         }
         else if( !strcmp(argv[i], "--profile") )
         {
@@ -263,7 +361,6 @@ int handleCmdLine(int argc, char **argv)
     }   // for i <argc
     if(user_config->m_profile)
     {
-        printf("Profiling: %d seconds.\n",user_config->m_profile);
         user_config->setSFX(UserConfig::UC_DISABLE);  // Disable sound effects 
         user_config->setMusic(UserConfig::UC_DISABLE);// and music when profiling
     }
@@ -287,18 +384,22 @@ void InitTuxkart()
     material_manager        = new MaterialManager      ();
     track_manager           = new TrackManager         ();
     stk_config              = new STKConfig            ();
-    const std::string STK_CONFIG = std::string("data")+DIR_SEPARATOR+ "stk_config.data";
-    stk_config->load(STK_CONFIG);
     kart_properties_manager = new KartPropertiesManager();
     projectile_manager      = new ProjectileManager    ();
     collectable_manager     = new CollectableManager   ();
     race_manager            = new RaceManager          ();
-    screen_manager          = new ScreenManager        ();
     callback_manager        = new CallbackManager      ();
     herring_manager         = new HerringManager       ();
     attachment_manager      = new AttachmentManager    ();
     highscore_manager       = new HighscoreManager     ();
     track_manager   ->loadTrackList () ;
+
+    // default settings for Quickstart
+    race_manager->setNumPlayers(1);
+    race_manager->setNumLaps   (3);
+    race_manager->setRaceMode  (RaceSetup::RM_QUICK_RACE);
+    race_manager->setDifficulty(RD_MEDIUM);
+    stk_config->load(std::string("data")+Loader::DIR_SEPARATOR+ "stk_config.data");
 }
 
 //=============================================================================
@@ -307,14 +408,38 @@ int main(int argc, char *argv[] )
 {
     try {
         initTranslations();
-#ifdef BULLET
         glutInit(&argc, argv);
-#endif
         InitTuxkart();
+
         //handleCmdLine() needs InitTuxkart() so it can't be called first
         if(!handleCmdLine(argc, argv)) exit(0);
-
+        
+        if (user_config->m_log_errors) //Enable logging of stdout and stderr to logfile
+        {
+            std::string logoutfile = user_config->getConfigDir();
+            logoutfile += Loader::DIR_SEPARATOR;
+            std::cout << "Error messages and other text output will be logged to " ;
+            std::cout << logoutfile << "stderr.log and stdout.log" << std::endl; 
+            std::string logerrfile = logoutfile;
+            logoutfile += "stdout.log";
+            logerrfile += "stderr.log";
+            if(freopen (logoutfile.c_str(),"w",stdout)!=stdout)
+            {
+                fprintf(stderr, "Can not open log file '%s'. Writing to stdout instead.\n",
+                        logoutfile.c_str());
+            }
+            if(freopen (logerrfile.c_str(),"w",stderr)!=stderr)
+            {
+                fprintf(stderr, "Can not open log file '%s'. Writing to stderr instead.\n",
+                        logerrfile.c_str());
+            }
+        }
+        
+        //FIXME: this needs a better organization
         drv_init();
+        ssgInit () ;
+        
+        game_manager = new GameManager ();
         // loadMaterials needs ssgLoadTextures (internally), which can
         // only be called after ssgInit (since this adds the actual loader)
         // so this next call can't be in InitTuxkart. And InitPlib needs
@@ -326,8 +451,15 @@ int main(int argc, char *argv[] )
         collectable_manager     -> loadCollectables   ();
         herring_manager         -> loadDefaultHerrings();
         attachment_manager      -> loadModels         ();
-        startScreen = new StartScreen();
-        widgetSet   = new WidgetSet;
+        scene = new Scene();
+
+        //For some reason, calling this before the material loading screws
+        //the background picture.
+        fntInit();
+        init_fonts();
+
+        widget_manager   = new WidgetManager;
+        menu_manager->switchToMainMenu();
 
         // Replay a race
         // =============
@@ -335,8 +467,8 @@ int main(int argc, char *argv[] )
         {
             // This will setup the race manager etc.
             history->Load();
-            startScreen->switchToGame();
-            screen_manager->run();
+            race_manager->start();
+            game_manager->run();
             // well, actually run() will never return, since
             // it exits after replaying history (see history::GetNextDT()).
             // So the next line is just to make this obvious here!
@@ -347,31 +479,24 @@ int main(int argc, char *argv[] )
         // =============
         if(!user_config->m_profile)
         {
-            if(!user_config->m_no_start_screen)
+            if(user_config->m_no_start_screen)
             {
-                
-                // Normal multi window start
-                // =========================
-                screen_manager->setScreen(startScreen);
-            }
-            else
-            {
-
                 // Quickstart (-N)
                 // ===============
-                race_manager->setNumPlayers(1);
-                race_manager->setNumKarts  (4);
-                race_manager->setRaceMode  (RaceSetup::RM_QUICK_RACE);
-                race_manager->setDifficulty(RD_MEDIUM);
-                race_manager->setPlayerKart(0, kart_properties_manager->getKart("tuxkart")->getIdent());
-                race_manager->setNumLaps   (3);
+                // all defaults are set in InitTuxkart()
+                //race_manager->setNumPlayers(1);
+                //race_manager->setNumKarts  (4);
+                //race_manager->setRaceMode  (RaceSetup::RM_QUICK_RACE);
+                //race_manager->setDifficulty(RD_MEDIUM);
+                //race_manager->setPlayerKart(0, kart_properties_manager->getKart("tuxkart")->getIdent());
+                //race_manager->setNumLaps   (3);
                 //race_manager->setTrack     ("tuxtrack");
                 //race_manager->setTrack     ("sandtrack");
-                race_manager->setTrack     ("race");
-                startScreen->switchToGame();
+                //race_manager->setTrack     ("race");
+                race_manager->start();
             }
         }
-        else  // profilie
+        else  // profile
         {
 
             // Profiling
@@ -379,11 +504,10 @@ int main(int argc, char *argv[] )
             race_manager->setNumPlayers(1);
             race_manager->setPlayerKart(0, kart_properties_manager->getKart("tuxkart")->getIdent());
             race_manager->setRaceMode  (RaceSetup::RM_QUICK_RACE);
-            race_manager->setDifficulty(RD_MEDIUM);
-            race_manager->setNumLaps   (999999); // profile end depends on time
-            startScreen->switchToGame  ();
+            race_manager->setDifficulty(RD_HARD);
+            race_manager->start();
         }
-        screen_manager->run();
+        game_manager->run();
 
     }  // try
     catch (std::exception &e)
@@ -392,11 +516,21 @@ int main(int argc, char *argv[] )
         fprintf(stderr,_("\nAborting SuperTuxKart\n"));
     }
 
+    /* Program closing...*/
+       
     user_config->saveConfig();
 
     drv_deinit();
+    
+    if (user_config->m_log_errors) //close logfiles
+    {
+        fclose(stderr);
+        fclose(stdout);
+    }
 
     delete highscore_manager;
+    delete_fonts();
+
     return 0 ;
 }
 
