@@ -1,4 +1,4 @@
-//  $Id: material.cpp 855 2006-11-17 01:50:37Z coz $
+//  $Id: material.cpp 1361 2007-12-17 13:16:09Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>
@@ -87,7 +87,7 @@ Material::Material ()
 }
 
 //-----------------------------------------------------------------------------
-Material::Material ( char *fname, char *description )
+Material::Material (const char *fname, char *description )
 {
     m_texname = new char [ strlen ( fname ) + 1 ] ;
     strcpy ( m_texname, fname ) ;
@@ -95,19 +95,21 @@ Material::Material ( char *fname, char *description )
 
     init () ;
 
-    m_clamp_tex    = parseBool  ( & description ) ? UCLAMP : 0 ;
-    m_clamp_tex   += parseBool  ( & description ) ? VCLAMP : 0 ;
-
-    m_transparency = parseBool  ( & description ) ;
-    m_alpha_ref    = parseFloat ( & description ) ;
-    m_lighting     = parseBool  ( & description ) ;
-    m_sphere_map   = parseBool  ( & description ) ;
-    m_friction     = parseFloat ( & description ) ;
-    m_ignore       = parseBool  ( & description ) ;
-    m_zipper       = parseBool  ( & description ) ;
-    m_resetter     = parseBool  ( & description ) ;
-    m_collideable  = parseBool  ( & description ) ;
-
+    if(strlen(description)>0)
+    {
+        m_clamp_tex    = parseBool  ( & description ) ? UCLAMP : 0 ;
+        m_clamp_tex   += parseBool  ( & description ) ? VCLAMP : 0 ;
+        
+        m_transparency = parseBool  ( & description ) ;
+        m_alpha_ref    = parseFloat ( & description ) ;
+        m_lighting     = parseBool  ( & description ) ;
+        m_sphere_map   = parseBool  ( & description ) ;
+        m_friction     = parseFloat ( & description ) ;
+        m_ignore       = parseBool  ( & description ) ;
+        m_zipper       = parseBool  ( & description ) ;
+        m_resetter     = parseBool  ( & description ) ;
+        m_collideable  = parseBool  ( & description ) ;
+    }
     install () ;
 }
 
@@ -127,7 +129,7 @@ void Material::init ()
     m_alpha_ref    = 0.1f  ;
     m_lighting     = true  ;
     m_sphere_map   = false ;
-    m_friction     = 1.0   ;
+    m_friction     = 1.0f  ;
     m_ignore       = false ;
     m_zipper       = false ;
     m_resetter     = false ;
@@ -156,8 +158,8 @@ void Material::install ()
     m_state -> setExternalPropertyIndex ( m_index ) ;
     if ( m_texname != NULL && m_texname [ 0 ] != '\0' )
     {
-        std::string fn=std::string("images")+DIR_SEPARATOR+m_texname;
-        m_state -> setTexture ( loader->getPath(fn.c_str()).c_str(), !(m_clamp_tex & UCLAMP),
+        std::string fn=std::string("images")+Loader::DIR_SEPARATOR+m_texname;
+        m_state -> setTexture ( loader->getPath(fn).c_str(), !(m_clamp_tex & UCLAMP),
                               !(m_clamp_tex & VCLAMP) );
         m_state -> enable  ( GL_TEXTURE_2D ) ;
     }
