@@ -1,4 +1,4 @@
-//  $Id: material_manager.cpp 1072 2007-05-14 12:08:12Z hiker $
+//  $Id: material_manager.cpp 1365 2007-12-20 04:37:35Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>
@@ -24,6 +24,7 @@
 #include "translation.hpp"
 #if defined(WIN32) && !defined(__CYGWIN__)
 #  define snprintf _snprintf
+#  define strdup _strdup
 #endif
 
 ssgState *fuzzy_gst;
@@ -46,7 +47,7 @@ MaterialManager::MaterialManager()
 int MaterialManager::addEntity(Material *m)
 {
     m_materials.push_back(m);
-    return m_materials.size()-1;
+    return (int)m_materials.size()-1;
 }
 
 //-----------------------------------------------------------------------------
@@ -187,7 +188,6 @@ Material *MaterialManager::getMaterial ( const char* fname )
     char basename [ 1024 ] ;
 
     strcpy ( basename, fn ) ;
-    free(fname_copy);
 
     /* Remove last trailing extension. */
 
@@ -231,13 +231,21 @@ Material *MaterialManager::getMaterial ( const char* fname )
 
             if ( *fn2 == '.' )
                 *fn2 = '\0' ;
-
             if ( strcmp ( basename, basename2 ) == 0 )
+            {
+                free(fname_copy);
                 return m_materials[i] ;
+            }
         }
     }
 
-    return NULL ;
+    // Add the material: the material constructor adds the material
+    // to (this) material_manager.
+    Material* m=new Material(fname,"");
+    // Since fn is a pointer into fname_copy, fname_copy must be freed
+    // here, not earlier.
+    free(fname_copy);
+    return m ;
 }
 
 //=============================================================================
