@@ -1,4 +1,4 @@
-//  $Id: moveable.hpp 886 2006-12-18 23:35:00Z hiker $
+//  $Id: moveable.hpp 1425 2008-01-30 05:30:10Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004-2005 Steve Baker <sjbaker1@airmail.net>
@@ -23,67 +23,61 @@
 
 #include <plib/ssg.h>
 #include "material.hpp"
+#include "btBulletDynamicsCommon.h"
+#include "user_pointer.hpp"
 
 /* Limits of Kart performance */
 #define CRASH_PITCH          -45.0f
 
-#define MAX_NATURAL_VELOCITY    ( 60.0f * KILOMETERS_PER_HOUR )
-#define MIN_CRASH_VELOCITY   (MAX_NATURAL_VELOCITY * 0.2f)
-#define MIN_COLLIDE_VELOCITY (MAX_NATURAL_VELOCITY * 0.1f)
-#define COLLIDE_BRAKING_RATE (MAX_NATURAL_VELOCITY * 1.0f)
-
 #define MAX_HERRING_EATEN    20
 
 
-class Moveable
+class Moveable 
 {
+private:
+    btVector3     m_velocityLC;      /* velocity in kart coordinates                */
 protected:
-    sgCoord       m_reset_pos;      /* Where to start in case of a reset           */
-    sgCoord       m_curr_pos;       /* current position                            */
-    sgCoord       m_velocity;       /* current velocity in local coordinates       */
-    sgVec3        m_abs_velocity;   /* world coordinates' velocity vector          */
+    UserPointer   m_user_pointer;
+    sgCoord       m_reset_pos;       /* Where to start in case of a reset           */
+    sgCoord       m_curr_pos;        /* current position                            */
     sgVec4*       m_normal_hot;      /* plane on which HOT was computed             */
     Material*     m_material_hot;    /* Material at HOT                             */
-    ssgTransform* m_model;
+    ssgTransform* m_model_transform;            // The transform where the model is under
     ssgTransform* m_shadow;
     int           m_collided;
     int           m_crashed;
     sgVec3        m_surface_avoidance_vector ;
     int           m_first_time ;
-    int           m_on_ground ;
-
-    float collectIsectData ( sgVec3 start, sgVec3 end ) ;
     sgCoord*      m_history_velocity;
     sgCoord*      m_history_position;
+    btRigidBody*          m_body;
+    btDefaultMotionState* m_motion_state;
+    btTransform   m_transform;
 
 public:
 
-    /* start - New Physics */
-
-
     Moveable (bool bHasHistory=false);
     virtual ~Moveable();
 
-    ssgTransform* getModel     ()              {return m_model ;                  }
-    int           isOnGround   ()              {return m_on_ground;               }
-    sgCoord*      getVelocity  ()              {return & m_velocity;              }
-    sgCoord*      getCoord     ()              {return &m_curr_pos;               }
-    void          setCoord     (sgCoord* pos)  {sgCopyCoord ( &m_curr_pos,pos);   }
-    virtual void  placeModel   ()              {m_model->setTransform(&m_curr_pos); }
+    ssgTransform* getModelTransform()          {return m_model_transform;          }
+    virtual const btVector3 &getVelocity() const {return m_body->getLinearVelocity();}
+    const btVector3 &getVelocityLC() const     {return m_velocityLC;               }
+    virtual void  setVelocity(const btVector3& v) {m_body->setLinearVelocity(v);    }
+    sgCoord*      getCoord     ()              {return &m_curr_pos;                }
+    const sgCoord* getCoord    ()  const       {return &m_curr_pos;                }
+    const sgVec4* getNormalHOT ()  const       {return m_normal_hot;               }
+    void          setCoord     (sgCoord* pos)  {sgCopyCoord ( &m_curr_pos,pos);    }
+    virtual void  placeModel   ();
     virtual void  handleZipper ()              {};
     virtual void  reset        ();
     virtual void  update       (float dt) ;
-    virtual void  updatePosition(float dt, sgMat4 result);
-    virtual void  doCollisionAnalysis(float dt, float hot);
-
-    // Gets called when no high of terrain can be determined (isReset=0), or
-    // there is a 'reset' material under the moveable --> karts need to be
-    // rescued, missiles should explode.
-    virtual void  OutsideTrack (int isReset) {}
-
-    float         getIsectData (sgVec3 start, sgVec3 end );
     void          WriteHistory (char* s, int kartNumber, int indx);
     void          ReadHistory  (char* s, int kartNumber, int indx);
+    btRigidBody*  getBody   () const {return m_body; }
+    void          createBody(float mass, btTransform& trans, 
+                             btCollisionShape *shape);
+    const btTransform&  getTrans() const {return m_transform;}
+    void          setTrans  (btTransform& t){m_transform=t;m_motion_state->setWorldTransform(t);}
 }
 ;   // class Moveable
 
