@@ -1,4 +1,4 @@
-//  $Id: player_kart.cpp 1034 2007-05-06 13:37:39Z hiker $
+//  $Id: player_kart.cpp 1474 2008-02-13 01:02:02Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004-2005 Steve Baker <sjbaker1@airmail.net>
@@ -28,73 +28,72 @@
 #include "gui/menu_manager.hpp"
 #include "gui/race_gui.hpp"
 #include "translation.hpp"
+#include "camera.hpp"
 
-void PlayerKart::action(KartActions action, int value, bool isKeyboard)
+void PlayerKart::action(KartAction action, int value)
 {
-    if(isKeyboard) m_action_keys_values[action]=value;
     switch (action)
     {
-    case KC_LEFT:
-        // Special case: when releasing a key on the keyboard, take the
-        // current value of the opposite steering direction. If the opposite
-        // key is not pressed, this value is zero, otherwise it will 
-        // pick up the previous direction (e.g. consider: press right,
-        // press left, release left --> will steer right;  or:
-        // press left, press right, release left --> will steer right)
-        if(isKeyboard && value==0)
-        {
-            m_steer_val = m_action_keys_values[KC_RIGHT];
-        }
+    case KA_LEFT:
+        m_steer_val_l = -value;
+        if (value)
+          m_steer_val = -value;
         else
-        {
-            m_steer_val = -value;
-        }
+          m_steer_val = m_steer_val_r;
+
         break;
-    case KC_RIGHT:
-        if(isKeyboard && value==0)
-        {
-            m_steer_val = -m_action_keys_values[KC_LEFT];
-        }
+    case KA_RIGHT:
+        m_steer_val_r = value;
+        if (value)
+          m_steer_val = value;
         else
-        {
-            m_steer_val = value;
-        }
+          m_steer_val = m_steer_val_l;
+
         break;
-    case KC_ACCEL:
+    case KA_ACCEL:
         m_accel_val = value;
         break;
-    case KC_BRAKE:
+    case KA_BRAKE:
         if (value)
             m_accel_val = 0;
-        m_controls.brake = value;
+        m_controls.brake = (value!=0);  // This syntax avoid visual c++ warning (when brake=value)
         break;
-    case KC_WHEELIE:
-        m_controls.wheelie = value;
+    case KA_WHEELIE:
+        m_controls.wheelie = (value!=0);
         break;
-    case KC_RESCUE:
-        m_controls.rescue = value;
+    case KA_RESCUE:
+        m_controls.rescue = (value!=0);
         break;
-    case KC_FIRE:
-        m_controls.fire = value;
+    case KA_FIRE:
+        m_controls.fire = (value!=0);
         break;
-    case KC_JUMP:
-#ifdef ENABLE_JUMPING
-        m_controls.jump = value;
-#endif
+    case KA_LOOK_BACK:
+        m_camera->setReverseHeading(value!=0);
+        break;
+    case KA_JUMP:
+        m_controls.jump = (value!=0);
         break;
     }
-}
+}   // action
 
 //-----------------------------------------------------------------------------
-void PlayerKart::smoothSteer(float dt, bool left, bool right)
+void PlayerKart::steer(float dt, int steer_val)
 {
     const float STEER_CHANGE = dt/getTimeFullSteer();  // amount the steering is changed
-    if       (left)
+    if (steer_val < 0)
     {
+      // If we got analog values do not cumulate.
+      if (steer_val > -32767)
+        m_controls.lr = -steer_val/32767.0f;
+      else
         m_controls.lr += STEER_CHANGE;
     }
-    else if(right)
+    else if(steer_val > 0)
     {
+      // If we got analog values do not cumulate.
+      if (steer_val < 32767)
+        m_controls.lr = -steer_val/32767.0f;
+      else
         m_controls.lr -= STEER_CHANGE;
     }
     else
@@ -113,16 +112,16 @@ void PlayerKart::smoothSteer(float dt, bool left, bool right)
 
     m_controls.lr = std::min(1.0f, std::max(-1.0f, m_controls.lr));
 
-}   // smoothSteer
+}   // steer
 
 //-----------------------------------------------------------------------------
 void PlayerKart::update(float dt)
 {
-    smoothSteer(dt, m_steer_val == -1, m_steer_val == 1);
+    steer(dt, m_steer_val);
 
-    m_controls.accel = m_accel_val;
+    m_controls.accel = m_accel_val / 32768.0f;
 
-    if(world->getPhase()==World::START_PHASE)
+    if(world->isStartPhase())
     {
         if(m_controls.accel!=0.0 || m_controls.brake!=false ||
            m_controls.fire|m_controls.wheelie|m_controls.jump)
@@ -140,15 +139,21 @@ void PlayerKart::update(float dt)
         return;
     }
 
-    if ( m_controls.fire && !m_rescue)
+    if ( m_controls.fire && !isRescue())
     {
         if (m_collectable.getType()==COLLECT_NOTHING) sound_manager->playSfx(SOUND_BEEP);
         // use() needs to be called even if there currently is no collecteable
-        // since use() tests for switching a magnet on/off.
+        // since use() can test if something needs to be switched on/off.
         m_collectable.use() ;
         m_controls.fire = false;
     }
-    if ( m_on_ground  &&  m_controls.rescue )
+#ifdef DEBUG
+    // Allow rescuing while still falling ... it's convenient for debugging, but
+    // not for a release
+    if ( m_controls.rescue )
+#else
+    if ( isOnGround()  &&  m_controls.rescue )
+#endif
     {
         sound_manager -> playSfx ( SOUND_BEEP ) ;
         forceRescue();
@@ -159,29 +164,40 @@ void PlayerKart::update(float dt)
 }   // update
 
 //-----------------------------------------------------------------------------
-void PlayerKart::forceCrash()
+void PlayerKart::crashed()
 {
-    Kart::forceCrash();
-    sound_manager->playSfx( SOUND_CRASH );
-}
+    Kart::crashed();
+    // A collision is usually reported several times, even when hitting
+    // something only once. This results in a kind of 'machine gun'
+    // noise by playing the crash sound over and over again. To prevent
+    // this, the crash sound is only played if there was at least 0.5
+    // seconds since the last time it was played (for this kart)
+    if(world->getTime() - m_time_last_crash_sound > 0.5f) 
+    {
+        sound_manager->playSfx( SOUND_CRASH );
+        m_time_last_crash_sound = world->getTime();
+    }
+}   // crashed
 
 //-----------------------------------------------------------------------------
 void PlayerKart::handleZipper()
 {
     Kart::handleZipper();
     sound_manager->playSfx ( SOUND_WEE );
-}
+}   // handleZipper
 
 //-----------------------------------------------------------------------------
 void PlayerKart::collectedHerring(Herring* herring)
 {
     Kart::collectedHerring(herring);
     sound_manager->playSfx ( ( herring->getType()==HE_GREEN ) ? SOUND_UGH:SOUND_GRAB);
-}
+}   // collectedHerring
 
 //-----------------------------------------------------------------------------
 void PlayerKart::reset()
 {
+    m_steer_val_l = 0;
+    m_steer_val_r = 0;
     m_steer_val = 0;
     m_accel_val = 0;
     m_controls.accel = 0.0;
@@ -190,12 +206,11 @@ void PlayerKart::reset()
     m_controls.wheelie = false;
     m_controls.jump = false;
     m_penalty_time = 0;
-    for(int i=KC_LEFT; i<=KC_FIRE; i++)
-    {
-        m_action_keys_values[i]=false;
-    }
+    m_time_last_crash_sound = -10.0f;
+    m_camera->setReverseHeading(false);
     Kart::reset();
-}
+}   // reset
+
 //-----------------------------------------------------------------------------
 /** This function is called by world to add any messages to the race gui. This
  *  can't be done (in some cases) in the update() function, since update can be
@@ -219,7 +234,7 @@ void PlayerKart::addMessages()
         // Display a warning message if the kart is going back way (unless
         // the kart has already finished the race).
         if ((angle_diff > 120.0f || angle_diff < -120.0f)   &&
-            getVelocity () -> xyz [ 1 ] > 0.0  && !raceIsFinished() )
+            getVelocity().getY() > 0.0f  && !raceIsFinished() )
         {
             m->addMessage(_("WRONG WAY!"), this, -1.0f, 60);
         }  // if angle is too big
