@@ -1,4 +1,4 @@
-//  $Id: player_kart.hpp 1034 2007-05-06 13:37:39Z hiker $
+//  $Id: player_kart.hpp 1447 2008-02-07 10:10:27Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004-2005 Steve Baker <sjbaker1@airmail.net>
@@ -26,35 +26,34 @@
 #include "player.hpp"
 
 class Player;
+class Camera;
 
 /** PlayerKart manages control events from the player and moves
     them to the Kart */
 class PlayerKart : public Kart
 {
 private:
-    int m_steer_val, m_accel_val;
+    int m_steer_val, m_steer_val_l, m_steer_val_r, m_accel_val;
 
     Player *m_player;
     float   m_penalty_time;
-    // This field saves all currently pressed keys for each action. This is used
-    // to solve problems like: press left, press right, release right --> 
-    // the kart should in this case steer left.
-    int     m_action_keys_values[KC_FIRE+1];
+    float   m_time_last_crash_sound;
+    Camera *m_camera;
 
-    void smoothSteer(float dt, bool left, bool right);
+    void steer(float, int);
 public:
     PlayerKart(const KartProperties *kart_properties,
                int position, Player *_player,
-               sgCoord init_pos) :
+               sgCoord init_pos, Camera *cam) :
         Kart(kart_properties, position, init_pos), m_player(_player),
-        m_penalty_time(0.0)         {reset(); }
+        m_penalty_time(0.0), m_camera(cam)       {reset(); }
 
     int     earlyStartPenalty () {return m_penalty_time>0; }
     Player* getPlayer         () {return m_player;        }
     void    update            (float);
     void    addMessages       ();
-    void    action            (KartActions action, int value, bool isKeyboard);
-    void    forceCrash        ();
+    void    action            (KartAction action, int value);
+    void    crashed           ();
     void    handleZipper      ();
     void    collectedHerring  (Herring* herring);
     int     isPlayerKart      () const {return 1;}
