@@ -1,4 +1,4 @@
-//  $Id: race_manager.cpp 1031 2007-05-03 16:58:34Z coz $
+//  $Id: race_manager.cpp 1451 2008-02-09 20:23:14Z scifly $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -22,13 +22,14 @@
 #include "loader.hpp"
 #include "track_manager.hpp"
 #include "race_setup.hpp"
-#include "screen_manager.hpp"
-#include "start_screen.hpp"
-#include "world_screen.hpp"
-#include "empty_screen.hpp"
+#include "game_manager.hpp"
 #include "kart_properties_manager.hpp"
 #include "race_manager.hpp"
 #include "gui/menu_manager.hpp"
+#include "world.hpp"
+#include "scene.hpp"
+#include "user_config.hpp"
+#include "stk_config.hpp"
 
 RaceManager* race_manager= NULL;
 
@@ -43,8 +44,15 @@ RaceMode::next()
 void
 RaceMode::exit_race()
 {
-    startScreen = new StartScreen();
-    screen_manager->setScreen(startScreen);
+    menu_manager->switchToMainMenu();
+
+    scene->clear();
+
+    delete world;
+    world = 0;
+
+    race_manager->m_active_race = false;
+
 }
 
 //=============================================================================
@@ -55,11 +63,15 @@ GrandPrixMode::GrandPrixMode(const std::vector<std::string>& players_,
 : m_difficulty(difficulty_), m_num_karts(numKarts_), m_players(players_),
     m_cup(cup_), m_track(0)
 {
+    const int NUM_PLAYERS = (int)m_players.size();
 
     std::vector<std::string> kart_names;
+    
+    if(m_num_karts < 0 ) m_num_karts = stk_config->m_max_karts;
+    if((size_t)m_num_karts < m_players.size()) m_num_karts = (int)m_players.size();
+
     kart_names.resize(m_num_karts);
 
-    const int NUM_PLAYERS = m_players.size();
     for(int i = 0; i < NUM_PLAYERS; ++i)
     {
         /*Players position is behind the AI in the first race*/
@@ -71,10 +83,10 @@ GrandPrixMode::GrandPrixMode(const std::vector<std::string>& players_,
     const int NUM_AI_KARTS = m_num_karts - NUM_PLAYERS;
     //Add the AI karts
     for(int i = 0; i < NUM_AI_KARTS; ++i)
-        m_karts.push_back(KartStatus(kart_names[i], 0, i, NUM_PLAYERS));
+        m_karts.push_back(KartStatus(kart_names[i], 0, 0.0f, i, NUM_PLAYERS));
     //Add the player karts
     for(int i = 0; i < NUM_PLAYERS; ++i)
-        m_karts.push_back(KartStatus(kart_names[i+NUM_AI_KARTS], 0, i+NUM_AI_KARTS, i));
+        m_karts.push_back(KartStatus(kart_names[i+NUM_AI_KARTS], 0, 0.0f, i+NUM_AI_KARTS, i));
 }
 
 //-----------------------------------------------------------------------------
@@ -84,23 +96,48 @@ GrandPrixMode::start_race(int n)
     RaceSetup raceSetup;
     raceSetup.m_mode       = RaceSetup::RM_GRAND_PRIX;
     raceSetup.m_difficulty = m_difficulty;
-    raceSetup.m_num_laps    = 2;
-    raceSetup.m_track      = m_cup.m_tracks[n];
+    raceSetup.m_num_laps   = m_cup.getLaps(n);
+    raceSetup.m_track      = m_cup.getTrack(n);
     raceSetup.m_karts.resize(m_karts.size());
     raceSetup.m_players.resize(m_players.size());
     raceSetup.setHerringStyle(m_cup.getHerringStyle());
 
-
-    for(int i = 0; i < int(m_karts.size()); ++i)
-    {
-        raceSetup.m_karts[m_karts[i].prev_finish_pos] = m_karts[i].ident;
-        if (m_karts[i].player < int(m_players.size()))
+    if (n == 0) //The first race, Players start at the back
+    { 
+        for(int i = 0; i < int(m_karts.size()); ++i)
         {
-            raceSetup.m_players[m_karts[i].player] = i;
+            raceSetup.m_karts[m_karts[i].prev_finish_pos] = m_karts[i].ident;
+            if (m_karts[i].player < int(m_players.size()))
+            {
+                raceSetup.m_players[m_karts[i].player] = m_karts[i].prev_finish_pos;
+            }
         }
     }
-
-    screen_manager->setScreen(new WorldScreen(raceSetup));
+    else //subsequent races where order of grid is determined by score
+    {
+        std::sort(m_karts.begin(), m_karts.end());//sort karts by increasing score
+        
+        //reverse kart order if flagged in stk_config
+        if (stk_config->m_grid_order)
+        {
+        std::reverse(m_karts.begin(), m_karts.end());
+        } 
+        
+        for(int i = 0;i < int(m_karts.size()); ++i)
+        {
+            raceSetup.m_karts[i] = m_karts[i].ident;
+            if (m_karts[i].player < int(m_players.size()))
+            {
+                raceSetup.m_players[m_karts[i].player] = i;
+            }
+        }
+    }
+    
+    // the constructor assigns this object to the global
+    // variable world. Admittedly a bit ugly, but simplifies
+    // handling of objects which get created in the constructor
+    // and need world to be defined.
+    new World(raceSetup);
 }
 
 //-----------------------------------------------------------------------------
@@ -115,8 +152,9 @@ void
 GrandPrixMode::next()
 {
     m_track += 1;
-    if (m_track < int(m_cup.m_tracks.size()))
+    if (m_track < int(m_cup.getTrackCount()))
     {
+        scene->clear();
         start_race(m_track);
 
     }
@@ -130,7 +168,7 @@ GrandPrixMode::next()
 void
 GrandPrixMode::exit_race()
 {
-    if (m_track < int(m_cup.m_tracks.size()))
+    if (m_track < int(m_cup.getTrackCount()))
     {
         RaceMode::exit_race();
     }
@@ -140,6 +178,7 @@ GrandPrixMode::exit_race()
     }
 }
 
+
 //=============================================================================
 QuickRaceMode::QuickRaceMode(const std::string& track_,
                              const std::vector<std::string>& players_,
@@ -147,7 +186,10 @@ QuickRaceMode::QuickRaceMode(const std::string& track_,
                              int numKarts_, int numLaps_)
         : m_track(track_), m_players(players_), m_difficulty(difficulty_),
         m_num_karts(numKarts_), m_num_laps(numLaps_)
-{}
+{
+    if(m_num_karts < 0 ) m_num_karts = stk_config->m_max_karts;
+    if((size_t)m_num_karts < m_players.size()) m_num_karts = (int)m_players.size();
+}
 
 //-----------------------------------------------------------------------------
 void
@@ -158,10 +200,10 @@ QuickRaceMode::start()
     raceSetup.m_mode       = RaceSetup::RM_QUICK_RACE;
     raceSetup.m_difficulty = m_difficulty;
     raceSetup.m_track      = m_track;
-    raceSetup.m_num_laps    = m_num_laps;
+    raceSetup.m_num_laps   = m_num_laps;
     raceSetup.m_karts.resize(m_num_karts);
 
-    const int FIRST_PLAYER = m_num_karts - m_players.size();
+    const int FIRST_PLAYER = m_num_karts - (int)m_players.size();
     for(int i = 0; i < int(m_players.size()); ++i)
     {
         raceSetup.m_karts[FIRST_PLAYER + i] = m_players[i]; // Players starts last in the race
@@ -170,7 +212,11 @@ QuickRaceMode::start()
 
     kart_properties_manager->fillWithRandomKarts(raceSetup.m_karts);
 
-    screen_manager->setScreen(new WorldScreen(raceSetup));
+    // the constructor assigns this object to the global
+    // variable world. Admittedly a bit ugly, but simplifies
+    // handling of objects which get created in the constructor
+    // and need world to be defined.
+    new World(raceSetup);
 }
 
 //=============================================================================
@@ -193,17 +239,22 @@ TimeTrialMode::start()
     raceSetup.m_karts.push_back(m_kart);
     raceSetup.m_players.push_back(0);
 
-    screen_manager->setScreen(new WorldScreen(raceSetup));
+    // the constructor assigns this object to the global
+    // variable world. Admittedly a bit ugly, but simplifies
+    // handling of objects which get created in the constructor
+    // and need world to be defined.
+    new World(raceSetup);
 }
 
 //=============================================================================
 RaceManager::RaceManager()
 {
-    m_mode       = 0;
-    m_num_karts  = 6;
-    m_difficulty = RD_MEDIUM;
-    m_race_mode  = RaceSetup::RM_QUICK_RACE;
-    m_track      = "race";
+    m_mode        = 0;
+    m_num_karts   = user_config->m_karts;
+    m_difficulty  = RD_MEDIUM;
+    m_race_mode   = RaceSetup::RM_QUICK_RACE;
+    m_track       = "race";
+    m_active_race = false;
 
     m_players.push_back("tuxkart");
 }
@@ -277,6 +328,8 @@ RaceManager::start()
     }
 
     m_mode->start();
+
+    m_active_race = true;
 }
 
 //-----------------------------------------------------------------------------
