@@ -36,6 +36,7 @@
 #include <cstdio>
 #include <iostream>
 #include "constants.hpp"
+#include "scene.hpp"
 #include "world.hpp"
 
 #include "default_robot.hpp"
@@ -44,8 +45,7 @@ DefaultRobot::DefaultRobot( const KartProperties *kart_properties,
                            int position, sgCoord init_pos ) :
     AutoKart( kart_properties, position, init_pos )
 {
-    m_time_since_last_shot = 0.0f;
-    m_start_kart_crash_direction = 0;
+    m_kart_properties      = kart_properties;
     reset();
 
     switch( world->m_race_setup.m_difficulty )
@@ -88,7 +88,7 @@ DefaultRobot::DefaultRobot( const KartProperties *kart_properties,
 //line, then move forward while turning.
 void DefaultRobot::update( float delta )
 {
-    if( world->getPhase() == World::START_PHASE )
+    if( world->isStartPhase())
     {
         handle_race_start();
         AutoKart::update( delta );
@@ -131,7 +131,7 @@ void DefaultRobot::handle_braking()
     //We may brake if we are about to get out of the road, but only if the
     //kart is on top of the road, and if we won't slow down below a certain
     //limit.
-    if ( m_crashes.m_road && m_on_road && m_velocity.xyz[1] > MIN_SPEED)
+    if ( m_crashes.m_road && m_on_road && getVelocityLC().getY() > MIN_SPEED)
     {
         float kart_ang_diff = world->m_track->m_angle[m_track_sector] -
             m_curr_pos.hpr[0];
@@ -170,8 +170,10 @@ void DefaultRobot::handle_braking()
         //Brake if the kart's speed is bigger than the speed we need
         //to go through the curve at the widest angle, or if the kart
         //is not going straight in relation to the road.
-        if(m_velocity.xyz[1] > m_curve_target_speed ||
-           kart_ang_diff > MIN_TRACK_ANGLE )
+        float angle_adjust = world->m_track->getAIAngleAdjustment();
+        float speed_adjust = world->m_track->getAICurveSpeedAdjustment();
+        if(getVelocityLC().getY() > speed_adjust*m_curve_target_speed ||
+           kart_ang_diff          > angle_adjust*MIN_TRACK_ANGLE         )
         {
 #ifdef AI_DEBUG
         std::cout << "BRAKING" << std::endl;
@@ -188,7 +190,7 @@ void DefaultRobot::handle_braking()
 //-----------------------------------------------------------------------------
 void DefaultRobot::handle_steering()
 {
-    const unsigned int DRIVELINE_SIZE = world->m_track->m_driveline.size();
+    const unsigned int DRIVELINE_SIZE = (unsigned int)world->m_track->m_driveline.size();
     const size_t NEXT_SECTOR = (unsigned int)m_track_sector + 1 < DRIVELINE_SIZE ?
         m_track_sector + 1 : 0;
     float steer_angle = 0.0f;
@@ -276,6 +278,9 @@ void DefaultRobot::handle_steering()
 #endif
 
     }
+    // avoid steer vibrations
+    if (fabsf(steer_angle) < 2.0f)
+        steer_angle = 0.f;
 
     m_controls.lr = angle_to_control( steer_angle );
 }
@@ -283,15 +288,13 @@ void DefaultRobot::handle_steering()
 //-----------------------------------------------------------------------------
 void DefaultRobot::handle_items( const float DELTA, const int STEPS )
 {
-    if( m_rescue )
+    m_controls.fire = false;
+
+    if(isRescue() )
     {
-        m_controls.fire = false;
         return;
     }
 
-    //FIXME: find real kart length
-    const float KART_LENGTH = 1.5f;
-
     m_time_since_last_shot += DELTA;
     if( m_collectable.getType() != COLLECT_NOTHING )
     {
@@ -303,7 +306,7 @@ void DefaultRobot::handle_items( const float DELTA, const int STEPS )
                 m_controls.fire = true;
                 m_time_since_last_shot = 0.0f;
             }
-            return;
+            break;
         case IT_CALCULATE:
             switch( m_collectable.getType() )
             {
@@ -320,21 +323,21 @@ void DefaultRobot::handle_items( const float DELTA, const int STEPS )
                         m_time_since_last_shot = 0.0f;
                     }
                 }
-                return;
+                break;
 
             case COLLECT_MISSILE:
-            case COLLECT_HOMING_MISSILE:
+            case COLLECT_HOMING:
                 if( m_time_since_last_shot > 5.0f && m_crashes.m_kart != -1 )
                 {
                     if( sgDistanceVec2( m_curr_pos.xyz,
                         world->getKart(m_crashes.m_kart)->getCoord()->xyz ) >
-                        KART_LENGTH * 2.5f )
+                        m_kart_properties->getKartLength() * 2.5f )
                     {
                         m_controls.fire = true;
                         m_time_since_last_shot = 0.0f;
                     }
                 }
-                return;
+                break;
 
             case COLLECT_SPARK:
                 if ( m_time_since_last_shot > 3.0f && m_crashes.m_kart != -1 )
@@ -342,8 +345,7 @@ void DefaultRobot::handle_items( const float DELTA, const int STEPS )
                     m_controls.fire = true;
                     m_time_since_last_shot = 0.0f;
                 }
-                return;
-                /*TODO: teach AI to use the magnet*/
+                break;
             default:
                 m_controls.fire = true;
                 m_time_since_last_shot = 0.0f;
@@ -353,8 +355,11 @@ void DefaultRobot::handle_items( const float DELTA, const int STEPS )
         }
     }
 
-
-    m_controls.fire = false;
+    if( m_controls.fire )
+    {
+        m_collectable.use() ;
+    }
+    return;
 }
 
 //-----------------------------------------------------------------------------
@@ -401,17 +406,16 @@ bool DefaultRobot::do_wheelie ( const int STEPS )
     if( m_crashes.m_road ) return false;
     if( m_crashes.m_kart != -1 ) return false;
 
-    //FIXME:The tuxkart is about 1.5f long and 1.0f wide, so I'm using
-    //these values for now, it won't work optimally on big or small karts.
-    const float KART_LENGTH = 1.5f;
-
-    sgVec2 vel_normal, step_coord, step_track_coord;
+    sgVec2 step_coord;
+    sgVec3 step_track_coord;
     float distance;
 
     //We have to be careful with normalizing, because if the source argument
     //has both the X and Y axis set to 0, it returns nan to the destiny.
-    if( m_abs_velocity[0] == 0.0 && m_abs_velocity[1] == 0.0 ) return false;
-    sgNormalizeVec2( vel_normal, m_abs_velocity );
+    const btVector3 &VEL=getVelocity();
+    if( VEL.getX() == 0.0 && VEL.getY() == 0.0 ) return false;
+    btVector3 vel_normal(VEL.getX(), VEL.getY(), 0.0);
+    vel_normal.normalize();
 
     //FIXME: instead of using 1.5, it should find out how much time it
     //will pass to stop doing the wheelie completely from the current state.
@@ -420,13 +424,13 @@ bool DefaultRobot::do_wheelie ( const int STEPS )
     /* The following method of finding if a position is outside of the track
        is less accurate than calling findRoadSector(), but a lot faster.
      */
-    const int WHEELIE_STEPS = int(( m_velocity.xyz[1] * CHECK_DIST )/
-        KART_LENGTH );
+    const int WHEELIE_STEPS = int(( getVelocityLC().getY() * CHECK_DIST )/
+        m_kart_properties->getKartLength() );
 
     for( int i = WHEELIE_STEPS; i > STEPS - 1; --i )
     {
         sgAddScaledVec2( step_coord, m_curr_pos.xyz, vel_normal,
-            KART_LENGTH * i );
+            m_kart_properties->getKartLength() * i );
 
         world->m_track->spatialToTrack( step_track_coord, step_coord,
             m_future_sector );
@@ -465,16 +469,33 @@ void DefaultRobot::handle_rescue(const float DELTA)
     //TODO: check if we collided against a dynamic object (ej.:kart) or
     //against the track's static object.
     //The m_crash_time measures if a kart has been crashing for too long
-    m_crash_time += (m_collided && m_on_ground) ? 3.0f * DELTA : -0.25f * DELTA;
+    m_crash_time += (m_collided && isOnGround()) ? 3.0f * DELTA : -0.25f * DELTA;
 
     if( m_crash_time < 0.0f ) m_crash_time = 0.0f;
 
     //Reaction to being stuck
     if( m_crash_time > 3.0f )
     {
-        m_rescue = true;
+        forceRescue();
         m_crash_time = 0.0f;
     }
+
+
+    // check if kart is stuck
+    if(getVehicle()->getRigidBody()->getLinearVelocity().length()<2.0f &&
+       !isRescue() && !world->isStartPhase())
+    {
+        m_time_since_stuck += DELTA;
+        if(m_time_since_stuck > 2.0f)
+        {
+            forceRescue();
+            m_time_since_stuck=0.0f;
+        }   // m_time_since_stuck > 2.0f
+    }
+    else
+    {
+        m_time_since_stuck = 0.0f;
+    }
 }
 
 //-----------------------------------------------------------------------------
@@ -524,27 +545,34 @@ void DefaultRobot::check_crashes( const int STEPS, sgVec3 const pos )
     //having karts too close in any direction. The crash with the track can
     //tell when a kart is going to get out of the track so it steers.
 
-    //FIXME:The tuxkart is about 1.5f long and 1.0f wide, so I'm using
-    //these values for now, it won't work optimally on big or small karts.
-    const float KART_LENGTH = 1.5f;
-
-    sgVec2 vel_normal, step_coord;
+    btVector3 vel_normal;
+	//in this func we use it as a 2d-vector, but later on it is passed
+	//to world->m_track->findRoadSector, there it is used as a 3d-vector
+	//to find distance to plane, so z must be initialized to zero
+	sgVec3 step_coord;
     SGfloat kart_distance;
 
+	step_coord[2] = 0.0;
+
     m_crashes.clear();
 
     const size_t NUM_KARTS = world->getNumKarts();
 
     //Protection against having vel_normal with nan values
-    if( m_abs_velocity[0] == 0.0 && m_abs_velocity[1] == 0.0 )
+    const btVector3 &VEL = getVelocity();
+    if( VEL.getX() == 0.0 && VEL.getY() == 0.0 )
     {
-        vel_normal[0] = vel_normal[1] = 0.0;
+        vel_normal.setValue(0.0, 0.0, 0.0);
+    }
+    else 
+    {
+        vel_normal.setValue(VEL.getX(), VEL.getY(), 0.0);
+        vel_normal.normalize();
     }
-    else sgNormalizeVec2( vel_normal, m_abs_velocity );
 
     for(int i = 1; STEPS > i; ++i)
     {
-        sgAddScaledVec2( step_coord, pos, vel_normal, KART_LENGTH * i );
+        sgAddScaledVec2( step_coord, pos, vel_normal, m_kart_properties->getKartLength() * i );
 
         /* Find if we crash with any kart, as long as we haven't found one
          * yet
@@ -558,14 +586,14 @@ void DefaultRobot::check_crashes( const int STEPS, sgVec3 const pos )
                 kart_distance = sgDistanceVec2( step_coord,
                     world->getKart(j)->getCoord()->xyz );
 
-                if( kart_distance < KART_LENGTH + 0.125f * i )
-                    if( m_velocity.xyz[1] > world->getKart(j)->
-                       getVelocity()->xyz[1] * 0.75f ) m_crashes.m_kart = j;
+                if( kart_distance < m_kart_properties->getKartLength() + 0.125f * i )
+                    if( getVelocityLC().getY() > world->getKart(j)->
+                       getVelocityLC().getY() * 0.75f ) m_crashes.m_kart = j;
             }
         }
 
         /*Find if we crash with the drivelines*/
-        int sector = world->m_track->findRoadSector( step_coord );
+        world->m_track->findRoadSector(step_coord, &m_sector);
 
 #ifdef SHOW_FUTURE_PATH
 
@@ -574,7 +602,7 @@ void DefaultRobot::check_crashes( const int STEPS, sgVec3 const pos )
 #ifdef ERASE_PATH
         static ssgaSphere *last_sphere = 0;
 
-        if( last_sphere ) world->m_scene->removeKid( last_sphere );
+        if( last_sphere ) scene->remove( last_sphere );
 
         last_sphere = sphere;
 #endif
@@ -584,8 +612,8 @@ void DefaultRobot::check_crashes( const int STEPS, sgVec3 const pos )
         center[1] = step_coord[1];
         center[2] = pos[2];
         sphere->setCenter( center );
-        sphere->setSize( KART_LENGTH );
-        if( sector == Track::UNKNOWN_SECTOR )
+        sphere->setSize( m_kart_properties->getKartLength() );
+        if( m_sector == Track::UNKNOWN_SECTOR )
         {
             sgVec4 colour;
             colour[0] = colour[3] = 255;
@@ -599,13 +627,13 @@ void DefaultRobot::check_crashes( const int STEPS, sgVec3 const pos )
             colour[3] = 255;
             sphere->setColour( colour );
         }
-        world->m_scene->addKid( sphere );
+        scene->add( sphere );
 #endif
 
         m_future_location[0] = step_coord[0]; m_future_location[1] =
             step_coord[1];
 
-        if( sector == Track::UNKNOWN_SECTOR )
+        if( m_sector == Track::UNKNOWN_SECTOR )
         {
             m_future_sector = world->m_track->findOutOfRoadSector( step_coord,
                 Track::RS_DONT_KNOW, m_future_sector );
@@ -614,7 +642,7 @@ void DefaultRobot::check_crashes( const int STEPS, sgVec3 const pos )
         }
         else
         {
-            m_future_sector = sector;
+            m_future_sector = m_sector;
         }
 
 
@@ -628,17 +656,14 @@ void DefaultRobot::check_crashes( const int STEPS, sgVec3 const pos )
  */
 void DefaultRobot::find_non_crashing_point( sgVec2 result )
 {
-    //FIXME:The tuxkart is about 1.5f long and 1.0f wide, so I'm using
-    //these values for now, it won't work correctly on big or small karts.
-    const float KART_LENGTH = 1.5f;
-
-    const unsigned int DRIVELINE_SIZE = world->m_track->m_driveline.size();
+    const unsigned int DRIVELINE_SIZE = (unsigned int)world->m_track->m_driveline.size();
 
     unsigned int sector = (unsigned int)m_track_sector + 1 < DRIVELINE_SIZE ?
         m_track_sector + 1 : 0;
     int target_sector;
 
-    sgVec2 direction, step_track_coord;
+    sgVec2 direction;
+    sgVec3 step_track_coord;
     SGfloat distance;
     int steps;
 
@@ -653,8 +678,8 @@ void DefaultRobot::find_non_crashing_point( sgVec2 result )
         sgSubVec2( direction, world->m_track->m_driveline[target_sector],
             m_curr_pos.xyz );
 
-        steps = int( sgLengthVec2( direction ) / KART_LENGTH );
-        if( steps < 1 ) steps = 1;
+        steps = int( sgLengthVec2( direction ) / m_kart_properties->getKartLength() );
+        if( steps < 3 ) steps = 3;
 
         sgVec2 step_coord;
 
@@ -668,7 +693,7 @@ void DefaultRobot::find_non_crashing_point( sgVec2 result )
         for( int i = 2; i < steps; ++i )
         {
             sgAddScaledVec2( step_coord, m_curr_pos.xyz, direction,
-                KART_LENGTH * i );
+                m_kart_properties->getKartLength() * i );
 
             world->m_track->spatialToTrack( step_track_coord, step_coord,
                 sector );
@@ -677,7 +702,7 @@ void DefaultRobot::find_non_crashing_point( sgVec2 result )
                        : -step_track_coord[0];
 
             //If we are outside, the previous sector is what we are looking for
-            if ( distance + KART_LENGTH * 0.5f > world->
+            if ( distance + m_kart_properties->getKartLength() * 0.5f > world->
                 m_track->getWidth()[sector] )
             {
                 sgCopyVec2( result, world->m_track->m_driveline[sector] );
@@ -687,7 +712,7 @@ void DefaultRobot::find_non_crashing_point( sgVec2 result )
 
                 static ssgaSphere *last_sphere = 0;
 
-                if(last_sphere) world->m_scene->removeKid( last_sphere );
+                if(last_sphere) scene->remove( last_sphere );
 
                 last_sphere = sphere;
 
@@ -703,7 +728,7 @@ void DefaultRobot::find_non_crashing_point( sgVec2 result )
                 colour[0] = colour[2] = 0;
                 sphere->setColour( colour );
 
-                world->m_scene->addKid( sphere );
+                scene->add( sphere );
 #endif
 
                 return;
@@ -716,6 +741,10 @@ void DefaultRobot::find_non_crashing_point( sgVec2 result )
 //-----------------------------------------------------------------------------
 void DefaultRobot::reset()
 {
+    m_time_since_last_shot = 0.0f;
+    m_start_kart_crash_direction = 0;
+
+    m_sector      = Track::UNKNOWN_SECTOR;
     m_inner_curve = 0;
     m_curve_target_speed = getMaxSpeed();
     m_curve_angle = 0.0;
@@ -728,8 +757,10 @@ void DefaultRobot::reset()
 
     m_crash_time = 0.0f;
 
+    m_time_since_stuck     = 0.0f;
+
     AutoKart::reset();
-}
+}   // reset
 
 //-----------------------------------------------------------------------------
 inline float DefaultRobot::normalize_angle( float angle )
@@ -748,18 +779,16 @@ inline float DefaultRobot::normalize_angle( float angle )
  */
 int DefaultRobot::calc_steps()
 {
-    //FIXME:The tuxkart is about 1.5f long and 1.0f wide, so I'm using
-    //these values for now, it won't work correctly on big or small karts.
-    const float KART_LENGTH = 1.5f;
-    int steps = int( m_velocity.xyz[1] / KART_LENGTH );
+    int steps = int( getVelocityLC().getY() / m_kart_properties->getKartLength() );
     if( steps < m_min_steps ) steps = m_min_steps;
 
     //Increase the steps depending on the width, if we steering hard,
     //mostly for curves.
     if( fabsf(m_controls.lr) > 0.95 )
     {
+        assert( m_future_sector > -1 );
         const int WIDTH_STEPS = (int)( world->m_track->getWidth(
-            )[m_future_sector] / ( KART_LENGTH * 2.0 ));
+            )[m_future_sector] / ( m_kart_properties->getKartLength() * 2.0 ));
 
         steps += WIDTH_STEPS;
     }
@@ -863,12 +892,12 @@ float DefaultRobot::get_approx_radius(const int START, const int END) const
  */
 void DefaultRobot::find_curve()
 {
-    const int DRIVELINE_SIZE = world->m_track->m_driveline.size();
+    const int DRIVELINE_SIZE = (unsigned int)world->m_track->m_driveline.size();
     float total_dist = 0.0f;
     int next_hint = m_track_sector;
     int i;
 
-    for(i = m_track_sector; total_dist < m_velocity.xyz[1]; i = next_hint)
+    for(i = m_track_sector; total_dist < getVelocityLC().getY(); i = next_hint)
     {
         next_hint = i + 1 < DRIVELINE_SIZE ? i + 1 : 0;
         total_dist += sgDistanceVec2(world->m_track->m_driveline[i], world->m_track->m_driveline[next_hint]);
