@@ -1,4 +1,4 @@
-//  $Id: skid_mark.cpp 1019 2007-04-24 09:11:04Z hiker $
+//  $Id: skid_mark.cpp 1333 2007-12-10 01:39:30Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Ingo Ruhnke <grumbel@gmx.de>
@@ -17,7 +17,7 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-#include "world.hpp"
+#include "scene.hpp"
 #include "skid_mark.hpp"
 
 float SkidMark::m_global_track_offset = 0.005f;
@@ -38,7 +38,7 @@ SkidMark::~SkidMark()
 {
     if(!m_skid_marks.empty())
     {
-        const unsigned int SIZE = m_skid_marks.size() -1;
+        const unsigned int SIZE = (unsigned int)m_skid_marks.size() -1;
         for(unsigned int i = 0; i < SIZE; ++i)
         {
             ssgDeRefDelete(m_skid_marks[i]);
@@ -52,14 +52,14 @@ void SkidMark::add(sgCoord* coord)
 {
     assert(m_skid_marking);
 
-    const int CURRENT = m_skid_marks.size() - 1;
+    const int CURRENT = (int)m_skid_marks.size() - 1;
     m_skid_marks[CURRENT]->add(coord);
 }   // add
 
 //-----------------------------------------------------------------------------
 void SkidMark::addBreak(sgCoord* coord)
 {
-    const unsigned int CURRENT = m_skid_marks.size() - 1;
+    const unsigned int CURRENT = (unsigned int)m_skid_marks.size() - 1;
     if(m_skid_marking)
         m_skid_marks[CURRENT]->addEnd(coord);
     else
@@ -103,7 +103,7 @@ void SkidMark::addBreak(sgCoord* coord)
                                    SkidMarkColors,
                                    m_global_track_offset);
         new_skid_mark->ref();
-        world->m_scene->addKid(new_skid_mark);
+        scene->add(new_skid_mark);
         new_skid_mark-> setState (m_skid_state);
 
         m_skid_marks.push_back(new_skid_mark);
