@@ -1,4 +1,4 @@
-//  $Id: track.cpp 1034 2007-05-06 13:37:39Z hiker $
+//  $Id: track.cpp 1513 2008-02-18 04:16:57Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>
@@ -20,6 +20,7 @@
 #include <iostream>
 #include <stdexcept>
 #include <sstream>
+#include <plib/ssgAux.h>
 #include "loader.hpp"
 #include "track.hpp"
 #include "string_utils.hpp"
@@ -27,10 +28,27 @@
 #include "lisp/parser.hpp"
 #include "stk_config.hpp"
 #include "translation.hpp"
+#include "scene.hpp"
+#include "moving_physics.hpp"
+#include "world.hpp"
+#include "material_manager.hpp"
+#include "isect.hpp"
+#include "ssg_help.hpp"
+#include "user_config.hpp"
+#include "herring.hpp"
+#include "herring_manager.hpp"
+
 #if defined(WIN32) && !defined(__CYGWIN__)
 #  define snprintf _snprintf
 #endif
 
+const float Track::NOHIT           = -99999.9f;
+const int   Track::QUAD_TRI_NONE   = -1;
+const int   Track::QUAD_TRI_FIRST  =  1;
+const int   Track::QUAD_TRI_SECOND =  2;
+const int   Track::UNKNOWN_SECTOR  = -1;
+
+// ----------------------------------------------------------------------------
 Track::Track( std::string filename_, float w, float h, bool stretch )
 {
     m_filename        = filename_;
@@ -51,6 +69,15 @@ Track::Track( std::string filename_, float w, float h, bool stretch )
 Track::~Track()
 {
 }   // ~Track
+//-----------------------------------------------------------------------------
+/** Removes the physical body from the world.
+ *  Called at the end of a race.
+ */
+void Track::cleanup()
+{
+    delete m_non_collision_mesh;
+    delete m_track_mesh;
+}   // ~Track
 
 //-----------------------------------------------------------------------------
 /** Finds on which side of the line segment a given point is.
@@ -104,93 +131,101 @@ int Track::pointInQuad
  *  The 'sector' could be defined as the number of the closest track
  *  segment to XYZ.
  */
-int Track::findRoadSector( const sgVec3 XYZ )const
+void Track::findRoadSector( const sgVec3 XYZ, int *sector )const
 {
+    if(*sector!=UNKNOWN_SECTOR)
+    {
+        int next = (unsigned)(*sector) + 1 <  m_left_driveline.size() ? *sector + 1 : 0;
+        if(pointInQuad( m_left_driveline[*sector], m_right_driveline[*sector],
+                        m_right_driveline[next],   m_left_driveline[next], 
+                        XYZ ) != QUAD_TRI_NONE)
+            // Still in the same sector, no changes
+            return;
+    }
     /* To find in which 'sector' of the track the kart is, we use a
        'point in triangle' algorithm for each triangle in the quad
        that forms each track segment.
      */
     std::vector <SegmentTriangle> possible_segment_tris;
-    const unsigned int DRIVELINE_SIZE = m_left_driveline.size();
+    const unsigned int DRIVELINE_SIZE = (unsigned int)m_left_driveline.size();
     int triangle;
     int next;
+
     for( size_t i = 0; i < DRIVELINE_SIZE ; ++i )
     {
-        next = i + 1 <  DRIVELINE_SIZE ? i + 1 : 0;
+        next = (unsigned int)i + 1 <  DRIVELINE_SIZE ? (int)i + 1 : 0;
         triangle = pointInQuad( m_left_driveline[i],     m_right_driveline[i],
                                 m_right_driveline[next], m_left_driveline[next], 
                                 XYZ );
 
-        if (triangle != QUAD_TRI_NONE)
+        if (triangle != QUAD_TRI_NONE && ((XYZ[2]-m_left_driveline[i][2]) < 1.0f))
         {
-            possible_segment_tris.push_back(SegmentTriangle(i, triangle));
+            possible_segment_tris.push_back(SegmentTriangle((int)i, triangle));
         }
     }
 
     /* Since xyz can be on more than one 2D track segment, we have to
        find on top of which one of the possible track segments it is.
      */
-    const int POS_SEG_SIZE = possible_segment_tris.size();
+    const int POS_SEG_SIZE = (int)possible_segment_tris.size();
     if( POS_SEG_SIZE == 0 )
     {
         //xyz is not on the road
-        return UNKNOWN_SECTOR;
+        *sector = UNKNOWN_SECTOR;
+        return;
     }
-    else //POS_SEG_SIZE > 1
+
+    //POS_SEG_SIZE > 1
+    /* To find on top of which track segment the variable xyz is,
+       we get which of the possible triangles that are under xyz
+       has the lower distance on the height(Y or Z) axis.
+    */
+    float dist;
+    float near_dist = 99999;
+    int nearest = QUAD_TRI_NONE;
+    size_t segment;
+    sgVec4 plane;
+    
+    for( int i = 0; i < POS_SEG_SIZE; ++i )
     {
-        /* To find on top of which track segment the variable xyz is,
-           we get which of the possible triangles that are under xyz
-           has the lower distance on the height(Y or Z) axis.
-         */
-        float dist;
-        float near_dist = 99999;
-        int nearest = QUAD_TRI_NONE;
-        size_t segment;
-        sgVec4 plane;
-
-        for( int i = 0; i < POS_SEG_SIZE; ++i )
+        segment = possible_segment_tris[i].segment;
+        next = segment + 1 < DRIVELINE_SIZE ? (int)segment + 1 : 0;
+        
+        if( possible_segment_tris[i].triangle == QUAD_TRI_FIRST )
         {
-             segment = possible_segment_tris[i].segment;
-             next = segment + 1 < DRIVELINE_SIZE ? segment + 1 : 0;
-
-             if( possible_segment_tris[i].triangle == QUAD_TRI_FIRST )
-             {
-                 sgMakePlane( plane, m_left_driveline[segment],
-                     m_right_driveline[segment], m_right_driveline[next] );
-             }
-             else //possible_segment_tris[i].triangle == QUAD_TRI_SECOND
-             {
-                  sgMakePlane( plane, m_right_driveline[next],
-                      m_left_driveline[next], m_left_driveline[segment] );
-             }
-
-             dist = sgHeightAbovePlaneVec3( plane, XYZ );
-
-             /* sgHeightAbovePlaneVec3 gives a negative dist if the plane
-                is on top, so we have to rule it out.
-
-                However, for some reason there are cases where we get
-                negative values for the track segment we should be on,
-                so we just use the absolute values, since the track
-                segment you are on will have a smaller absolute value
-                of dist anyways.
-              */
-             if( dist > -3.5 && dist < near_dist)
-             {
-                 near_dist = dist;
-                 nearest = i;
-             }
+            sgMakePlane( plane, m_left_driveline[segment],
+                         m_right_driveline[segment], m_right_driveline[next] );
         }
-
-        if( nearest != QUAD_TRI_NONE )
+        else //possible_segment_tris[i].triangle == QUAD_TRI_SECOND
         {
-            return possible_segment_tris[nearest].segment;
+            sgMakePlane( plane, m_right_driveline[next],
+                         m_left_driveline[next], m_left_driveline[segment] );
+        }
+        
+        dist = sgHeightAbovePlaneVec3( plane, XYZ );
+        
+        /* sgHeightAbovePlaneVec3 gives a negative dist if the plane
+           is on top, so we have to rule it out.
+           
+           However, for some reason there are cases where we get
+           negative values for the track segment we should be on.
+        */
+        if( dist > -2.0 && dist < near_dist)
+        {
+            near_dist = dist;
+            nearest = i;
         }
-        else return UNKNOWN_SECTOR; //This only happens if the position is
-                                    //under all the possible sectors
     }
+    
+    if( nearest != QUAD_TRI_NONE )
+    {
+        *sector=possible_segment_tris[nearest].segment;
+        return;
+    }
+    *sector = UNKNOWN_SECTOR;
+    return;                         // This only happens if the position is
+                                    // under all the possible sectors
 }   // findRoadSector
-
 //-----------------------------------------------------------------------------
 /** findOutOfRoadSector finds the sector where XYZ is, but as it name
     implies, it is more accurate for the outside of the track than the
@@ -228,8 +263,16 @@ int Track::findOutOfRoadSector
 {
     int sector = UNKNOWN_SECTOR;
     float dist;
-    float nearest_dist = 99999;
-    const unsigned int DRIVELINE_SIZE = m_left_driveline.size();
+    //FIXME: it can happen that dist is bigger than nearest_dist for all the
+    //the points we check (currently a limit of +/- 10), and if so, the
+    //function will return UNKNOWN_SECTOR, and if the AI get this, it will
+    //trigger an assertion. I increased the nearest_dist default value from
+    //99999 to 9999999, which is a lot more than the situation that caused
+    //the discovery of this problem, but the best way to solve this, is to
+    //find a better way of handling the shortcuts, and maybe a better way of
+    //calculating the distance.
+    float nearest_dist = 9999999;
+    const unsigned int DRIVELINE_SIZE = (unsigned int)m_left_driveline.size();
 
     int begin_sector = 0;
     int end_sector = DRIVELINE_SIZE - 1;
@@ -311,7 +354,7 @@ int Track::spatialToTrack
         return -1;
     }
 
-    const unsigned int DRIVELINE_SIZE = m_driveline.size();
+    const unsigned int DRIVELINE_SIZE = (unsigned int)m_driveline.size();
     const size_t PREV = SECTOR == 0 ? DRIVELINE_SIZE - 1 : SECTOR - 1;
     const size_t NEXT = (size_t)SECTOR+1 >= DRIVELINE_SIZE ? 0 : SECTOR + 1;
 
@@ -328,7 +371,7 @@ int Track::spatialToTrack
         p1 = PREV; p2 = SECTOR;
     }
 
-    sgVec2 line_eqn;
+    sgVec3 line_eqn;
     sgVec2 tmp;
 
     sgMake2DLine ( line_eqn, m_driveline[p1], m_driveline[p2] );
@@ -348,7 +391,7 @@ int Track::spatialToTrack
                    / (m_distance_from_start[p2]-m_distance_from_start[p1]);
     dst[2] = m_path_width[p1]*(1-fraction)+fraction*m_path_width[p2];
 
-    return p1;
+    return (int)p1;
 }   // spatialToTrack
 
 //-----------------------------------------------------------------------------
@@ -357,6 +400,33 @@ void Track::trackToSpatial ( sgVec3 xyz, const int SECTOR ) const
     sgCopyVec3 ( xyz, m_driveline [ SECTOR ] ) ;
 }   // trackToSpatial
 
+//-----------------------------------------------------------------------------
+/** Returns the start coordinates for a kart on a given position pos
+    (with 0<=pos).
+ */
+void Track::getStartCoords(unsigned int pos, sgCoord* coords) const {
+  // Bug fix/workaround: sometimes the first kart would be too close
+  // to the first driveline point and not to the last one -->
+  // This kart would not get any lap counting done in the first
+  // lap! Therefor -1.5 is subtracted from the y position - which
+  // is a somewhat arbitrary value.
+  coords->xyz[0] = pos<m_start_x.size() ? m_start_x[pos] : ((pos%2==0)?1.5f:-1.5f);
+  coords->xyz[1] = pos<m_start_y.size() ? m_start_y[pos] : -1.5f*pos-1.5f;
+   // height must be larger than the actual hight for which hot is computed.
+  coords->xyz[2] = pos<m_start_z.size() ? m_start_z[pos] : 1.0f;
+
+  coords->hpr[0] = pos<m_start_heading.size() ? m_start_heading[pos] : 0.0f;
+  coords->hpr[1] = 0.0f;
+  coords->hpr[2] = 0.0f;
+
+  btVector3 tmp_pos(coords->xyz[0],coords->xyz[1],coords->xyz[2]);
+
+  btVector3 normal;
+  const Material *material=NULL;
+  getTerrainInfo(tmp_pos, &(coords->xyz[2]), &normal, &material);
+
+}   // getStartCoords
+
 //-----------------------------------------------------------------------------
 /** Determines if a kart moving from sector OLDSEC to sector NEWSEC
  *  would be taking a shortcut, i.e. if the distance is larger
@@ -369,11 +439,71 @@ bool Track::isShortcut(const int OLDSEC, const int NEWSEC) const
     unsigned int distance_sectors = abs(OLDSEC-NEWSEC);
     // Handle 'wrap around': if the distance is more than half the 
     // number of driveline poins, assume it's a 'wrap around'
-    if(2*distance_sectors > m_driveline.size())
-        distance_sectors = m_driveline.size() - distance_sectors;
+    if(2*distance_sectors > (unsigned int)m_driveline.size())
+        distance_sectors = (unsigned int)m_driveline.size() - distance_sectors;
     return (distance_sectors>stk_config->m_shortcut_segments);
 }   // isShortcut
 
+//-----------------------------------------------------------------------------
+void Track::addDebugToScene(int type) const
+{
+    if(type&1)
+    {
+        ssgaSphere *sphere;
+        sgVec3 center;
+        sgVec4 colour;
+        for(unsigned int i = 0; i < m_driveline.size(); ++i)
+        {
+            sphere = new ssgaSphere;
+            sgCopyVec3(center, m_driveline[i]);
+            sphere->setCenter(center);
+            sphere->setSize(getWidth()[i] / 4.0f);
+        
+            if(i == 0)
+            {
+                colour[0] = colour[2] = colour[3] = 255;
+                colour[1] = 0;
+            }
+            else
+            {
+                colour[0] = colour[1] = colour[3] = 255;
+                colour[2] = 0;
+            }
+            sphere->setColour(colour);
+            scene->add(sphere);
+        }   // for i
+    }  /// type ==1
+    if(type&2)
+    {
+        ssgVertexArray* v_array = new ssgVertexArray();
+        ssgColourArray* c_array = new ssgColourArray();
+        for(unsigned int i = 0; i < m_driveline.size(); i++)
+        {
+            int ip1 = i==m_driveline.size()-1 ? 0 : i+1;
+            // The segment display must be slightly higher than the
+            // track, otherwise it's not clearly visible.
+            sgVec3 v;
+            sgCopyVec3(v,m_left_driveline [i  ]); v[2]+=0.1f; v_array->add(v);
+            sgCopyVec3(v,m_right_driveline[i  ]); v[2]+=0.1f; v_array->add(v);
+            sgCopyVec3(v,m_right_driveline[ip1]); v[2]+=0.1f; v_array->add(v);
+            sgCopyVec3(v,m_left_driveline [ip1]); v[2]+=0.1f; v_array->add(v);
+            sgVec4 vc;
+            vc[0] = i%2==0 ? 1.0f : 0.0f;
+            vc[1] = 1.0f-v[0];
+            vc[2] = 0.0f;
+            vc[3] = 0.1f;
+            c_array->add(vc);c_array->add(vc);c_array->add(vc);c_array->add(vc);
+        }   // for i
+        // if GL_QUAD_STRIP is used, the colours are smoothed, so the changes
+        // from one segment to the next are not visible.
+        ssgVtxTable* l = new ssgVtxTable(GL_QUADS, v_array,
+                                         (ssgNormalArray*)NULL,
+                                         (ssgTexCoordArray*)NULL,
+                                         c_array);
+        scene->add(l);
+    }
+}   // addDebugToScene
+
 //-----------------------------------------------------------------------------
 /** It's not the nicest solution to have two very similar version of a function,
  *  i.e. drawScaled2D and draw2Dview - but to keep both versions const, the
@@ -386,14 +516,13 @@ bool Track::isShortcut(const int OLDSEC, const int NEWSEC) const
 
 void Track::drawScaled2D(float x, float y, float w, float h) const
 {
-    sgVec2 sc ;
-    sgSubVec2 ( sc, m_driveline_max, m_driveline_min ) ;
+    sgVec2 sc;
+    sgSubVec2 ( sc, m_driveline_max, m_driveline_min );
 
-    float sx = (w-20.0f) / sc[0]; // leave 10 pix space left and right
-    x+=10.0;
-    float sy = h / sc[1] ;
+    float sx = w / sc[0];
+    float sy = h / sc[1];
 
-    if(sx>sy)
+    if( sx > sy )
     {
         sx = sy;
         x += w/2 - sc[0]*sx/2;
@@ -403,7 +532,7 @@ void Track::drawScaled2D(float x, float y, float w, float h) const
         sy = sx;
     }
 
-    const unsigned int DRIVELINE_SIZE = m_driveline.size();
+    const unsigned int DRIVELINE_SIZE = (unsigned int)m_driveline.size();
 
     glPushAttrib ( GL_ENABLE_BIT | GL_COLOR_BUFFER_BIT | GL_LINE_BIT );
 
@@ -517,7 +646,7 @@ void Track::drawScaled2D(float x, float y, float w, float h) const
 void Track::draw2Dview (float x_offset, float y_offset) const
 {
 
-    const unsigned int DRIVELINE_SIZE = m_driveline.size();
+    const unsigned int DRIVELINE_SIZE = (unsigned int)m_driveline.size();
 
     glPushAttrib ( GL_ENABLE_BIT | GL_COLOR_BUFFER_BIT | GL_LINE_BIT );
 
@@ -648,17 +777,19 @@ void Track::loadTrack(std::string filename_)
     // Default values
     m_use_fog = false;
     sgSetVec4 ( m_fog_color  , 0.3f, 0.7f, 0.9f, 1.0f ) ;
-    m_fog_density = 1.0f/100.0f;
-    m_fog_start   = 0.0f;
-    m_fog_end     = 1000.0f;
-    m_gravity     = 9.80665f;
+    m_fog_density               = 1.0f/100.0f;
+    m_fog_start                 = 0.0f;
+    m_fog_end                   = 1000.0f;
+    m_gravity                   = 9.80665f;
+    m_AI_angle_adjustment       = 1.0f;
+    m_AI_curve_speed_adjustment = 1.0f;
 
-    sgSetVec3 ( m_sun_position, 0.4f, 0.4f, 0.4f );
-    sgSetVec4 ( m_sky_color  ,  0.3f, 0.7f, 0.9f, 1.0f );
-    sgSetVec4 ( m_fog_color  ,  0.3f, 0.7f, 0.9f, 1.0f );
-    sgSetVec4 ( m_ambient_col ,  0.5f, 0.5f, 0.5f, 1.0f );
+    sgSetVec3 ( m_sun_position,  0.4f, 0.4f, 0.4f      );
+    sgSetVec4 ( m_sky_color,     0.3f, 0.7f, 0.9f, 1.0f );
+    sgSetVec4 ( m_fog_color,     0.3f, 0.7f, 0.9f, 1.0f );
+    sgSetVec4 ( m_ambient_col,   0.5f, 0.5f, 0.5f, 1.0f );
     sgSetVec4 ( m_specular_col,  1.0f, 1.0f, 1.0f, 1.0f );
-    sgSetVec4 ( m_diffuse_col ,  1.0f, 1.0f, 1.0f, 1.0f );
+    sgSetVec4 ( m_diffuse_col,   1.0f, 1.0f, 1.0f, 1.0f );
 
     lisp::Parser parser;
     const lisp::Lisp* const ROOT = parser.parse(loader->getPath(m_filename));
@@ -674,35 +805,45 @@ void Track::loadTrack(std::string filename_)
         throw std::runtime_error(msg);
     }
 
-    LISP->get("name",          m_name);
-    LISP->get("description",   m_description);
-    LISP->get("music",         m_music_filename);
-    LISP->get("herring",       m_herring_style);
-    LISP->get("screenshot",    m_screenshot);
-    LISP->get("topview",       m_top_view);
-    LISP->get("sky-color",     m_sky_color);
+    LISP->get      ("name",                  m_name);
+    LISP->get      ("description",           m_description);
+    LISP->getVector("music",                 m_music_filenames);
+    LISP->get      ("herring",               m_herring_style);
+    LISP->get      ("screenshot",            m_screenshot);
+    LISP->get      ("topview",               m_top_view);
+    LISP->get      ("sky-color",             m_sky_color);
+    LISP->getVector("start-x",               m_start_x);
+    LISP->getVector("start-y",               m_start_y);
+    LISP->getVector("start-z",               m_start_z);
+    LISP->getVector("start-heading",         m_start_heading);
+    LISP->get      ("use-fog",               m_use_fog);
+    LISP->get      ("fog-color",             m_fog_color);
+    LISP->get      ("fog-density",           m_fog_density);
+    LISP->get      ("fog-start",             m_fog_start);
+    LISP->get      ("fog-end",               m_fog_end);
+    LISP->get      ("sun-position",          m_sun_position);
+    LISP->get      ("sun-ambient",           m_ambient_col);
+    LISP->get      ("sun-specular",          m_specular_col);
+    LISP->get      ("sun-diffuse",           m_diffuse_col);
+    LISP->get      ("gravity",               m_gravity);
+    LISP->get      ("AI-angle-adjust",       m_AI_angle_adjustment);
+    LISP->get      ("AI-curve-speed-adjust", m_AI_curve_speed_adjustment);
 
-    LISP->get("use-fog",       m_use_fog);
-    LISP->get("fog-color",     m_fog_color);
-    LISP->get("fog-density",   m_fog_density);
-    LISP->get("fog-start",     m_fog_start);
-    LISP->get("fog-end",       m_fog_end);
-
-    LISP->get("sun-position",  m_sun_position);
-    LISP->get("sun-ambient",   m_ambient_col);
-    LISP->get("sun-specular",  m_specular_col);
-    LISP->get("sun-diffuse",   m_diffuse_col);
-    LISP->get("m_gravity",     m_gravity);
     delete ROOT;
 }
 
+//-----------------------------------------------------------------------------
+const std::string& Track::getMusic() const {
+    return m_music_filenames[rand()% m_music_filenames.size()];
+}   // getMusic
+
 //-----------------------------------------------------------------------------
 void
 Track::loadDriveline()
 {
     readDrivelineFromFile(m_left_driveline, ".drvl");
 
-    const unsigned int DRIVELINE_SIZE = m_left_driveline.size();
+    const unsigned int DRIVELINE_SIZE = (unsigned int)m_left_driveline.size();
     m_right_driveline.reserve(DRIVELINE_SIZE);
     readDrivelineFromFile(m_right_driveline, ".drvr");
 
@@ -753,14 +894,27 @@ Track::loadDriveline()
     float d = 0.0f ;
     for ( size_t i = 0 ; i < DRIVELINE_SIZE ; ++i )
     {
-        if ( m_driveline[i][0] < m_driveline_min[0] )
-            m_driveline_min[0] = m_driveline[i][0] ;
-        if ( m_driveline[i][1] < m_driveline_min[1] )
-            m_driveline_min[1] = m_driveline[i][1] ;
-        if ( m_driveline[i][0] > m_driveline_max[0] )
-            m_driveline_max[0] = m_driveline[i][0] ;
-        if ( m_driveline[i][1] > m_driveline_max[1] )
-            m_driveline_max[1] = m_driveline[i][1] ;
+        //Both drivelines must be checked to get the true size of
+        //the drivelines, and using the center driveline is not
+        //good enough.
+        if ( m_right_driveline[i][0] < m_driveline_min[0] )
+            m_driveline_min[0] = m_right_driveline[i][0] ;
+        if ( m_right_driveline[i][1] < m_driveline_min[1] )
+            m_driveline_min[1] = m_right_driveline[i][1] ;
+        if ( m_right_driveline[i][0] > m_driveline_max[0] )
+            m_driveline_max[0] = m_right_driveline[i][0] ;
+        if ( m_right_driveline[i][1] > m_driveline_max[1] )
+            m_driveline_max[1] = m_right_driveline[i][1] ;
+
+        if ( m_left_driveline[i][0] < m_driveline_min[0] )
+            m_driveline_min[0] = m_left_driveline[i][0] ;
+        if ( m_left_driveline[i][1] < m_driveline_min[1] )
+            m_driveline_min[1] = m_left_driveline[i][1] ;
+        if ( m_left_driveline[i][0] > m_driveline_max[0] )
+            m_driveline_max[0] = m_left_driveline[i][0] ;
+        if ( m_left_driveline[i][1] > m_driveline_max[1] )
+            m_driveline_max[1] = m_left_driveline[i][1] ;
+
 
         m_distance_from_start.push_back(d);  // dfs[i] is not valid in windows here!
 
@@ -787,7 +941,7 @@ void
 Track::readDrivelineFromFile(std::vector<sgVec3Wrapper>& line, const std::string& file_ext)
 {
     std::string path = "data";
-    path += DIR_SEPARATOR;
+    path += Loader::DIR_SEPARATOR;
     path += m_ident;
     path += file_ext;
     path = loader->getPath(path.c_str());
@@ -857,5 +1011,314 @@ Track::readDrivelineFromFile(std::vector<sgVec3Wrapper>& line, const std::string
     }
 
     fclose ( fd ) ;
-}
+}   // readDrivelineFromFile
 
+// -----------------------------------------------------------------------------
+//* Convert the ssg track tree into its physics equivalents.
+void Track::createPhysicsModel()
+{
+    if(!m_model) return;
+
+    m_track_mesh         = new TriangleMesh();
+    m_non_collision_mesh = new TriangleMesh();
+    
+    // Collect all triangles in the track_mesh
+    sgMat4 mat;
+    sgMakeIdentMat4(mat);
+    convertTrackToBullet(m_model, mat);
+    m_track_mesh->createBody();
+    m_non_collision_mesh->createBody(btCollisionObject::CF_NO_CONTACT_RESPONSE);
+    
+}   // createPhysicsModel
+
+// -----------------------------------------------------------------------------
+//* Convert the ssg track tree into its physics equivalents.
+void Track::convertTrackToBullet(ssgEntity *track, sgMat4 m)
+{
+    if(!track) return;
+    MovingPhysics *mp = dynamic_cast<MovingPhysics*>(track);
+    if(mp)
+    {
+        // If the track contains obect of type MovingPhysics,
+        // these objects will be real rigid body and are already
+        // part of the world. So these objects must not be converted
+        // to triangle meshes.
+    } 
+    else if(track->isAKindOf(ssgTypeLeaf()))
+    {
+        ssgLeaf  *leaf     = (ssgLeaf*)(track);
+        Material *material = material_manager->getMaterial(leaf);
+        // Don't convert triangles with material that is ignored (e.g. fuzzy_sand)
+        if(!material || material->isIgnore()) return;
+
+        for(int i=0; i<leaf->getNumTriangles(); i++) 
+        {
+            short v1,v2,v3;
+            sgVec3 vv1, vv2, vv3;
+            
+            leaf->getTriangle(i, &v1, &v2, &v3);
+            sgXformPnt3 ( vv1, leaf->getVertex(v1), m );
+            sgXformPnt3 ( vv2, leaf->getVertex(v2), m );
+            sgXformPnt3 ( vv3, leaf->getVertex(v3), m );
+            btVector3 vb1(vv1[0],vv1[1],vv1[2]);
+            btVector3 vb2(vv2[0],vv2[1],vv2[2]);
+            btVector3 vb3(vv3[0],vv3[1],vv3[2]);
+            if(material->isZipper()) 
+            {
+                m_non_collision_mesh->addTriangle(vb1, vb2, vb3, material);
+            }
+            else
+            {
+                m_track_mesh->addTriangle(vb1, vb2, vb3, material);
+            }
+        }
+        
+    }   // if(track isAKindOf leaf)
+    else if(track->isAKindOf(ssgTypeTransform()))
+    {
+        ssgBaseTransform *t = (ssgBaseTransform*)(track);
+        sgMat4 tmpT, tmpM;
+        t->getTransform(tmpT);
+        sgCopyMat4(tmpM, m);
+        sgPreMultMat4(tmpM,tmpT);
+        for(ssgEntity *e = t->getKid(0); e!=NULL; e=t->getNextKid())
+        {
+            convertTrackToBullet(e, tmpM);
+        }   // for i
+    }
+    else if (track->isAKindOf(ssgTypeBranch())) 
+    {
+        ssgBranch *b =(ssgBranch*)track;
+        for(ssgEntity* e=b->getKid(0); e!=NULL; e=b->getNextKid()) {
+            convertTrackToBullet(e, m);
+        }   // for i<getNumKids
+    }
+    else
+    {
+        assert(!"Unkown ssg type in convertTrackToBullet");
+    }
+}   // convertTrackToBullet
+
+// ----------------------------------------------------------------------------
+void Track::loadTrackModel()
+{
+    std::string path = "data/";
+    path += getIdent();
+    path += ".loc";
+    path = loader->getPath(path.c_str());
+
+    FILE *fd = fopen (path.c_str(), "r" );
+    if ( fd == NULL )
+    {
+        char msg[MAX_ERROR_MESSAGE_LENGTH];
+        snprintf(msg, sizeof(msg),"Can't open track location file '%s'.\n",
+                 path.c_str());
+        throw std::runtime_error(msg);
+    }
+
+    // Start building the scene graph
+    m_model = new ssgBranch ;
+    scene->add(m_model);
+
+    char s [ 1024 ] ;
+
+    while ( fgets ( s, 1023, fd ) != NULL )
+    {
+        if ( *s == '#' || *s < ' ' )
+            continue ;
+
+        int need_hat = false ;
+        int fit_skin = false ;
+        char fname [ 1024 ] ;
+        sgCoord loc ;
+        sgZeroVec3 ( loc.xyz ) ;
+        sgZeroVec3 ( loc.hpr ) ;
+
+        char htype = '\0' ;
+
+        if ( sscanf ( s, "%cHERRING,%f,%f,%f", &htype,
+                      &(loc.xyz[0]), &(loc.xyz[1]), &(loc.xyz[2]) ) == 4 )
+        {
+            herring_command(&loc.xyz, htype, false) ;
+        }
+        else if ( sscanf ( s, "%cHERRING,%f,%f", &htype,
+                           &(loc.xyz[0]), &(loc.xyz[1]) ) == 3 )
+        {
+            herring_command (&loc.xyz, htype, true) ;
+        }
+        else if ( s[0] == '\"' )
+        {
+            if ( sscanf ( s, "\"%[^\"]\",%f,%f,%f,%f,%f,%f",
+                          fname, &(loc.xyz[0]), &(loc.xyz[1]), &(loc.xyz[2]),
+                          &(loc.hpr[0]), &(loc.hpr[1]), &(loc.hpr[2]) ) == 7 )
+            {
+                /* All 6 DOF specified */
+                need_hat = false;
+            }
+            else if ( sscanf ( s, "\"%[^\"]\",%f,%f,{},%f,%f,%f",
+                               fname, &(loc.xyz[0]), &(loc.xyz[1]),
+                               &(loc.hpr[0]), &(loc.hpr[1]), &(loc.hpr[2])) == 6 )
+            {
+                /* All 6 DOF specified - but need height */
+                need_hat = true ;
+            }
+            else if ( sscanf ( s, "\"%[^\"]\",%f,%f,%f,%f",
+                               fname, &(loc.xyz[0]), &(loc.xyz[1]), &(loc.xyz[2]),
+                               &(loc.hpr[0]) ) == 5 )
+            {
+                /* No Roll/Pitch specified - assumed zero */
+                need_hat = false ;
+            }
+            else if ( sscanf ( s, "\"%[^\"]\",%f,%f,{},%f,{},{}",
+                               fname, &(loc.xyz[0]), &(loc.xyz[1]),
+                               &(loc.hpr[0]) ) == 4 )
+            {
+                /* All 6 DOF specified - but need height, roll, pitch */
+                need_hat = true ;
+                fit_skin = true ;
+            }
+            else if ( sscanf ( s, "\"%[^\"]\",%f,%f,{},%f",
+                               fname, &(loc.xyz[0]), &(loc.xyz[1]),
+                               &(loc.hpr[0]) ) == 4 )
+            {
+                /* No Roll/Pitch specified - but need height */
+                need_hat = true ;
+            }
+            else if ( sscanf ( s, "\"%[^\"]\",%f,%f,%f",
+                               fname, &(loc.xyz[0]), &(loc.xyz[1]),
+                               &(loc.xyz[2]) ) == 4 )
+            {
+                /* No Heading/Roll/Pitch specified - but need height */
+                need_hat = false ;
+            }
+            else if ( sscanf ( s, "\"%[^\"]\",%f,%f,{}",
+                               fname, &(loc.xyz[0]), &(loc.xyz[1]) ) == 3 )
+            {
+                /* No Roll/Pitch specified - but need height */
+                need_hat = true ;
+            }
+            else if ( sscanf ( s, "\"%[^\"]\",%f,%f",
+                               fname, &(loc.xyz[0]), &(loc.xyz[1]) ) == 3 )
+            {
+                /* No Z/Heading/Roll/Pitch specified */
+                need_hat = false ;
+            }
+            else if ( sscanf ( s, "\"%[^\"]\"", fname ) == 1 )
+            {
+                /* Nothing specified */
+                need_hat = false ;
+            }
+            else
+            {
+                fclose(fd);
+                char msg[MAX_ERROR_MESSAGE_LENGTH];
+                snprintf(msg, sizeof(msg), "Syntax error in '%s': %s",
+                         path.c_str(), s);
+                throw std::runtime_error(msg);
+            }
+
+            if ( need_hat )
+            {
+                sgVec3 nrm ;
+
+                loc.xyz[2] = 1000.0f ;
+                loc.xyz[2] = getHeightAndNormal ( m_model, loc.xyz, nrm ) ;
+
+                if ( fit_skin )
+                {
+                    float sy = sin ( -loc.hpr [ 0 ] * SG_DEGREES_TO_RADIANS ) ;
+                    float cy = cos ( -loc.hpr [ 0 ] * SG_DEGREES_TO_RADIANS ) ;
+
+                    loc.hpr[2] =  SG_RADIANS_TO_DEGREES * atan2 ( nrm[0] * cy -
+                                  nrm[1] * sy, nrm[2] ) ;
+                    loc.hpr[1] = -SG_RADIANS_TO_DEGREES * atan2 ( nrm[1] * cy +
+                                 nrm[0] * sy, nrm[2] ) ;
+                }
+            }   // if need_hat
+
+            ssgEntity        *obj   = loader->load(fname, CB_TRACK);
+            createDisplayLists(obj);
+            ssgRangeSelector *lod   = new ssgRangeSelector ;
+            ssgTransform     *trans = new ssgTransform ( & loc ) ;
+
+            float r [ 2 ] = { -10.0f, 2000.0f } ;
+
+            lod    -> addKid(obj   );
+            trans  -> addKid(lod   );
+            m_model-> addKid(trans );
+            lod    -> setRanges(r, 2);
+            if(user_config->m_track_debug)
+                addDebugToScene(user_config->m_track_debug);
+
+        }
+        else
+        {
+//            fclose(fd);
+//            char msg[MAX_ERROR_MESSAGE_LENGTH];
+//            snprintf(msg, sizeof(msg), "Syntax error in '%s': %s",
+            fprintf(stderr, "Warning: Syntax error in '%s': %s",
+                     path.c_str(), s);
+//            throw std::runtime_error(msg);
+        }
+    }   // while fgets
+
+    fclose ( fd ) ;
+
+    createPhysicsModel();
+}   // loadTrack
+
+//-----------------------------------------------------------------------------
+void Track::herring_command (sgVec3 *xyz, char htype, int bNeedHeight )
+{
+
+    // if only 2d coordinates are given, let the herring fall from very heigh
+    if(bNeedHeight) (*xyz)[2] = 1000000.0f;
+
+    // Even if 3d data are given, make sure that the herring is on the ground
+    (*xyz)[2] = getHeight ( m_model, *xyz ) + 0.06f;
+    herringType type=HE_GREEN;
+    if ( htype=='Y' || htype=='y' ) { type = HE_GOLD   ;}
+    if ( htype=='G' || htype=='g' ) { type = HE_GREEN  ;}
+    if ( htype=='R' || htype=='r' ) { type = HE_RED    ;}
+    if ( htype=='S' || htype=='s' ) { type = HE_SILVER ;}
+    herring_manager->newHerring(type, xyz);
+}   // herring_command
+
+// ----------------------------------------------------------------------------
+void  Track::getTerrainInfo(btVector3 &pos, float *hot, btVector3* normal, 
+                            const Material **material) const
+{
+    btVector3 to_pos(pos);
+    to_pos.setZ(-100000.f);
+
+    class MaterialCollision : public btCollisionWorld::ClosestRayResultCallback
+    {
+    public:
+        const Material* m_material;
+        MaterialCollision(btVector3 p1, btVector3 p2) : 
+            btCollisionWorld::ClosestRayResultCallback(p1,p2) {m_material=NULL;}
+        virtual btScalar AddSingleResult(btCollisionWorld::LocalRayResult& rayResult,
+                                         bool normalInWorldSpace) {
+             if(rayResult.m_localShapeInfo && rayResult.m_localShapeInfo->m_shapePart>=0 )
+             {
+                 m_material = ((TriangleMesh*)rayResult.m_collisionObject->getUserPointer())->getMaterial(rayResult.m_localShapeInfo->m_triangleIndex);
+             }
+             return btCollisionWorld::ClosestRayResultCallback::AddSingleResult(rayResult, 
+                                                                                normalInWorldSpace);
+        }   // AddSingleResult
+    };   // myCollision
+    MaterialCollision rayCallback(pos, to_pos);
+    world->getPhysics()->getPhysicsWorld()->rayTest(pos, to_pos, rayCallback);
+
+    if(!rayCallback.HasHit()) 
+    {
+        *hot      = NOHIT;
+        *material = NULL;
+        return;
+    }
+
+    *hot      = rayCallback.m_hitPointWorld.getZ();
+    *normal   = rayCallback.m_hitNormalWorld;
+    *material = rayCallback.m_material;
+    // FIXME: material must be set!
+}   // getTerrainInfo
