@@ -1,4 +1,4 @@
-//  $Id: track.hpp 1030 2007-05-03 03:31:24Z hiker $
+//  $Id: track.hpp 1513 2008-02-18 04:16:57Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>
@@ -23,35 +23,50 @@
 #ifdef __APPLE__
 #  include <OpenGL/gl.h>
 #else
+#  ifdef WIN32
+#    define WIN32_LEAN_AND_MEAN
+#    include <windows.h>
+#  endif
 #  include <GL/gl.h>
 #endif
 #include <plib/sg.h>
+#include <plib/ssg.h>
 #include <string>
 #include <vector>
-
+#include "btBulletDynamicsCommon.h"
+#include "material.hpp"
+#include "triangle_mesh.hpp"
 
 class Track
 {
 private:
-    float       m_gravity;
-    std::string m_ident;
-    std::string m_screenshot;
-    std::string m_top_view;
-    std::string m_music_filename;
-    std::string m_herring_style;
-    std::string m_description;
-    std::string m_filename;
-
+    float                    m_gravity;
+    std::string              m_ident;
+    std::string              m_screenshot;
+    std::string              m_top_view;
+    std::vector<std::string> m_music_filenames;
+    std::vector<float>       m_start_x, m_start_y, m_start_z, m_start_heading;
+    std::string              m_herring_style;
+    std::string              m_description;
+    std::string              m_filename;
+    ssgBranch*               m_model;
+    TriangleMesh*            m_track_mesh;
+    TriangleMesh*            m_non_collision_mesh;
+    // The next two variables are for AI improvements: the AI sometimes does
+    // not estimate curve speed and/or angle correctly, resulting in too much
+    // braking. These factors are used to adjust this.
+    float                    m_AI_angle_adjustment;
+    float                    m_AI_curve_speed_adjustment;
 public:
     enum RoadSide{ RS_DONT_KNOW = -1, RS_LEFT = 0, RS_RIGHT = 1 };
 
     //An enum is not used for the QUAD_TRI_* constants because of limitations
     //of the conversion between enums and ints.
-    static const int QUAD_TRI_NONE = -1;
-    static const int QUAD_TRI_FIRST = 1;
-    static const int QUAD_TRI_SECOND = 2;
+    static const int QUAD_TRI_NONE;
+    static const int QUAD_TRI_FIRST;
+    static const int QUAD_TRI_SECOND;
 
-    static const int UNKNOWN_SECTOR = -1;
+    static const int UNKNOWN_SECTOR;
 
     struct SegmentTriangle
     {
@@ -119,6 +134,7 @@ public:
 
 
     float m_total_distance;
+    static const float NOHIT;
 
     float m_track_2d_width,  // Width and heigth of the 2d display of the track
           m_track_2d_height;
@@ -129,13 +145,14 @@ public:
     Track            (std::string filename,float w=100,
                       float h=100, bool stretch=1);
     ~Track            ();
-
+    void               cleanup          ();
+    void               addDebugToScene  (int type                    ) const;
     void               draw2Dview       (float x_offset,
                                          float y_offset              ) const;
     void               drawScaled2D     (float x, float y, float w,
                                          float h                     ) const;
 
-    int                findRoadSector   (const sgVec3 XYZ            ) const;
+    void               findRoadSector   (const sgVec3 XYZ, int *sector) const;
     int                findOutOfRoadSector(const sgVec3 XYZ,
                                            const RoadSide SIDE,
                                            const int CURR_SECTOR
@@ -144,29 +161,37 @@ public:
                                          const sgVec2 POS,
                                          const int SECTOR            ) const;
     void               trackToSpatial   (sgVec3 xyz, const int SECTOR) const;
+    ssgBranch*         getModel         () const {return m_model;}
+    void               loadTrackModel   ();
     bool               isShortcut       (const int OLDSEC, const int NEWSEC) const;
-    float              getGravity       () const {return m_gravity;       }
-    float              getTrackLength   () const {return m_total_distance;}
-    const char*        getIdent         () const {return m_ident.c_str(); }
-    const char*        getName          () const {return m_name.c_str();  }
-    const std::string& getMusic         () const {return m_music_filename;}
+    float              getGravity       () const {return m_gravity;        }
+    float              getTrackLength   () const {return m_total_distance; }
+    const char*        getIdent         () const {return m_ident.c_str();  }
+    const char*        getName          () const {return m_name.c_str();   }
+    const std::string& getMusic         () const;
     const std::string& getFilename      () const {return m_filename; }
-    const sgVec3& getSunPos             () const {return m_sun_position;  }
+    const sgVec3& getSunPos             () const {return m_sun_position;   }
     const sgVec4& getAmbientCol         () const {return m_ambient_col;    }
     const sgVec4& getDiffuseCol         () const {return m_diffuse_col;    }
     const sgVec4& getSpecularCol        () const {return m_specular_col;   }
-    const bool&   useFog                () const {return m_use_fog;       }
-    const sgVec4& getFogColor           () const {return m_fog_color;     }
-    const float&  getFogDensity         () const {return m_fog_density;   }
-    const float&  getFogStart           () const {return m_fog_start;     }
-    const float&  getFogEnd             () const {return m_fog_end;       }
-    const sgVec4& getSkyColor           () const {return m_sky_color;     }
-    const std::string& getDescription   () const {return m_description;   }
+    const bool&   useFog                () const {return m_use_fog;        }
+    const sgVec4& getFogColor           () const {return m_fog_color;      }
+    const float&  getFogDensity         () const {return m_fog_density;    }
+    const float&  getFogStart           () const {return m_fog_start;      }
+    const float&  getFogEnd             () const {return m_fog_end;        }
+    const float&  getAIAngleAdjustment  () const {return m_AI_angle_adjustment;}
+    const float&  getAICurveSpeedAdjustment() const {return m_AI_curve_speed_adjustment;}
+    const sgVec4& getSkyColor           () const {return m_sky_color;      }
+    const std::string& getDescription   () const {return m_description;    }
     const std::string& getTopviewFile   () const {return m_top_view;       }
-    const std::string& getScreenshotFile() const {return m_screenshot;    }
+    const std::string& getScreenshotFile() const {return m_screenshot;     }
     const std::vector<sgVec3Wrapper>& getDriveline () const {return m_driveline;}
-    const std::vector<SGfloat>& getWidth() const {return m_path_width;    }
+    const std::vector<SGfloat>& getWidth() const {return m_path_width;     }
     const std::string& getHerringStyle  () const {return m_herring_style;  }
+    void               getStartCoords   (unsigned int pos, sgCoord* coords) const;
+    void  getTerrainInfo(btVector3 &pos, float *hot, btVector3* normal, 
+                         const Material **material) const;
+    void createPhysicsModel             ();
     void               glVtx            (sgVec2 v, float x_offset, float y_offset) const
     {
         glVertex2f(
@@ -175,15 +200,17 @@ public:
     }
 
 private:
-    void loadTrack                      (std::string filename);
-    void loadDriveline                  ();
-    void readDrivelineFromFile          (std::vector<sgVec3Wrapper>& line,
+    void  loadTrack                      (std::string filename);
+    void  herring_command                (sgVec3 *xyz, char htype, int bNeedHeight);
+    void  loadDriveline                  ();
+    void  readDrivelineFromFile          (std::vector<sgVec3Wrapper>& line,
                                          const std::string& file_ext      );
+    void  convertTrackToBullet           (ssgEntity *track, sgMat4 m);
 
-    float pointSideToLine( const sgVec2 L1, const sgVec2 L2,
-        const sgVec2 P ) const;
-    int pointInQuad( const sgVec2 A, const sgVec2 B,
-        const sgVec2 C, const sgVec2 D, const sgVec2 POINT ) const;
+    float pointSideToLine(const sgVec2 L1, const sgVec2 L2,
+                          const sgVec2 P ) const;
+    int   pointInQuad(const sgVec2 A, const sgVec2 B,
+                      const sgVec2 C, const sgVec2 D, const sgVec2 POINT ) const;
 }
 ;   // class Track
 
