@@ -1,4 +1,4 @@
-//  $Id: track_manager.cpp 918 2007-02-26 23:34:43Z hiker $
+//  $Id: track_manager.cpp 1302 2007-11-22 01:20:57Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -37,7 +37,7 @@ TrackManager::~TrackManager()
 }
 
 //-----------------------------------------------------------------------------
-const Track*
+Track*
 TrackManager::getTrack(const std::string& ident) const
 {
     for(Tracks::const_iterator i = m_tracks.begin(); i != m_tracks.end(); ++i)
@@ -52,7 +52,7 @@ TrackManager::getTrack(const std::string& ident) const
 }
 
 //-----------------------------------------------------------------------------
-const Track*
+Track*
 TrackManager::getTrack(size_t id) const
 {
     return m_tracks[id];
@@ -76,7 +76,7 @@ TrackManager::loadTrackList ()
         {
             if(StringUtils::has_suffix(*i, ".track"))
             {
-                std::string track_name= std::string("data")+DIR_SEPARATOR + *i;
+                std::string track_name= std::string("data")+Loader::DIR_SEPARATOR + *i;
                 m_tracks.push_back(new Track(track_name.c_str()));
             }
         }
