@@ -1,4 +1,4 @@
-//  $Id: track_manager.hpp 855 2006-11-17 01:50:37Z coz $
+//  $Id: track_manager.hpp 1261 2007-09-26 03:43:02Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2006 SuperTuxKart-Team
@@ -38,8 +38,8 @@ public:
 
     /** get TrackData by the track ident (aka filename without
         .track) */
-    const Track* getTrack(const std::string& ident) const;
-    const Track* getTrack(size_t id) const;
+    Track* getTrack(const std::string& ident) const;
+    Track* getTrack(size_t id) const;
 
     size_t getTrackCount() const;
 
