@@ -1,4 +1,4 @@
-//  $Id: traffic.cpp 855 2006-11-17 01:50:37Z coz $
+//  $Id: traffic.cpp 1373 2008-01-07 00:53:22Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 Steve Baker <sjbaker1@airmail.net>
@@ -19,6 +19,7 @@
 
 #include "kart.hpp"
 #include "constants.hpp"
+#include "world.hpp"
 
 inline float sgnsq ( float x ) { return ( x < 0 ) ? -(x * x) : (x * x) ; }
 
@@ -35,12 +36,12 @@ void TrafficDriver::update (float dt)
 
     if ( ( track_velocity [ 0 ] < 0.0f && m_curr_track_coords [ 0 ] > 0.0f ) ||
          ( track_velocity [ 0 ] > 0.0f && m_curr_track_coords [ 0 ] < 0.0f ) )
-        m_velocity.hpr[0] = sgnsq(m_curr_track_coords[0])*3.0f ;
+;//FIXME        m_velocity.hpr[0] = sgnsq(m_curr_track_coords[0])*3.0f ;
     else
-        m_velocity.hpr[0] = sgnsq(m_curr_track_coords[0])*12.0f ;
+//FIXME        m_velocity.hpr[0] = sgnsq(m_curr_track_coords[0])*12.0f ;
 
-    m_velocity.xyz[1]  = TRAFFIC_VELOCITY ;
-    m_velocity.xyz[2] -= GRAVITY * dt ;
+//FIXME    m_velocity.xyz[1]  = TRAFFIC_VELOCITY ;
+//FIXME    m_velocity.xyz[2] -= world->getGravity()* dt ;
 
     if ( m_wheelie_angle != 0.0f )
         m_wheelie_angle = 0.0f ;
@@ -48,8 +49,6 @@ void TrafficDriver::update (float dt)
     Kart::update (dt) ;
 }
 
-//-----------------------------------------------------------------------------
-void TrafficDriver::doObjectInteractions () {}
 //-----------------------------------------------------------------------------
 void TrafficDriver::doLapCounting        () {}
 //-----------------------------------------------------------------------------
