@@ -1,4 +1,4 @@
-//  $Id: world.hpp 943 2007-03-28 02:20:44Z hiker $
+//  $Id: world.hpp 1489 2008-02-14 02:24:54Z hikerstk $
 //
 //  SuperTuxKart - a fun racing game with go-kart
 //  Copyright (C) 2004 SuperTuxKart-Team
@@ -23,23 +23,22 @@
 #include <vector>
 #include <plib/ssg.h>
 #include "race_setup.hpp"
-#include "static_ssg.hpp"
 #include "track.hpp"
 #include "player_kart.hpp"
 #include "physics.hpp"
 #include "kart.hpp"
 #include "highscores.hpp"
-
+#ifdef HAVE_GHOST_REPLAY
+#  include "replay_recorder.hpp"
+   class ReplayPlayer;
+#endif
 
 /** This class keeps all the state of a race, scenegraph, time,
     etc. */
 class World
 {
 public:
-    ssgRoot      *m_scene;
-
     typedef std::vector<Kart*> Karts;
-    float      m_clock;
 
     /** resources, this should be put in a separate class or replaced by a smart
      * resource manager
@@ -47,9 +46,15 @@ public:
     RaceSetup m_race_setup;
 
     enum Phase {
-        // The traffic light is shown and all players stay in position
-        START_PHASE,
-        // The traffic light turned green and all driver, drive around the track
+        // Game setup, e.g. track loading
+        SETUP_PHASE,
+        // 'Ready' is displayed
+        READY_PHASE,
+        // 'Set' is displayed
+        SET_PHASE,
+        // 'Go' is displayed, but this is already race phase
+        GO_PHASE,
+        // the actual race has started, no ready/set/go is displayed anymore
         RACE_PHASE,
         // All players have finished, now wait a certain amount of time for AI
         // karts to finish. If they do not finish in that time, finish the race
@@ -61,27 +66,19 @@ public:
         LIMBO_PHASE,
     };
 
-    int m_ready_set_go;
-
-    const Track* m_track;
+    Track* m_track;
     
         /** debug text that will be overlaid to the screen */
     std::string m_debug_text[10];
 
-    /** Reference to the track inside the scene */
-    ssgBranch *m_track_branch ;
-
-    World(const RaceSetup& raceSetup);
-    virtual ~World();
-    float GetHOT(sgVec3 start, sgVec3 end, ssgLeaf** leaf, sgVec4** nrm)
-    {return m_static_ssg->hot(start, end, leaf, nrm);}
-    int   Collision(sgSphere* s, AllHits *a)
-    {return m_static_ssg->collision(s,a);       }
-
-    void draw();
-    void update(float delta);
-    void restartRace();
-    void disableRace(); // Put race into limbo phase
+                World(const RaceSetup& raceSetup);
+    virtual    ~World();
+    void        update(float delta);
+    // Note: GO_PHASE is both: start phase and race phase
+    bool        isStartPhase() const  {return m_phase<GO_PHASE;}
+    bool        isRacePhase() const   {return m_phase>=GO_PHASE && m_phase<LIMBO_PHASE;}
+    void        restartRace();
+    void        disableRace(); // Put race into limbo phase
 
     PlayerKart* getPlayerKart(int player)
     {
@@ -95,37 +92,48 @@ public:
                kartId < int(m_kart.size()));
         return m_kart[kartId];
     }
-    int  getNumKarts() const                  {return (int)m_kart.size();          }
-    void addToScene(ssgEntity *kid)           {m_scene->addKid(kid);               }
-    void removeFromScene(ssgEntity *kid)      {m_scene->removeKid(kid);            }
-    void addCollisions(int kartNumber, int n) {m_number_collisions[kartNumber]+=n; }
+    unsigned int  getNumKarts() const         {return (int)m_kart.size();          }
 
     /** Returns the phase of the game */
     Phase getPhase() const                    { return m_phase;                    }
     float getGravity() const                  { return m_track->getGravity();      }
     Physics *getPhysics() const               { return m_physics;                  }
+    Track *getTrack() const                   { return m_track;                    }
     Kart* getFastestKart() const              { return m_fastest_kart;             }
     float getFastestLapTime() const           { return m_fastest_lap;              }
     void  setFastestLap(Kart *k, float time)  {m_fastest_kart=k;m_fastest_lap=time;}
     const Highscores* getHighscores() const   { return m_highscores;               }
+    float getTime() const                     { return m_clock;                    }
+
+    void  pause();
+    void  unpause();
+
 private:
     Karts       m_kart;
-    StaticSSG*  m_static_ssg;
     float       m_finish_delay_start_time;
-    int*        m_number_collisions;
     Physics*    m_physics;
     float       m_fastest_lap;
     Kart*       m_fastest_kart;
     Highscores* m_highscores;
     Phase       m_phase;
+    float       m_clock;
 
     void updateRacePosition( int k );
     void loadTrack();
-    void herring_command(sgVec3* loc, char htype, int bNeedHeight);
-    void checkRaceStatus();
+    void updateRaceStatus(float dt);
     void resetAllKarts();
     Kart* loadRobot(const KartProperties *kart_properties, int position,
                  sgCoord init_pos);
+
+#ifdef HAVE_GHOST_REPLAY
+private:
+    bool    saveReplayHumanReadable( std::string const &filename ) const;
+    bool    loadReplayHumanReadable( std::string const &filename );
+
+    ReplayRecorder  m_replay_recorder;
+    ReplayPlayer    *m_p_replay_player;
+#endif
+
 };
 
 extern World* world;
